/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <string.h>
#include <iostream>
#include <fstream>

#include "DebugOutput.hpp"
#include "StdioFilebuf.hpp"

#include <JobProperties.hpp>

static ostream *vostreamErr_d = &cerr;

ostream& DebugOutput::
getErrorStream ()
{
   return *vostreamErr_d;
}

void DebugOutput::
setErrorStream (FILE *pFile)
{
   // @TBD store and clean up new filebuf
   vostreamErr_d = new ostream (new stdio_filebuf (pFile));
}

void DebugOutput::
applyAllDebugOutput (char *pszJobProperties)
{
#ifndef RETAIL
   if (  pszJobProperties
      && *pszJobProperties
      )
   {
      JobProperties jobProp (pszJobProperties);

      while (jobProp.hasKeyNameLeft ("debugoutput"))
      {
         setDebugOutput (jobProp.getCurrentValue ());

         jobProp.getNextKey ("debugoutput");
      }
   }
#endif
}

static bool vfOutputDeviceBlitter_d         = false;
static bool vfOutputDeviceDither_d          = false;
static bool vfOutputDeviceInstance_d        = false;
static bool vfOutputDevicePrintMode_d       = false;
static bool vfOutputDeviceTester_d          = false;
static bool vfOutputGplCompression_d        = false;
static bool vfOutputGplDitherInstance_d     = false;
static bool vfOutputOmni_d                  = false;
static bool vfOutputOmniInterface_d         = false;
static bool vfOutputOmniProxy_d             = false;
static bool vfOutputPrintDevice_d           = false;
static bool vfOutputBlitter_d               = false;
static bool vfOutputInstance_d              = false;
static bool vfOutputOmniPDCProxy_d          = false;
static bool vfOutputOmniServer_d            = false;
static bool vfOutputPrinterCommand_d        = false;
static bool vfOutputPluggableInstance_d     = false;
static bool vfOutputPluggableBlitter_d      = false;
static bool vfOutputDriverInfo_d            = false;
static bool vfOutputPDCBlitter_d            = false;
static bool vfOutputPDCBlitterClient_d      = false;
static bool vfOutputDeviceString_d          = false;
static bool vfOutputUPDFDevice_d            = false;
static bool vfOutputUPDFDeviceOrientation_d = false;
static bool vfOutputUPDFDeviceForm_d        = false;
static bool vfOutputUPDFDeviceTray_d        = false;
static bool vfOutputUPDFDeviceMedia_d       = false;
static bool vfOutputUPDFDevicePrintMode_d   = false;
static bool vfOutputUPDFDeviceResolution_d  = false;
static bool vfOutputUPDFDeviceBooklet_d     = false;
static bool vfOutputUPDFDeviceCollation_d   = false;
static bool vfOutputUPDFDeviceCopies_d      = false;
static bool vfOutputUPDFDeviceDestination_d = false;
static bool vfOutputUPDFDeviceDuplex_d      = false;
static bool vfOutputUPDFDeviceJogging_d     = false;
static bool vfOutputUPDFDeviceNUp_d         = false;
static bool vfOutputUPDFDeviceScaling_d     = false;
static bool vfOutputUPDFDeviceStapling_d    = false;
static bool vfOutputUPDFDeviceInstance_d    = false;
static bool vfOutputUPDFDeviceBlitter_d     = false;
static bool vfOutputXMLDevice_d             = false;
static bool vfOutputXMLDeviceOrientation_d  = false;
static bool vfOutputXMLDeviceForm_d         = false;
static bool vfOutputXMLDeviceTray_d         = false;
static bool vfOutputXMLDeviceMedia_d        = false;
static bool vfOutputXMLDevicePrintMode_d    = false;
static bool vfOutputXMLDeviceResolution_d   = false;
static bool vfOutputXMLDeviceBooklet_d      = false;
static bool vfOutputXMLDeviceCollation_d    = false;
static bool vfOutputXMLDeviceCopies_d       = false;
static bool vfOutputXMLDeviceDestination_d  = false;
static bool vfOutputXMLDeviceDuplex_d       = false;
static bool vfOutputXMLDeviceJogging_d      = false;
static bool vfOutputXMLDeviceNUp_d          = false;
static bool vfOutputXMLDeviceScaling_d      = false;
static bool vfOutputXMLDeviceStapling_d     = false;
static bool vfOutputXMLDeviceInstance_d     = false;
static bool vfOutputXMLDeviceBlitter_d      = false;

void DebugOutput::
setDebugOutput (char *pszValue)
{
   if (0 == strcasecmp ("DeviceBlitter", pszValue))
   {
      setOutputDeviceBlitter (true);
   }
   else if (0 == strcasecmp ("DeviceDither", pszValue))
   {
      setOutputDeviceDither (true);
   }
   else if (0 == strcasecmp ("DeviceInstance", pszValue))
   {
      setOutputDeviceInstance (true);
   }
   else if (0 == strcasecmp ("DevicePrintMode", pszValue))
   {
      setOutputDevicePrintMode (true);
   }
   else if (0 == strcasecmp ("DeviceTester", pszValue))
   {
      setOutputDeviceTester (true);
   }
   else if (0 == strcasecmp ("GplCompression", pszValue))
   {
      setOutputGplCompression (true);
   }
   else if (0 == strcasecmp ("GplDitherInstance", pszValue))
   {
      setOutputGplDitherInstance (true);
   }
   else if (0 == strcasecmp ("Omni", pszValue))
   {
      setOutputOmni (true);
   }
   else if (0 == strcasecmp ("OmniInterface", pszValue))
   {
      setOutputOmniInterface (true);
   }
   else if (0 == strcasecmp ("OmniProxy", pszValue))
   {
      setOutputOmniProxy (true);
   }
   else if (0 == strcasecmp ("PrintDevice", pszValue))
   {
      setOutputPrintDevice (true);
   }
   else if (0 == strcasecmp ("Blitter", pszValue))
   {
      setOutputBlitter (true);
   }
   else if (0 == strcasecmp ("Instance", pszValue))
   {
      setOutputInstance (true);
   }
   else if (0 == strcasecmp ("OmniPDCProxy", pszValue))
   {
      setOutputOmniPDCProxy (true);
   }
   else if (0 == strcasecmp ("OmniServer", pszValue))
   {
      setOutputOmniServer (true);
   }
   else if (0 == strcasecmp ("PrinterCommand", pszValue))
   {
      setOutputPrinterCommand (true);
   }
   else if (0 == strcasecmp ("PluggableInstance", pszValue))
   {
      setOutputPluggableInstance (true);
   }
   else if (0 == strcasecmp ("PluggableBlitter", pszValue))
   {
      setOutputPluggableBlitter (true);
   }
   else if (0 == strcasecmp ("DriverInfo", pszValue))
   {
      setOutputDriverInfo (true);
   }
   else if (0 == strcasecmp ("PDCBlitter", pszValue))
   {
      setOutputPDCBlitter (true);
   }
   else if (0 == strcasecmp ("PDCBlitterClient", pszValue))
   {
      setOutputPDCBlitterClient (true);
   }
   else if (0 == strcasecmp ("DeviceString", pszValue))
   {
      setOutputDeviceString (true);
   }
   else if (0 == strcasecmp ("UPDFDevice", pszValue))
   {
      setOutputUPDFDevice (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceOrientation", pszValue))
   {
      setOutputUPDFDeviceOrientation (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceForm", pszValue))
   {
      setOutputUPDFDeviceForm (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceTray", pszValue))
   {
      setOutputUPDFDeviceTray (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceMedia", pszValue))
   {
      setOutputUPDFDeviceMedia (true);
   }
   else if (0 == strcasecmp ("UPDFDevicePrintMode", pszValue))
   {
      setOutputUPDFDevicePrintMode (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceResolution", pszValue))
   {
      setOutputUPDFDeviceResolution (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceBooklet", pszValue))
   {
      setOutputUPDFDeviceBooklet (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceCollation", pszValue))
   {
      setOutputUPDFDeviceCollation (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceCopies", pszValue))
   {
      setOutputUPDFDeviceCopies (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceDestination", pszValue))
   {
      setOutputUPDFDeviceDestination (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceDuplex", pszValue))
   {
      setOutputUPDFDeviceDuplex (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceJogging", pszValue))
   {
      setOutputUPDFDeviceJogging (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceNUp", pszValue))
   {
      setOutputUPDFDeviceNUp (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceScaling", pszValue))
   {
      setOutputUPDFDeviceScaling (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceStapling", pszValue))
   {
      setOutputUPDFDeviceStapling (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceInstance", pszValue))
   {
      setOutputUPDFDeviceInstance (true);
   }
   else if (0 == strcasecmp ("UPDFDeviceBlitter", pszValue))
   {
      setOutputUPDFDeviceBlitter (true);
   }
   else if (0 == strcasecmp ("XMLDevice", pszValue))
   {
      setOutputXMLDevice (true);
   }
   else if (0 == strcasecmp ("XMLDeviceOrientation", pszValue))
   {
      setOutputXMLDeviceOrientation (true);
   }
   else if (0 == strcasecmp ("XMLDeviceForm", pszValue))
   {
      setOutputXMLDeviceForm (true);
   }
   else if (0 == strcasecmp ("XMLDeviceTray", pszValue))
   {
      setOutputXMLDeviceTray (true);
   }
   else if (0 == strcasecmp ("XMLDeviceMedia", pszValue))
   {
      setOutputXMLDeviceMedia (true);
   }
   else if (0 == strcasecmp ("XMLDevicePrintMode", pszValue))
   {
      setOutputXMLDevicePrintMode (true);
   }
   else if (0 == strcasecmp ("XMLDeviceResolution", pszValue))
   {
      setOutputXMLDeviceResolution (true);
   }
   else if (0 == strcasecmp ("XMLDeviceBooklet", pszValue))
   {
      setOutputXMLDeviceBooklet (true);
   }
   else if (0 == strcasecmp ("XMLDeviceCollation", pszValue))
   {
      setOutputXMLDeviceCollation (true);
   }
   else if (0 == strcasecmp ("XMLDeviceCopies", pszValue))
   {
      setOutputXMLDeviceCopies (true);
   }
   else if (0 == strcasecmp ("XMLDeviceDestination", pszValue))
   {
      setOutputXMLDeviceDestination (true);
   }
   else if (0 == strcasecmp ("XMLDeviceDuplex", pszValue))
   {
      setOutputXMLDeviceDuplex (true);
   }
   else if (0 == strcasecmp ("XMLDeviceJogging", pszValue))
   {
      setOutputXMLDeviceJogging (true);
   }
   else if (0 == strcasecmp ("XMLDeviceNUp", pszValue))
   {
      setOutputXMLDeviceNUp (true);
   }
   else if (0 == strcasecmp ("XMLDeviceScaling", pszValue))
   {
      setOutputXMLDeviceScaling (true);
   }
   else if (0 == strcasecmp ("XMLDeviceStapling", pszValue))
   {
      setOutputXMLDeviceStapling (true);
   }
   else if (0 == strcasecmp ("XMLDeviceInstance", pszValue))
   {
      setOutputXMLDeviceInstance (true);
   }
   else if (0 == strcasecmp ("XMLDeviceBlitter", pszValue))
   {
      setOutputXMLDeviceBlitter (true);
   }
   else if (0 == strcasecmp ("all", pszValue))
   {
      setOutputDeviceBlitter         (true);
      setOutputDeviceDither          (true);
      setOutputDeviceInstance        (true);
      setOutputDevicePrintMode       (true);
      setOutputDeviceTester          (true);
      setOutputGplCompression        (true);
      setOutputGplDitherInstance     (true);
      setOutputOmni                  (true);
      setOutputOmniInterface         (true);
      setOutputOmniProxy             (true);
      setOutputPrintDevice           (true);
      setOutputBlitter               (true);
      setOutputInstance              (true);
      setOutputOmniPDCProxy          (true);
      setOutputOmniServer            (true);
      setOutputPrinterCommand        (true);
      setOutputPluggableInstance     (true);
      setOutputPluggableBlitter      (true);
      setOutputDriverInfo            (true);
      setOutputPDCBlitter            (true);
      setOutputPDCBlitterClient      (true);
      setOutputDeviceString          (true);
      setOutputUPDFDevice            (true);
      setOutputUPDFDeviceOrientation (true);
      setOutputUPDFDeviceForm        (true);
      setOutputUPDFDeviceTray        (true);
      setOutputUPDFDeviceMedia       (true);
      setOutputUPDFDevicePrintMode   (true);
      setOutputUPDFDeviceResolution  (true);
      setOutputUPDFDeviceInstance    (true);
      setOutputUPDFDeviceBlitter     (true);
      setOutputXMLDevice             (true);
      setOutputXMLDeviceOrientation  (true);
      setOutputXMLDeviceForm         (true);
      setOutputXMLDeviceTray         (true);
      setOutputXMLDeviceMedia        (true);
      setOutputXMLDevicePrintMode    (true);
      setOutputXMLDeviceResolution   (true);
      setOutputXMLDeviceBooklet      (true);
      setOutputXMLDeviceCollation    (true);
      setOutputXMLDeviceCopies       (true);
      setOutputXMLDeviceDestination  (true);
      setOutputXMLDeviceDuplex       (true);
      setOutputXMLDeviceJogging      (true);
      setOutputXMLDeviceNUp          (true);
      setOutputXMLDeviceScaling      (true);
      setOutputXMLDeviceStapling     (true);
      setOutputXMLDeviceInstance     (true);
      setOutputXMLDeviceBlitter      (true);
   }
   else if (0 == strcasecmp ("PDC", pszValue))
   {
      setOutputOmniPDCProxy     (true);
      setOutputOmniServer       (true);
      setOutputPrinterCommand   (true);
      setOutputPDCBlitter       (true);
      setOutputPDCBlitterClient (true);
   }
   else if (0 == strcasecmp ("Pluggable", pszValue))
   {
      setOutputPluggableInstance (true);
      setOutputPluggableBlitter  (true);
   }
   else if (0 == strcasecmp ("UPDF", pszValue))
   {
      setOutputUPDFDevice            (true);
      setOutputUPDFDeviceOrientation (true);
      setOutputUPDFDeviceForm        (true);
      setOutputUPDFDeviceTray        (true);
      setOutputUPDFDeviceMedia       (true);
      setOutputUPDFDevicePrintMode   (true);
      setOutputUPDFDeviceResolution  (true);
      setOutputUPDFDeviceBooklet     (true);
      setOutputUPDFDeviceCollation   (true);
      setOutputUPDFDeviceCopies      (true);
      setOutputUPDFDeviceDestination (true);
      setOutputUPDFDeviceDuplex      (true);
      setOutputUPDFDeviceJogging     (true);
      setOutputUPDFDeviceNUp         (true);
      setOutputUPDFDeviceScaling     (true);
      setOutputUPDFDeviceStapling    (true);
      setOutputUPDFDeviceInstance    (true);
      setOutputUPDFDeviceBlitter     (true);
   }
   else if (0 == strcasecmp ("XML", pszValue))
   {
      setOutputXMLDevice             (true);
      setOutputXMLDeviceOrientation  (true);
      setOutputXMLDeviceForm         (true);
      setOutputXMLDeviceTray         (true);
      setOutputXMLDeviceMedia        (true);
      setOutputXMLDevicePrintMode    (true);
      setOutputXMLDeviceResolution   (true);
      setOutputXMLDeviceBooklet      (true);
      setOutputXMLDeviceCollation    (true);
      setOutputXMLDeviceCopies       (true);
      setOutputXMLDeviceDestination  (true);
      setOutputXMLDeviceDuplex       (true);
      setOutputXMLDeviceJogging      (true);
      setOutputXMLDeviceNUp          (true);
      setOutputXMLDeviceScaling      (true);
      setOutputXMLDeviceStapling     (true);
      setOutputXMLDeviceInstance     (true);
      setOutputXMLDeviceBlitter      (true);
   }
}

bool DebugOutput::
shouldOutputDeviceBlitter ()
{
   return vfOutputDeviceBlitter_d;
}

bool DebugOutput::
shouldOutputDeviceDither ()
{
   return vfOutputDeviceDither_d;
}

bool DebugOutput::
shouldOutputDeviceInstance ()
{
   return vfOutputDeviceInstance_d;
}

bool DebugOutput::
shouldOutputDevicePrintMode ()
{
   return vfOutputDevicePrintMode_d;
}

bool DebugOutput::
shouldOutputDeviceTester ()
{
   return vfOutputDeviceTester_d;
}

bool DebugOutput::
shouldOutputGplCompression ()
{
   return vfOutputGplCompression_d;
}

bool DebugOutput::
shouldOutputGplDitherInstance ()
{
   return vfOutputGplDitherInstance_d;
}

bool DebugOutput::
shouldOutputOmni ()
{
   return vfOutputOmni_d;
}

bool DebugOutput::
shouldOutputOmniInterface ()
{
   return vfOutputOmniInterface_d;
}

bool DebugOutput::
shouldOutputOmniProxy ()
{
   return vfOutputOmniProxy_d;
}

bool DebugOutput::
shouldOutputPrintDevice ()
{
   return vfOutputPrintDevice_d;
}

bool DebugOutput::
shouldOutputBlitter ()
{
   return vfOutputBlitter_d;
}

bool DebugOutput::
shouldOutputInstance ()
{
   return vfOutputInstance_d;
}

bool DebugOutput::
shouldOutputOmniPDCProxy ()
{
   return vfOutputOmniPDCProxy_d;
}

bool DebugOutput::
shouldOutputOmniServer ()
{
   return vfOutputOmniServer_d;
}

bool DebugOutput::
shouldOutputPrinterCommand ()
{
   return vfOutputPrinterCommand_d;
}

bool DebugOutput::
shouldOutputPluggableInstance ()
{
   return vfOutputPluggableInstance_d;
}

bool DebugOutput::
shouldOutputPluggableBlitter ()
{
   return vfOutputPluggableBlitter_d;
}

bool DebugOutput::
shouldOutputDriverInfo ()
{
   return vfOutputDriverInfo_d;
}

bool DebugOutput::
shouldOutputPDCBlitter ()
{
   return vfOutputPDCBlitter_d;
}

bool DebugOutput::
shouldOutputPDCBlitterClient ()
{
   return vfOutputPDCBlitterClient_d;
}

bool DebugOutput::
shouldOutputDeviceString ()
{
   return vfOutputDeviceString_d;
}

bool DebugOutput::
shouldOutputUPDFDevice ()
{
   return vfOutputUPDFDevice_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceOrientation ()
{
   return vfOutputUPDFDeviceOrientation_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceForm ()
{
   return vfOutputUPDFDeviceForm_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceTray ()
{
   return vfOutputUPDFDeviceTray_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceMedia ()
{
   return vfOutputUPDFDeviceMedia_d;
}

bool DebugOutput::
shouldOutputUPDFDevicePrintMode ()
{
   return vfOutputUPDFDevicePrintMode_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceResolution ()
{
   return vfOutputUPDFDeviceResolution_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceBooklet ()
{
   return vfOutputUPDFDeviceBooklet_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceCollation ()
{
   return vfOutputUPDFDeviceCollation_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceCopies ()
{
   return vfOutputUPDFDeviceCopies_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceDestination ()
{
   return vfOutputUPDFDeviceDestination_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceDuplex ()
{
   return vfOutputUPDFDeviceDuplex_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceJogging ()
{
   return vfOutputUPDFDeviceJogging_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceNUp ()
{
   return vfOutputUPDFDeviceNUp_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceScaling ()
{
   return vfOutputUPDFDeviceScaling_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceStapling ()
{
   return vfOutputUPDFDeviceStapling_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceInstance ()
{
   return vfOutputUPDFDeviceInstance_d;
}

bool DebugOutput::
shouldOutputUPDFDeviceBlitter ()
{
   return vfOutputUPDFDeviceBlitter_d;
}

bool DebugOutput::
shouldOutputXMLDevice ()
{
   return vfOutputXMLDevice_d;
}

bool DebugOutput::
shouldOutputXMLDeviceOrientation ()
{
   return vfOutputXMLDeviceOrientation_d;
}

bool DebugOutput::
shouldOutputXMLDeviceForm ()
{
   return vfOutputXMLDeviceForm_d;
}

bool DebugOutput::
shouldOutputXMLDeviceTray ()
{
   return vfOutputXMLDeviceTray_d;
}

bool DebugOutput::
shouldOutputXMLDeviceMedia ()
{
   return vfOutputXMLDeviceMedia_d;
}

bool DebugOutput::
shouldOutputXMLDevicePrintMode ()
{
   return vfOutputXMLDevicePrintMode_d;
}

bool DebugOutput::
shouldOutputXMLDeviceResolution ()
{
   return vfOutputXMLDeviceResolution_d;
}

bool DebugOutput::
shouldOutputXMLDeviceBooklet ()
{
   return vfOutputXMLDeviceBooklet_d;
}

bool DebugOutput::
shouldOutputXMLDeviceCollation ()
{
   return vfOutputXMLDeviceCollation_d;
}

bool DebugOutput::
shouldOutputXMLDeviceCopies ()
{
   return vfOutputXMLDeviceCopies_d;
}

bool DebugOutput::
shouldOutputXMLDeviceDestination ()
{
   return vfOutputXMLDeviceDestination_d;
}

bool DebugOutput::
shouldOutputXMLDeviceDuplex ()
{
   return vfOutputXMLDeviceDuplex_d;
}

bool DebugOutput::
shouldOutputXMLDeviceJogging ()
{
   return vfOutputXMLDeviceJogging_d;
}

bool DebugOutput::
shouldOutputXMLDeviceNUp ()
{
   return vfOutputXMLDeviceNUp_d;
}

bool DebugOutput::
shouldOutputXMLDeviceScaling ()
{
   return vfOutputXMLDeviceScaling_d;
}

bool DebugOutput::
shouldOutputXMLDeviceStapling ()
{
   return vfOutputXMLDeviceStapling_d;
}

bool DebugOutput::
shouldOutputXMLDeviceInstance ()
{
   return vfOutputXMLDeviceInstance_d;
}

bool DebugOutput::
shouldOutputXMLDeviceBlitter ()
{
   return vfOutputXMLDeviceBlitter_d;
}

void DebugOutput::
setOutputDeviceBlitter (bool fOutputDeviceBlitter)
{
   vfOutputDeviceBlitter_d = fOutputDeviceBlitter;
}

void DebugOutput::
setOutputDeviceDither (bool fOutputDeviceDither)
{
   vfOutputDeviceDither_d = fOutputDeviceDither;
}

void DebugOutput::
setOutputDeviceInstance (bool fOutputDeviceInstance)
{
   vfOutputDeviceInstance_d = fOutputDeviceInstance;
}

void DebugOutput::
setOutputDevicePrintMode (bool fOutputDevicePrintMode)
{
   vfOutputDevicePrintMode_d = fOutputDevicePrintMode;
}

void DebugOutput::
setOutputDeviceTester (bool fOutputDeviceTester)
{
   vfOutputDeviceTester_d = fOutputDeviceTester;
}

void DebugOutput::
setOutputGplCompression (bool fOutputGplCompression)
{
   vfOutputGplCompression_d = fOutputGplCompression;
}

void DebugOutput::
setOutputGplDitherInstance (bool fOutputGplDitherInstance)
{
   vfOutputGplDitherInstance_d = fOutputGplDitherInstance;
}

void DebugOutput::
setOutputOmni (bool fOutputOmni)
{
   vfOutputOmni_d = fOutputOmni;
}

void DebugOutput::
setOutputOmniInterface (bool fOutputOmniInterface)
{
   vfOutputOmniInterface_d = fOutputOmniInterface;
}

void DebugOutput::
setOutputOmniProxy (bool fOutputOmniProxy)
{
   vfOutputOmniProxy_d = fOutputOmniProxy;
}

void DebugOutput::
setOutputPrintDevice (bool fOutputPrintDevice)
{
   vfOutputPrintDevice_d = fOutputPrintDevice;
}

void DebugOutput::
setOutputBlitter (bool fOutputBlitter)
{
   vfOutputBlitter_d = fOutputBlitter;
}

void DebugOutput::
setOutputInstance (bool fOutputInstance)
{
   vfOutputInstance_d = fOutputInstance;
}

void DebugOutput::
setOutputOmniPDCProxy (bool fOutputOmniPDCProxy)
{
   vfOutputOmniPDCProxy_d = fOutputOmniPDCProxy;
}

void DebugOutput::
setOutputOmniServer (bool fOutputOmniServer)
{
   vfOutputOmniServer_d = fOutputOmniServer;
}

void DebugOutput::
setOutputPrinterCommand (bool fOutputPrinterCommand)
{
   vfOutputPrinterCommand_d = fOutputPrinterCommand;
}

void DebugOutput::
setOutputPluggableInstance (bool fOutputPluggableInstance)
{
   vfOutputPluggableInstance_d = fOutputPluggableInstance;
}

void DebugOutput::
setOutputPluggableBlitter (bool fOutputPluggableBlitter)
{
   vfOutputPluggableBlitter_d = fOutputPluggableBlitter;
}

void DebugOutput::
setOutputDriverInfo (bool fOutputDriverInfo)
{
   vfOutputDriverInfo_d = fOutputDriverInfo;
}

void DebugOutput::
setOutputPDCBlitter (bool fOutputPDCBlitter)
{
   vfOutputPDCBlitter_d = fOutputPDCBlitter;
}

void DebugOutput::
setOutputPDCBlitterClient (bool fOutputPDCBlitterClient)
{
   vfOutputPDCBlitterClient_d = fOutputPDCBlitterClient;
}

void DebugOutput::
setOutputDeviceString (bool fOutputDeviceString)
{
   vfOutputDeviceString_d = fOutputDeviceString;
}

void DebugOutput::
setOutputUPDFDevice (bool fOutputUPDFDevice)
{
   vfOutputUPDFDevice_d = fOutputUPDFDevice;
}

void DebugOutput::
setOutputUPDFDeviceOrientation (bool fOutputUPDFDeviceOrientation)
{
   vfOutputUPDFDeviceOrientation_d = fOutputUPDFDeviceOrientation;
}

void DebugOutput::
setOutputUPDFDeviceForm (bool fOutputUPDFDeviceForm)
{
   vfOutputUPDFDeviceForm_d = fOutputUPDFDeviceForm;
}

void DebugOutput::
setOutputUPDFDeviceTray (bool fOutputUPDFDeviceTray)
{
   vfOutputUPDFDeviceTray_d = fOutputUPDFDeviceTray;
}

void DebugOutput::
setOutputUPDFDeviceMedia (bool fOutputUPDFDeviceMedia)
{
   vfOutputUPDFDeviceMedia_d = fOutputUPDFDeviceMedia;
}

void DebugOutput::
setOutputUPDFDevicePrintMode (bool fOutputUPDFDevicePrintMode)
{
   vfOutputUPDFDevicePrintMode_d = fOutputUPDFDevicePrintMode;
}

void DebugOutput::
setOutputUPDFDeviceResolution (bool fOutputUPDFDeviceResolution)
{
   vfOutputUPDFDeviceResolution_d = fOutputUPDFDeviceResolution;
}

void DebugOutput::
setOutputUPDFDeviceBooklet (bool fOutputUPDFDeviceBooklet)
{
   vfOutputUPDFDeviceBooklet_d = fOutputUPDFDeviceBooklet;
}

void DebugOutput::
setOutputUPDFDeviceCollation (bool fOutputUPDFDeviceCollation)
{
   vfOutputUPDFDeviceCollation_d = fOutputUPDFDeviceCollation;
}

void DebugOutput::
setOutputUPDFDeviceCopies (bool fOutputUPDFDeviceCopies)
{
   vfOutputUPDFDeviceCopies_d = fOutputUPDFDeviceCopies;
}

void DebugOutput::
setOutputUPDFDeviceDestination (bool fOutputUPDFDeviceDestination)
{
   vfOutputUPDFDeviceDestination_d = fOutputUPDFDeviceDestination;
}

void DebugOutput::
setOutputUPDFDeviceDuplex (bool fOutputUPDFDeviceDuplex)
{
   vfOutputUPDFDeviceDuplex_d = fOutputUPDFDeviceDuplex;
}

void DebugOutput::
setOutputUPDFDeviceJogging (bool fOutputUPDFDeviceJogging)
{
   vfOutputUPDFDeviceJogging_d = fOutputUPDFDeviceJogging;
}

void DebugOutput::
setOutputUPDFDeviceNUp (bool fOutputUPDFDeviceNUp)
{
   vfOutputUPDFDeviceNUp_d = fOutputUPDFDeviceNUp;
}

void DebugOutput::
setOutputUPDFDeviceScaling (bool fOutputUPDFDeviceScaling)
{
   vfOutputUPDFDeviceScaling_d = fOutputUPDFDeviceScaling;
}

void DebugOutput::
setOutputUPDFDeviceStapling (bool fOutputUPDFDeviceStapling)
{
   vfOutputUPDFDeviceStapling_d = fOutputUPDFDeviceStapling;
}

void DebugOutput::
setOutputUPDFDeviceInstance (bool fOutputUPDFDeviceInstance)
{
   vfOutputUPDFDeviceInstance_d = fOutputUPDFDeviceInstance;
}

void DebugOutput::
setOutputUPDFDeviceBlitter (bool fOutputUPDFDeviceBlitter)
{
   vfOutputUPDFDeviceBlitter_d = fOutputUPDFDeviceBlitter;
}

void DebugOutput::
setOutputXMLDevice (bool fOutputXMLDevice)
{
   vfOutputXMLDevice_d = fOutputXMLDevice;
}

void DebugOutput::
setOutputXMLDeviceOrientation (bool fOutputXMLDeviceOrientation)
{
   vfOutputXMLDeviceOrientation_d = fOutputXMLDeviceOrientation;
}

void DebugOutput::
setOutputXMLDeviceForm (bool fOutputXMLDeviceForm)
{
   vfOutputXMLDeviceForm_d = fOutputXMLDeviceForm;
}

void DebugOutput::
setOutputXMLDeviceTray (bool fOutputXMLDeviceTray)
{
   vfOutputXMLDeviceTray_d = fOutputXMLDeviceTray;
}

void DebugOutput::
setOutputXMLDeviceMedia (bool fOutputXMLDeviceMedia)
{
   vfOutputXMLDeviceMedia_d = fOutputXMLDeviceMedia;
}

void DebugOutput::
setOutputXMLDevicePrintMode (bool fOutputXMLDevicePrintMode)
{
   vfOutputXMLDevicePrintMode_d = fOutputXMLDevicePrintMode;
}

void DebugOutput::
setOutputXMLDeviceResolution (bool fOutputXMLDeviceResolution)
{
   vfOutputXMLDeviceResolution_d = fOutputXMLDeviceResolution;
}

void DebugOutput::
setOutputXMLDeviceBooklet (bool fOutputXMLDeviceBooklet)
{
   vfOutputXMLDeviceBooklet_d = fOutputXMLDeviceBooklet;
}

void DebugOutput::
setOutputXMLDeviceCollation (bool fOutputXMLDeviceCollation)
{
   vfOutputXMLDeviceCollation_d = fOutputXMLDeviceCollation;
}

void DebugOutput::
setOutputXMLDeviceCopies (bool fOutputXMLDeviceCopies)
{
   vfOutputXMLDeviceCopies_d = fOutputXMLDeviceCopies;
}

void DebugOutput::
setOutputXMLDeviceDestination (bool fOutputXMLDeviceDestination)
{
   vfOutputXMLDeviceDestination_d = fOutputXMLDeviceDestination;
}

void DebugOutput::
setOutputXMLDeviceDuplex (bool fOutputXMLDeviceDuplex)
{
   vfOutputXMLDeviceDuplex_d = fOutputXMLDeviceDuplex;
}

void DebugOutput::
setOutputXMLDeviceJogging (bool fOutputXMLDeviceJogging)
{
   vfOutputXMLDeviceJogging_d = fOutputXMLDeviceJogging;
}

void DebugOutput::
setOutputXMLDeviceNUp (bool fOutputXMLDeviceNUp)
{
   vfOutputXMLDeviceNUp_d = fOutputXMLDeviceNUp;
}

void DebugOutput::
setOutputXMLDeviceScaling (bool fOutputXMLDeviceScaling)
{
   vfOutputXMLDeviceScaling_d = fOutputXMLDeviceScaling;
}

void DebugOutput::
setOutputXMLDeviceStapling (bool fOutputXMLDeviceStapling)
{
   vfOutputXMLDeviceStapling_d = fOutputXMLDeviceStapling;
}

void DebugOutput::
setOutputXMLDeviceInstance (bool fOutputXMLDeviceInstance)
{
   vfOutputXMLDeviceInstance_d = fOutputXMLDeviceInstance;
}

void DebugOutput::
setOutputXMLDeviceBlitter (bool fOutputXMLDeviceBlitter)
{
   vfOutputXMLDeviceBlitter_d = fOutputXMLDeviceBlitter;
}
