/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "DeviceCopies.hpp"

DeviceCopies::
DeviceCopies (Device     *pDevice,
              int         iNumCopies,
              BinaryData *pbdData,
              int         iMinimum,
              int         iMaximum,
              bool        fSimulationRequired)
{
   pDevice_d             = pDevice;
   iNumCopies_d          = iNumCopies;
   pbdData_d             = pbdData;
   iMinimum_d            = iMinimum;
   iMaximum_d            = iMaximum;
   fSimulationRequired_d = fSimulationRequired;

   sprintf (achName_d, "%d", iNumCopies);
}

DeviceCopies::
~DeviceCopies ()
{
   delete pbdData_d;

   pDevice_d             = 0;
   iNumCopies_d          = 0;
   pbdData_d             = 0;
   fSimulationRequired_d = false;
}

DeviceCopies * DeviceCopies::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

int DeviceCopies::
nameToID (char *pszId)
{
   if (  !pszId
      || !*pszId
      )
   {
      return -1;
   }

   int iId = -1;

   sscanf (pszId, "%d", &iId);

   return iId;
}

char * DeviceCopies::
IDToName (int id)
{
   static char achName[25]; // @TBD

   sprintf (achName, "%d", id);

   return achName;
}

char * DeviceCopies::
getName ()
{
   return achName_d;
}

int DeviceCopies::
getNumCopies ()
{
   return iNumCopies_d;
}

int DeviceCopies::
getMinimum ()
{
   return iMinimum_d;
}

int DeviceCopies::
getMaximum ()
{
   return iMaximum_d;
}

BinaryData * DeviceCopies::
getData ()
{
   return pbdData_d;
}

bool DeviceCopies::
needsSimulation ()
{
   return fSimulationRequired_d;
}

string DeviceCopies::
toString (ostringstream& oss)
{
   oss << "{"
       << "iNumCopies_d = " << iNumCopies_d
       << ", fSimulationRequired_d = " << fSimulationRequired_d
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceCopies& const_self)
{
   DeviceCopies& self = const_cast<DeviceCopies&>(const_self);
   ostringstream oss;

   os << self.toString (oss);

   return os;
}
