/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DeviceCopies
#define _DeviceCopies

#include "Device.hpp"

#include <iostream>
#include <sstream>
#include <string>

using namespace std;

class DeviceCopies
{
public:
                         DeviceCopies         (Device             *pDevice,
                                               int                 iNumCopies,
                                               BinaryData         *pbdData,
                                               int                 iMinimum = 1,
                                               int                 iMaximum = 65536,
                                               bool                fSimulationRequired = false);
   virtual              ~DeviceCopies         ();

   static DeviceCopies  *create               (int                 id)
   {
      return (DeviceCopies *)0;
   }
   DeviceCopies         *create               (char               *pszId);
   static int            nameToID             (char               *pszId);
   static char          *IDToName             (int                 id);

   char                 *getName              ();

   int                   getNumCopies         ();
   int                   getMinimum           ();
   int                   getMaximum           ();

   BinaryData           *getData              ();

   bool                  needsSimulation      ();

   virtual bool          isCopiesSupported    (int                 id) = 0;

   virtual string        toString             (ostringstream&      oss);
   friend ostream&       operator<<           (ostream&            os,
                                               const DeviceCopies& self);

protected:
   virtual DeviceCopies *createV              (int                 id) = 0;

   Device     *pDevice_d;
   int         iNumCopies_d;
   int         iMinimum_d;
   int         iMaximum_d;
   bool        fSimulationRequired_d;
   char        achName_d[25]; // @TBD
   BinaryData *pbdData_d;
};

#endif
