/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DeviceForm
#define _DeviceForm

#include "Device.hpp"

#include <iostream>
#include <sstream>
#include <string>

using namespace std;

class DeviceForm
{
public:
   enum {
      FORM_UNLISTED               = -1,
      FORM_NONE,
      FORM_USER_DEFINED,
      FORM_TEST,
      FORM_TEST2,
      FORM_LEDGER,
      FORM_LEGAL,
      FORM_LEGAL_EXTRA,
      FORM_LETTER,
      FORM_LETTER_PLUS,
      FORM_LETTER_EXTRA,
      FORM_HALF_LETTER,
      FORM_EXECUTIVE,
      FORM_STATEMENT,
      FORM_TABLOID,
      FORM_UNIVERSAL,
      FORM_FOOLSCAP,
      FORM_QUARTO,
      FORM_FOLIO,
      FORM_FOLIO_SP,
      FORM_ROC_8K,
      FORM_ROC_16K,
      FORM_GOVERNMENT_LETTER,
      FORM_GOVERNMENT_LEGAL,
      FORM_EDP,
      FORM_EUROPEAN_EDP,
      FORM_PRC_16K,
      FORM_PRC_32K,
      FORM_DAI_PA_KAI,
      FORM_JUURO_KU_KAI,
      FORM_PA_KAI,
      FORM_WIDE,                         // @TBD - do we need the wide versions?
      FORM_LETTER_WIDE,                  //        can they be the difference
      FORM_FOOLSCAP_WIDE,                //        between US & English?
      FORM_A3_WIDE,
      FORM_A4_WIDE,
      FORM_WIDE_FORMAT,
      FORM_10_X_11,
      FORM_11_X_12,
      FORM_11_X_15,
      FORM_12_X_19,
      FORM_15_X_11,
      FORM_5_X_7,
      FORM_3_X_5_CARD,
      FORM_4_X_6_CARD,
      FORM_5_X_8_CARD,
      FORM_8_X_10_CARD,
      FORM_A6_CARD,
      FORM_CARD_148,
      FORM_HAGAKI_CARD,
      FORM_OUFUKU_CARD,
      FORM_POSTCARD,
      FORM_INDEX_4_X_6_EXT,
      FORM_A2_ENVELOPE,
      FORM_C5_ENVELOPE,
      FORM_C6_ENVELOPE,
      FORM_C7_ENVELOPE,
      FORM_C9_ENVELOPE,
      FORM_C10_ENVELOPE,
      FORM_D5_ENVELOPE,
      FORM_DL_ENVELOPE,
      FORM_ENVELOPE_132_X_220,
      FORM_ENVELOPE_6_X_9,
      FORM_ENVELOPE_7_X_9,
      FORM_ENVELOPE_6_1_2,
      FORM_ENVELOPE_9_X_11,
      FORM_ENVELOPE_10_X_13,
      FORM_ENVELOPE_10_X_14,
      FORM_ENVELOPE_10_X_15,
      FORM_MONARCH_ENVELOPE,
      FORM_PERSONAL_ENVELOPE,
      FORM_NUMBER_9_ENVELOPE,
      FORM_NUMBER_10_ENVELOPE,
      FORM_NUMBER_11_ENVELOPE,
      FORM_NUMBER_12_ENVELOPE,
      FORM_NUMBER_14_ENVELOPE,
      FORM_ITALIAN_ENVELOPE,
      FORM_POSTFIX_ENVELOPE,
      FORM_INVITE_ENVELOPE,
      FORM_CHOU2_ENVELOPE,
      FORM_CHOU3_ENVELOPE,
      FORM_CHOU4_ENVELOPE,
      FORM_KAHU_ENVELOPE,
      FORM_KAKU2_ENVELOPE,
      FORM_YOU4_ENVELOPE,
      FORM_PRC1_ENVELOPE,
      FORM_PRC2_ENVELOPE,
      FORM_PRC3_ENVELOPE,
      FORM_PRC4_ENVELOPE,
      FORM_PRC5_ENVELOPE,
      FORM_PRC6_ENVELOPE,
      FORM_PRC7_ENVELOPE,
      FORM_PRC8_ENVELOPE,
      FORM_PRC9_ENVELOPE,
      FORM_PRC10_ENVELOPE,
      FORM_DISK_LABELS,
      FORM_EURO_LABELS,
      FORM_SHIPPING_LABELS,
      FORM_STANDARD_LABELS_CLEAR,
      FORM_STANDARD_LABELS_WHITE,
      FORM_2A0,
      FORM_4A0,
      FORM_A0,
      FORM_A1,
      FORM_A2,
      FORM_A3,
      FORM_A3_EXTRA,
      FORM_A4,
      FORM_A4_EXTRA,
      FORM_A4_TAB,
      FORM_A5,
      FORM_A5_EXTRA,
      FORM_A6,
      FORM_A7,
      FORM_A8,
      FORM_A9,
      FORM_A10,
      FORM_B_PLUS,
      FORM_B0,
      FORM_B1,
      FORM_B2,
      FORM_B3,
      FORM_B4,
      FORM_B5,
      FORM_B5_EXTRA,
      FORM_B6,
      FORM_B6_C4,
      FORM_B7,
      FORM_B8,
      FORM_B9,
      FORM_B10,
      FORM_JIS_B0,
      FORM_JIS_B1,
      FORM_JIS_B2,
      FORM_JIS_B3,
      FORM_JIS_B4,
      FORM_JIS_B5,
      FORM_JIS_B6,
      FORM_JIS_B7,
      FORM_JIS_B8,
      FORM_JIS_B9,
      FORM_JIS_B10,
      FORM_JIS_EXEC,
      FORM_C0,
      FORM_C1,
      FORM_C2,
      FORM_C3,
      FORM_C4,
      FORM_C5,
      FORM_C6,
      FORM_C6_C5,
      FORM_C7,
      FORM_C7_C6,
      FORM_C8,
      FORM_C9,
      FORM_C10,
      FORM_E1,
      FORM_RA0,
      FORM_RA1,
      FORM_RA2,
      FORM_SRA0,
      FORM_SRA1,
      FORM_SRA2,
      FORM_US_FANFOLD,
      FORM_FANFOLD_1,                    // @TBD is there a real name for these fanfolds?
      FORM_FANFOLD_2,
      FORM_FANFOLD_3,
      FORM_FANFOLD_4,
      FORM_FANFOLD_5,
      FORM_GERMAN_LEGAL_FANFOLD,
      FORM_EUROPEAN_FANFOLD,
      FORM_PANORAMIC,
      FORM_PHOTO_100_150,
      FORM_PHOTO_200_300,
      FORM_PHOTO_4_6,
      FORM_PLOTTER_SIZE_A,
      FORM_PLOTTER_SIZE_B,
      FORM_PLOTTER_SIZE_C,
      FORM_PLOTTER_SIZE_D,
      FORM_PLOTTER_SIZE_E,
      FORM_PLOTTER_SIZE_F,
      FORM_US_A_ARCHITECTURAL,
      FORM_US_B_ARCHITECTURAL,
      FORM_US_C_ARCHITECTURAL,
      FORM_US_D_ARCHITECTURAL,
      FORM_US_E_ARCHITECTURAL,
      FORM_SUPER_A3_B,
      FORM_SUPER_A,
      FORM_SUPER_B,
      FORM_8_5_X_12_4,                  // @TBD is there a standard name for it?
      FORM_8_X_10_5,                    // @TBD is there a standard name for it?
      FORM_8_25_X_13,                   // @TBD is there a standard name for it?
      FORM_170_X_210,                   // @TBD is there a standard name for it?
      FORM_182_X_210,                   // @TBD is there a standard name for it?
      FORM_ROLL_76_2MM,
      FORM_ROLL_69_5MM
   };
   enum {
      COUNT = FORM_ROLL_69_5MM - FORM_UNLISTED + 1
   };
   enum {
      NO_CAPABILITIES             = 0x00000000,
      FORM_CAPABILITY_ROLL        = 0x00000001,
      FORM_CAPABILITY_USERDEFINED = 0x00000002
   };
   enum {
      RESERVED_BITS = 2
   };

                        DeviceForm        (Device           *pDevice,
                                           int               id,
                                           int               iCapabilities,
                                           BinaryData       *data,
                                           HardCopyCap      *hcInfo);
   virtual             ~DeviceForm        ();

   // By bringing named creates into base class, it creates a lot of complexity
   static DeviceForm   *create            (int               id)    { return (DeviceForm *)0; }
   DeviceForm          *create            (char             *pszId);
   static int           nameToID          (char             *pszId);
   static char         *IDToName          (int               id);

   void                 associateWith     (DeviceResolution *pResolution);

   static char         *getName           (Device           *pDevice,
                                           int               id);
   char                *getName           ();

   bool                 isID              (int               id);
   int                  getID             ();
   virtual char        *getDeviceID       ();
   bool                 hasCapability     (int               iFlag);
   int                  getCapabilities   ();
   BinaryData          *getData           ();
   HardCopyCap         *getHardCopyCap    ();

   static int           sizeToID          (int               cx,
                                           int               cy);
   int                  getCx             ();
   int                  getCy             ();
   void                 setCx             (int               id);
   void                 setCy             (int               id);
   int                  getOverrideX      ();
   int                  getOverrideY      ();

   virtual bool         isFormSupported   (int               id) = 0;

   virtual Enumeration *getEnumeration    () = 0;

   static bool          isReservedKeyword (const char       *pszId);
   static int           getReservedValue  (const char       *pszId);

   virtual string       toString          (ostringstream&    oss);
   friend ostream&      operator<<        (ostream&          os,
                                           const DeviceForm& self);

protected:
   virtual DeviceForm  *createV           (int               id) = 0;

   Device      *pDevice_d;
   int          id_d;
   int          iCapabilities_d;
   BinaryData  *data_d;
   HardCopyCap *hcInfo_d;

   int          iXOverride_d;
   int          iYOverride_d;
};

#endif
