/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceMedia.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

DeviceMedia::
DeviceMedia (Device     *pDevice,
             int         id,
             BinaryData *pbdData,
             int         iColorAdjustRequired,
             int         iAbsorption)
{
   pDevice_d              = pDevice;
   id_d                   = id;
   pbdData_d              = pbdData;
   iColorAdjustRequired_d = iColorAdjustRequired;
   iAbsorption_d          = iAbsorption;
}

DeviceMedia::
~DeviceMedia ()
{
   delete pbdData_d;

   pDevice_d              = 0;
   id_d                   = 0;
   pbdData_d              = 0;
   iColorAdjustRequired_d = 0;
   iAbsorption_d          = 0;
}

char * DeviceMedia::
getName (Device *pDevice, int id)
{
   return StringResource::getString (pDevice->getLanguageResource (),
                                     StringResource::STRINGGROUP_MEDIAS,
                                     id);
}

char * DeviceMedia::
getName ()
{
   return StringResource::getString (pDevice_d->getLanguageResource (),
                                     StringResource::STRINGGROUP_MEDIAS,
                                     id_d);
}

bool DeviceMedia::
isID (int id)
{
   return id == id_d;
}

int DeviceMedia::
getID ()
{
   return id_d;
}

char * DeviceMedia::
getDeviceID ()
{
   return 0;
}

BinaryData * DeviceMedia::
getData ()
{
   return pbdData_d;
}

int DeviceMedia::
getColorAdjustRequired ()
{
   return iColorAdjustRequired_d;
}

int DeviceMedia::
getAbsorption ()
{
   return iAbsorption_d;
}

DeviceMedia * DeviceMedia::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

#define STRING_OF(x) (apszMediaNames[x - MEDIA_UNLISTED])

static char *apszMediaNames[] = {
   "MEDIA_UNLISTED",
   "MEDIA_NONE",
   "MEDIA_AUTO",
   "MEDIA_USE_PRINTER_SETTING",
   "MEDIA_PLAIN",
   "MEDIA_TRANSPARENCY",
   "MEDIA_GLOSSY",
   "MEDIA_SPECIAL",
   "MEDIA_COATED",
   "MEDIA_BACKPRINT",
   "MEDIA_CLOTH",
   "MEDIA_THICK",
   "MEDIA_HIGH_GLOSS_FILM",
   "MEDIA_HIGH_RESOLUTION",
   "MEDIA_SPECIAL_360",
   "MEDIA_SPECIAL_720",
   "MEDIA_PLAIN_ENHANCED",
   "MEDIA_IRON_ON",
   "MEDIA_LABECA",
   "MEDIA_THERMAL",
   "MEDIA_CD_MASTER",
   "MEDIA_CARDBOARD",
   "MEDIA_POSTCARD",
   "MEDIA_PHOTOGRAPHIC_PAPER",
   "MEDIA_PHOTOGRAPHIC_LABEL",
   "MEDIA_HP_PREMIUM_PAPER",
   "MEDIA_HP_PHOTOGRAPHIC_PAPER",
   "MEDIA_ENVELOPE",
   "MEDIA_PREPRINTED",
   "MEDIA_LETTERHEAD",
   "MEDIA_PREPUNCHED",
   "MEDIA_LABELS",
   "MEDIA_BOND",
   "MEDIA_RECYCLED",
   "MEDIA_COLOR",
   "MEDIA_CARDSTOCK",
   "MEDIA_ROUGH",
   "MEDIA_ARCHIVAL_MATTE_PAPER",
   "MEDIA_BRIGHT_WHITE_INK_JET_PAPER",
   "MEDIA_COLORLIFE_PHOTO_PAPER",
   "MEDIA_DOUBLE_SIDED_MATTE_PAPER",
   "MEDIA_HEAVYWEIGH_MATTE_PAPER",
   "MEDIA_PREMIUM_SEMIGLOSS_PHOTO_PAPER",
   "MEDIA_PHOTOGRAPHIC_INK_JET_PAPER",
   "MEDIA_CUSTOM_1",
   "MEDIA_CUSTOM_2",
   "MEDIA_CUSTOM_3",
   "MEDIA_CUSTOM_4",
   "MEDIA_CUSTOM_5",
   "MEDIA_SPECIAL_BLUE",
   "MEDIA_SPECIAL_GREEN",
   "MEDIA_SPECIAL_GREY",
   "MEDIA_SPECIAL_IVORY",
   "MEDIA_SPECIAL_LETTERHEAD",
   "MEDIA_SPECIAL_ORANGE",
   "MEDIA_SPECIAL_PINK",
   "MEDIA_SPECIAL_PURPLE",
   "MEDIA_SPECIAL_RED",
   "MEDIA_SPECIAL_YELLOW",
   "MEDIA_SPECIAL_USER_COLOR",
   "MEDIA_TABSTOCK",
   "MEDIA_TABSTOCK_2",
   "MEDIA_TABSTOCK_3",
   "MEDIA_TABSTOCK_4",
   "MEDIA_TABSTOCK_5",
   "MEDIA_TABSTOCK_6",
   "MEDIA_TABSTOCK_7",
   "MEDIA_TABSTOCK_8",
   "MEDIA_TABSTOCK_9",
   "MEDIA_THICK_1",
   "MEDIA_THICK_2",
   "MEDIA_THICK_3",
   "MEDIA_THICK_BLUE",
   "MEDIA_THICK_GREEN",
   "MEDIA_THICK_GREY",
   "MEDIA_THICK_IVORY",
   "MEDIA_THICK_LETTERHEAD",
   "MEDIA_THICK_ORANGE",
   "MEDIA_THICK_PINK",
   "MEDIA_THICK_PURPLE",
   "MEDIA_THICK_RED",
   "MEDIA_THICK_YELLOW",
   "MEDIA_THICK_USER_COLOR",
   "MEDIA_TRANSLUCENT",
   "MEDIA_OTHER"
};

int DeviceMedia::
nameToID (char *pszId)
{
   static short int asiEntries[] = {
      MEDIA_ARCHIVAL_MATTE_PAPER,
      MEDIA_AUTO,
      MEDIA_BACKPRINT,
      MEDIA_BOND,
      MEDIA_BRIGHT_WHITE_INK_JET_PAPER,
      MEDIA_CARDBOARD,
      MEDIA_CARDSTOCK,
      MEDIA_CD_MASTER,
      MEDIA_CLOTH,
      MEDIA_COATED,
      MEDIA_COLOR,
      MEDIA_COLORLIFE_PHOTO_PAPER,
      MEDIA_CUSTOM_1,
      MEDIA_CUSTOM_2,
      MEDIA_CUSTOM_3,
      MEDIA_CUSTOM_4,
      MEDIA_CUSTOM_5,
      MEDIA_DOUBLE_SIDED_MATTE_PAPER,
      MEDIA_ENVELOPE,
      MEDIA_GLOSSY,
      MEDIA_HEAVYWEIGH_MATTE_PAPER,
      MEDIA_HIGH_GLOSS_FILM,
      MEDIA_HIGH_RESOLUTION,
      MEDIA_HP_PHOTOGRAPHIC_PAPER,
      MEDIA_HP_PREMIUM_PAPER,
      MEDIA_IRON_ON,
      MEDIA_LABECA,
      MEDIA_LABELS,
      MEDIA_LETTERHEAD,
      MEDIA_NONE,
      MEDIA_OTHER,
      MEDIA_PHOTOGRAPHIC_INK_JET_PAPER,
      MEDIA_PHOTOGRAPHIC_LABEL,
      MEDIA_PHOTOGRAPHIC_PAPER,
      MEDIA_PLAIN,
      MEDIA_PLAIN_ENHANCED,
      MEDIA_POSTCARD,
      MEDIA_PREMIUM_SEMIGLOSS_PHOTO_PAPER,
      MEDIA_PREPRINTED,
      MEDIA_PREPUNCHED,
      MEDIA_RECYCLED,
      MEDIA_ROUGH,
      MEDIA_SPECIAL,
      MEDIA_SPECIAL_360,
      MEDIA_SPECIAL_720,
      MEDIA_SPECIAL_BLUE,
      MEDIA_SPECIAL_GREEN,
      MEDIA_SPECIAL_GREY,
      MEDIA_SPECIAL_IVORY,
      MEDIA_SPECIAL_LETTERHEAD,
      MEDIA_SPECIAL_ORANGE,
      MEDIA_SPECIAL_PINK,
      MEDIA_SPECIAL_PURPLE,
      MEDIA_SPECIAL_RED,
      MEDIA_SPECIAL_USER_COLOR,
      MEDIA_SPECIAL_YELLOW,
      MEDIA_TABSTOCK,
      MEDIA_TABSTOCK_2,
      MEDIA_TABSTOCK_3,
      MEDIA_TABSTOCK_4,
      MEDIA_TABSTOCK_5,
      MEDIA_TABSTOCK_6,
      MEDIA_TABSTOCK_7,
      MEDIA_TABSTOCK_8,
      MEDIA_TABSTOCK_9,
      MEDIA_THERMAL,
      MEDIA_THICK,
      MEDIA_THICK_1,
      MEDIA_THICK_2,
      MEDIA_THICK_3,
      MEDIA_THICK_BLUE,
      MEDIA_THICK_GREEN,
      MEDIA_THICK_GREY,
      MEDIA_THICK_IVORY,
      MEDIA_THICK_LETTERHEAD,
      MEDIA_THICK_ORANGE,
      MEDIA_THICK_PINK,
      MEDIA_THICK_PURPLE,
      MEDIA_THICK_RED,
      MEDIA_THICK_USER_COLOR,
      MEDIA_THICK_YELLOW,
      MEDIA_TRANSLUCENT,
      MEDIA_TRANSPARENCY,
      MEDIA_USE_PRINTER_SETTING
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return MEDIA_UNLISTED;
}

char * DeviceMedia::
IDToName (int id)
{
   id -= MEDIA_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszMediaNames)
      )
      return apszMediaNames[id];
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + MEDIA_UNLISTED);

      return achUnknown;
   }
}

typedef struct _ReservedMap {
   const char *pszName;
   int         iValue;
} RESERVEDMAP, *PRESERVEDMAP;

static const RESERVEDMAP vaReservedKeywords[] = {
   { "MEDIA_NO_ABSORPTION",    0 },
   { "MEDIA_LIGHT_ABSORPTION", 1 },
   { "MEDIA_HEAVY_ABSORPTION", 2 }
};

bool DeviceMedia::
isReservedKeyword (const char *pszId)
{
   for (int i = 0; i < (int)dimof (vaReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, vaReservedKeywords[i].pszName))
         return true;
   }

   return false;
}

int DeviceMedia::
getReservedValue (const char *pszId)
{
   for (int i = 0; i < (int)dimof (vaReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, vaReservedKeywords[i].pszName))
         return vaReservedKeywords[i].iValue;
   }

   return 0;
}

string DeviceMedia::
toString (ostringstream& oss)
{
   oss << "{"
       << STRING_OF (id_d)
       << ", iColorAdjustRequired_d = " << iColorAdjustRequired_d
       << ", iAbsorption_d = " << iAbsorption_d
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceMedia& const_self)
{
   DeviceMedia&  self = const_cast<DeviceMedia&>(const_self);
   ostringstream oss;

   os << self.toString (oss);

   return os;
}
