/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DeviceMedia
#define _DeviceMedia

#include "Device.hpp"

#include <iostream>
#include <sstream>
#include <string>

using namespace std;

class DeviceMedia
{
public:
   enum {
      MEDIA_UNLISTED              = -1,
      MEDIA_NONE,
      MEDIA_AUTO,                 /* aka default */
      MEDIA_USE_PRINTER_SETTING,
      MEDIA_PLAIN,
      MEDIA_TRANSPARENCY,
      MEDIA_GLOSSY,
      MEDIA_SPECIAL,
      MEDIA_COATED,
      MEDIA_BACKPRINT,
      MEDIA_CLOTH,
      MEDIA_THICK,
      MEDIA_HIGH_GLOSS_FILM,
      MEDIA_HIGH_RESOLUTION,
      MEDIA_SPECIAL_360,
      MEDIA_SPECIAL_720,
      MEDIA_PLAIN_ENHANCED,
      MEDIA_IRON_ON,
      MEDIA_LABECA,
      MEDIA_THERMAL,
      MEDIA_CD_MASTER,
      MEDIA_CARDBOARD,
      MEDIA_POSTCARD,
      MEDIA_PHOTOGRAPHIC_PAPER,
      MEDIA_PHOTOGRAPHIC_LABEL,
      MEDIA_HP_PREMIUM_PAPER,
      MEDIA_HP_PHOTOGRAPHIC_PAPER,
      MEDIA_ENVELOPE,
      MEDIA_PREPRINTED,
      MEDIA_LETTERHEAD,
      MEDIA_PREPUNCHED,
      MEDIA_LABELS,
      MEDIA_BOND,
      MEDIA_RECYCLED,
      MEDIA_COLOR,
      MEDIA_CARDSTOCK,
      MEDIA_ROUGH,
      MEDIA_ARCHIVAL_MATTE_PAPER,
      MEDIA_BRIGHT_WHITE_INK_JET_PAPER,
      MEDIA_COLORLIFE_PHOTO_PAPER,
      MEDIA_DOUBLE_SIDED_MATTE_PAPER,
      MEDIA_HEAVYWEIGH_MATTE_PAPER,
      MEDIA_PREMIUM_SEMIGLOSS_PHOTO_PAPER,
      MEDIA_PHOTOGRAPHIC_INK_JET_PAPER,
      MEDIA_CUSTOM_1,
      MEDIA_CUSTOM_2,
      MEDIA_CUSTOM_3,
      MEDIA_CUSTOM_4,
      MEDIA_CUSTOM_5,
      MEDIA_SPECIAL_BLUE,
      MEDIA_SPECIAL_GREEN,
      MEDIA_SPECIAL_GREY,
      MEDIA_SPECIAL_IVORY,
      MEDIA_SPECIAL_LETTERHEAD,
      MEDIA_SPECIAL_ORANGE,
      MEDIA_SPECIAL_PINK,
      MEDIA_SPECIAL_PURPLE,
      MEDIA_SPECIAL_RED,
      MEDIA_SPECIAL_YELLOW,
      MEDIA_SPECIAL_USER_COLOR,
      MEDIA_TABSTOCK,
      MEDIA_TABSTOCK_2,
      MEDIA_TABSTOCK_3,
      MEDIA_TABSTOCK_4,
      MEDIA_TABSTOCK_5,
      MEDIA_TABSTOCK_6,
      MEDIA_TABSTOCK_7,
      MEDIA_TABSTOCK_8,
      MEDIA_TABSTOCK_9,
      MEDIA_THICK_1,
      MEDIA_THICK_2,
      MEDIA_THICK_3,
      MEDIA_THICK_BLUE,
      MEDIA_THICK_GREEN,
      MEDIA_THICK_GREY,
      MEDIA_THICK_IVORY,
      MEDIA_THICK_LETTERHEAD,
      MEDIA_THICK_ORANGE,
      MEDIA_THICK_PINK,
      MEDIA_THICK_PURPLE,
      MEDIA_THICK_RED,
      MEDIA_THICK_YELLOW,
      MEDIA_THICK_USER_COLOR,
      MEDIA_TRANSLUCENT,
      MEDIA_OTHER
   };
   enum {
      COUNT = MEDIA_OTHER - MEDIA_UNLISTED + 1
   };

   enum {
      MEDIA_NO_ABSORPTION,
      MEDIA_LIGHT_ABSORPTION,
      MEDIA_HEAVY_ABSORPTION
   };

                        DeviceMedia            (Device            *pDevice,
                                                int                id,
                                                BinaryData        *pbdData,
                                                int                iColorAdjustRequired,
                                                int                iAbsorption);
   virtual             ~DeviceMedia            ();

   // By bringing named creates into base class, it creates a lot of complexity
   static  DeviceMedia *create                 (int                id)    { return (DeviceMedia *)0; }
   DeviceMedia         *create                 (char              *pszId);
   static  int          nameToID               (char              *pszId);
   static  char        *IDToName               (int                id);

   static  char        *getName                (Device            *pDevice,
                                                int                id);
   char                *getName                ();

   bool                 isID                   (int                id);
   int                  getID                  ();
   virtual char        *getDeviceID            ();
   int                  getTypeID              ();
   int                  getStringID            ();
   BinaryData          *getData                ();
   int                  getColorAdjustRequired ();
   int                  getAbsorption          ();

   virtual bool         isMediaSupported       (int                id) = 0;

   virtual Enumeration *getEnumeration         () = 0;

   static bool          isReservedKeyword      (const char        *pszId);
   static int           getReservedValue       (const char        *pszId);

   virtual string       toString               (ostringstream&     oss);
   friend ostream&      operator<<             (ostream&           os,
                                                const DeviceMedia& self);

protected:
   virtual DeviceMedia *createV                (int                id) = 0;

   Device     *pDevice_d;
   int         id_d;
   BinaryData *pbdData_d;
   int         iColorAdjustRequired_d;
   int         iAbsorption_d;
};

#endif
