/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Device.hpp"
#include "Omni.hpp"

Device *
loadDevice (char *pszFullDeviceName, char *pszJobProperties)
{
   Device            *pDevice           = 0;
   PFNNEWDEVICEWARGS  pfnNewDeviceWArgs = 0;
   GModule           *hmodDevice        = 0;

   cout << "Trying to load \""
        << pszFullDeviceName
        << "\" with \""
        << pszJobProperties
        << "\""
        << endl;

   hmodDevice = g_module_open (pszFullDeviceName, (GModuleFlags)0);

   if (DebugOutput::shouldOutputDeviceTester ()) DebugOutput::getErrorStream () << __FUNCTION__ << ": hmodDevice = " << hex << hmodDevice << endl;

   if (!hmodDevice)
   {
      DebugOutput::getErrorStream () << "g_module_error returns " << g_module_error () << endl;
      return 0;
   }

   g_module_symbol (hmodDevice,
                    "newDeviceW_JopProp_Advanced",
                    (gpointer *)&pfnNewDeviceWArgs);

   if (DebugOutput::shouldOutputDeviceTester ()) DebugOutput::getErrorStream () << __FUNCTION__ << ": pfnNewDeviceWArgs = 0x" << hex << (int)pfnNewDeviceWArgs << dec << endl;

   if (!pfnNewDeviceWArgs)
      return 0;

   pDevice = pfnNewDeviceWArgs (pszJobProperties, true);

   return pDevice;
}

int
handleDevice (char *pszFullDeviceName, char *pszJobProperties)
{
   Device *pDevice = 0;

   if (strstr (pszFullDeviceName, "/lib"))
   {
//    pszFullDeviceName = strstr (pszFullDeviceName, "/lib") + 1;
   }

//#define PDC 1
#ifdef PDC
   pDevice = new OmniPDCProxy (0,                 // client exe to spawn
                               pszFullDeviceName, // device name
                               pszJobProperties,  // job properties
                               true);             // is renderer
#else
   pDevice = loadDevice (pszFullDeviceName, pszJobProperties);
#endif

   if (!pDevice)
   {
      DebugOutput::getErrorStream () << "DeviceTester4::" << __FUNCTION__ << ": the new device failed to open!" << endl;

      return 2;
   }

   if (pDevice->hasError ())
   {
      DebugOutput::getErrorStream () << "DeviceTester4::" << __FUNCTION__ << ": the new device has an error!" << endl;

      return 3;
   }

   Enumeration *pEnumKeys   = pDevice->listJobPropertyKeys ();
   Enumeration *pEnumValues = 0;

   while (pEnumKeys->hasMoreElements ())
   {
      char *pszKey = (char *)pEnumKeys->nextElement ();

      if (!pszKey)
      {
         // @TBD
         continue;
      }

      string *pValue = pDevice->getJobProperty (pszKey);

      if (!pValue)
      {
         // @TBD
         continue;
      }

      cout << pszKey << " = " << *pValue << endl;

      pEnumValues = pDevice->listKeyValues (pszKey);

      while (pEnumValues->hasMoreElements ())
      {
         char *pszValue = (char *)pEnumValues->nextElement ();

         cout << "\t"
              << pszValue;

         string *pRet = pDevice->translateKeyValue (pszKey, pszValue);

         if (pRet)
         {
            cout << "("
                 << *pRet
                 << ")";
         }

         cout << endl;

         delete pRet;
      }

      delete pValue;
      delete pEnumValues;
   }

   delete pEnumKeys;

   pEnumKeys = pDevice->listDeviceJobPropertyKeys ();

   while (pEnumKeys->hasMoreElements ())
   {
      char *pszKey = (char *)pEnumKeys->nextElement ();

      cout << pszKey << " = ";

      string *pValue = pDevice->getJobProperty (pszKey);

      cout << *pValue << endl;

      pEnumValues = pDevice->listKeyValues (pszKey);

      while (pEnumValues->hasMoreElements ())
      {
         char *pszValue = (char *)pEnumValues->nextElement ();

         cout << "\t"
              << pszValue;

         string *pRet = pDevice->translateKeyValue (pszKey, pszValue);

         if (pRet)
         {
            cout << "("
                 << *pRet
                 << ")";
         }

         cout << endl;

         delete pRet;
      }

      delete pValue;
      delete pEnumValues;
   }

   delete pEnumKeys;

   delete pDevice;

   return 0;
}

int
main (int argc, char *argv[])
{
#define JP_BLOCK_SIZE  256
   char *pszJobProperties  = 0;
   int   cbJobPropertyLeft = 0;
   int   iFailed           = 0;

   if (!g_module_supported ())
   {
      cerr << "This program needs glib's module routines!" << endl;

      return 1;
   }

   if (2 > argc)
   {
      cerr << "Usage: " << argv[0] << " printer_name" << endl;

      return 1;
   }

   Omni::initialize ();

   for (int i = 1; i < argc; i++)
   {
      if (0 == strcasecmp (argv[i], "all"))
      {
         Enumeration *pEnum = Omni::listDevices ();

         while (pEnum->hasMoreElements ())
         {
            OmniDevice *pOD = (OmniDevice *)pEnum->nextElement ();

            if (pOD)
            {
               char *pszLibName = pOD->getLibraryName ();

               cout << "Checking: " << pszLibName << endl;

               iFailed |= handleDevice (pszLibName, pOD->getJobProperties ());

               delete pOD;
            }
            else
            {
               cerr << "Error" << endl;

               iFailed = 1;
            }
         }

         delete pEnum;
      }
      else if (0 == strcasecmp (argv[i], "driver"))
      {
         char achLibDeviceName[512];  // @TBD

         if (0 != strncasecmp (argv[i + 1], "lib", 3))
         {
            sprintf (achLibDeviceName, "lib%s.so", argv[i + 1]);
         }
         else
         {
            strcpy (achLibDeviceName, argv[i + 1]);
         }

         iFailed |= handleDevice (achLibDeviceName, pszJobProperties);
      }
      else
      {
         // Assume that it is a Job Property (where there needs to be at least
         // 2 parameters)
         if (2 <= argc - i)
         {  //                  key               '='  value                 ' \0'
            int  iSpaceNeeded = strlen (argv[i]) + 1 + strlen (argv[i + 1]) + 2;
            bool fSpaceFound  = false;

            if (strchr (argv[i + 1], ' '))
            {
               fSpaceFound = true;
               iSpaceNeeded += 2;
            }

            // Out of space?
            if (  !pszJobProperties
               || cbJobPropertyLeft < iSpaceNeeded
               )
            {
               cbJobPropertyLeft = JP_BLOCK_SIZE;
               if (pszJobProperties)
                  cbJobPropertyLeft += strlen (pszJobProperties) + 1;

               char *pszTemp = (char *)calloc (1, cbJobPropertyLeft);

               if (!pszTemp)
               {
                  DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": Out of memory!" << endl;

                  iFailed = 2;
                  goto BUGOUT;
               }

               *pszTemp = '\0';

               if (pszJobProperties)
               {
                  strcpy (pszTemp, pszJobProperties);
                  free (pszJobProperties);
               }
               pszJobProperties = pszTemp;
            }

            strcat (pszJobProperties, argv[i]);
            strcat (pszJobProperties, "=");
            if (fSpaceFound)
               strcat (pszJobProperties, "\"");
            strcat (pszJobProperties, argv[i + 1]);
            if (fSpaceFound)
               strcat (pszJobProperties, "\"");
            strcat (pszJobProperties, " ");

            cbJobPropertyLeft -= strlen (argv[i]) + 1 + strlen (argv[i + 1]) + 2;

            if (fSpaceFound)
               cbJobPropertyLeft -= 2;

            i++;
         }
      }
   }

BUGOUT:
   if (pszJobProperties)
   {
      free (pszJobProperties);
   }

   Omni::terminate ();

   return iFailed;
}
