/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "Device.hpp"
#include "Omni.hpp"

#define USE_TEST_1 1
//#define USE_TEST_2 1
#define USE_TEST_3 1
//#define USE_TEST_4 1
//#define USE_TEST_5 1
#define USE_TEST_6 1
#define USE_TEST_7 1

#if defined (USE_TEST_1) || defined (USE_TEST_2)

static int aiOmniIDs[] = {
   DeviceForm::FORM_USER_DEFINED,
   DeviceForm::FORM_TEST,
   DeviceForm::FORM_TEST2,
   DeviceForm::FORM_LEDGER,
   DeviceForm::FORM_LEGAL,
   DeviceForm::FORM_LEGAL_EXTRA,
   DeviceForm::FORM_LETTER,
   DeviceForm::FORM_LETTER_PLUS,
   DeviceForm::FORM_LETTER_EXTRA,
   DeviceForm::FORM_HALF_LETTER,
   DeviceForm::FORM_EXECUTIVE,
   DeviceForm::FORM_STATEMENT,
   DeviceForm::FORM_TABLOID,
   DeviceForm::FORM_UNIVERSAL,
   DeviceForm::FORM_FOOLSCAP,
   DeviceForm::FORM_QUARTO,
   DeviceForm::FORM_FOLIO,
   DeviceForm::FORM_FOLIO_SP,
   DeviceForm::FORM_ROC_8K,
   DeviceForm::FORM_ROC_16K,
   DeviceForm::FORM_GOVERNMENT_LETTER,
   DeviceForm::FORM_GOVERNMENT_LEGAL,
   DeviceForm::FORM_EDP,
   DeviceForm::FORM_EUROPEAN_EDP,
   DeviceForm::FORM_PRC_16K,
   DeviceForm::FORM_PRC_32K,
   DeviceForm::FORM_DAI_PA_KAI,
   DeviceForm::FORM_JUURO_KU_KAI,
   DeviceForm::FORM_PA_KAI,
   DeviceForm::FORM_WIDE,                         // @TBD - do we need the wide versions?
   DeviceForm::FORM_LETTER_WIDE,                  //        can they be the difference
   DeviceForm::FORM_FOOLSCAP_WIDE,                //        between US & English?
   DeviceForm::FORM_A3_WIDE,
   DeviceForm::FORM_A4_WIDE,
   DeviceForm::FORM_WIDE_FORMAT,
   DeviceForm::FORM_10_X_11,
   DeviceForm::FORM_11_X_12,
   DeviceForm::FORM_11_X_15,
   DeviceForm::FORM_12_X_19,
   DeviceForm::FORM_15_X_11,
   DeviceForm::FORM_5_X_7,
   DeviceForm::FORM_3_X_5_CARD,
   DeviceForm::FORM_4_X_6_CARD,
   DeviceForm::FORM_5_X_8_CARD,
   DeviceForm::FORM_8_X_10_CARD,
   DeviceForm::FORM_A6_CARD,
   DeviceForm::FORM_CARD_148,
   DeviceForm::FORM_HAGAKI_CARD,
   DeviceForm::FORM_OUFUKU_CARD,
   DeviceForm::FORM_POSTCARD,
   DeviceForm::FORM_INDEX_4_X_6_EXT,
   DeviceForm::FORM_A2_ENVELOPE,
   DeviceForm::FORM_C5_ENVELOPE,
   DeviceForm::FORM_C6_ENVELOPE,
   DeviceForm::FORM_C7_ENVELOPE,
   DeviceForm::FORM_C9_ENVELOPE,
   DeviceForm::FORM_C10_ENVELOPE,
   DeviceForm::FORM_D5_ENVELOPE,
   DeviceForm::FORM_DL_ENVELOPE,
   DeviceForm::FORM_ENVELOPE_132_X_220,
   DeviceForm::FORM_ENVELOPE_6_X_9,
   DeviceForm::FORM_ENVELOPE_7_X_9,
   DeviceForm::FORM_ENVELOPE_6_1_2,
   DeviceForm::FORM_ENVELOPE_9_X_11,
   DeviceForm::FORM_ENVELOPE_10_X_13,
   DeviceForm::FORM_ENVELOPE_10_X_14,
   DeviceForm::FORM_ENVELOPE_10_X_15,
   DeviceForm::FORM_MONARCH_ENVELOPE,
   DeviceForm::FORM_PERSONAL_ENVELOPE,
   DeviceForm::FORM_NUMBER_9_ENVELOPE,
   DeviceForm::FORM_NUMBER_10_ENVELOPE,
   DeviceForm::FORM_NUMBER_11_ENVELOPE,
   DeviceForm::FORM_NUMBER_12_ENVELOPE,
   DeviceForm::FORM_NUMBER_14_ENVELOPE,
   DeviceForm::FORM_ITALIAN_ENVELOPE,
   DeviceForm::FORM_POSTFIX_ENVELOPE,
   DeviceForm::FORM_INVITE_ENVELOPE,
   DeviceForm::FORM_CHOU2_ENVELOPE,
   DeviceForm::FORM_CHOU3_ENVELOPE,
   DeviceForm::FORM_CHOU4_ENVELOPE,
   DeviceForm::FORM_KAHU_ENVELOPE,
   DeviceForm::FORM_KAKU2_ENVELOPE,
   DeviceForm::FORM_YOU4_ENVELOPE,
   DeviceForm::FORM_PRC1_ENVELOPE,
   DeviceForm::FORM_PRC2_ENVELOPE,
   DeviceForm::FORM_PRC3_ENVELOPE,
   DeviceForm::FORM_PRC4_ENVELOPE,
   DeviceForm::FORM_PRC5_ENVELOPE,
   DeviceForm::FORM_PRC6_ENVELOPE,
   DeviceForm::FORM_PRC7_ENVELOPE,
   DeviceForm::FORM_PRC8_ENVELOPE,
   DeviceForm::FORM_PRC9_ENVELOPE,
   DeviceForm::FORM_PRC10_ENVELOPE,
   DeviceForm::FORM_DISK_LABELS,
   DeviceForm::FORM_EURO_LABELS,
   DeviceForm::FORM_SHIPPING_LABELS,
   DeviceForm::FORM_STANDARD_LABELS_CLEAR,
   DeviceForm::FORM_STANDARD_LABELS_WHITE,
   DeviceForm::FORM_2A0,
   DeviceForm::FORM_4A0,
   DeviceForm::FORM_A0,
   DeviceForm::FORM_A1,
   DeviceForm::FORM_A2,
   DeviceForm::FORM_A3,
   DeviceForm::FORM_A3_EXTRA,
   DeviceForm::FORM_A4,
   DeviceForm::FORM_A4_EXTRA,
   DeviceForm::FORM_A4_TAB,
   DeviceForm::FORM_A5,
   DeviceForm::FORM_A5_EXTRA,
   DeviceForm::FORM_A6,
   DeviceForm::FORM_A7,
   DeviceForm::FORM_A8,
   DeviceForm::FORM_A9,
   DeviceForm::FORM_A10,
   DeviceForm::FORM_B_PLUS,
   DeviceForm::FORM_B0,
   DeviceForm::FORM_B1,
   DeviceForm::FORM_B2,
   DeviceForm::FORM_B3,
   DeviceForm::FORM_B4,
   DeviceForm::FORM_B5,
   DeviceForm::FORM_B5_EXTRA,
   DeviceForm::FORM_B6,
   DeviceForm::FORM_B6_C4,
   DeviceForm::FORM_B7,
   DeviceForm::FORM_B8,
   DeviceForm::FORM_B9,
   DeviceForm::FORM_B10,
   DeviceForm::FORM_JIS_B0,
   DeviceForm::FORM_JIS_B1,
   DeviceForm::FORM_JIS_B2,
   DeviceForm::FORM_JIS_B3,
   DeviceForm::FORM_JIS_B4,
   DeviceForm::FORM_JIS_B5,
   DeviceForm::FORM_JIS_B6,
   DeviceForm::FORM_JIS_B7,
   DeviceForm::FORM_JIS_B8,
   DeviceForm::FORM_JIS_B9,
   DeviceForm::FORM_JIS_B10,
   DeviceForm::FORM_JIS_EXEC,
   DeviceForm::FORM_C0,
   DeviceForm::FORM_C1,
   DeviceForm::FORM_C2,
   DeviceForm::FORM_C3,
   DeviceForm::FORM_C4,
   DeviceForm::FORM_C5,
   DeviceForm::FORM_C6,
   DeviceForm::FORM_C6_C5,
   DeviceForm::FORM_C7,
   DeviceForm::FORM_C7_C6,
   DeviceForm::FORM_C8,
   DeviceForm::FORM_C9,
   DeviceForm::FORM_C10,
   DeviceForm::FORM_E1,
   DeviceForm::FORM_RA0,
   DeviceForm::FORM_RA1,
   DeviceForm::FORM_RA2,
   DeviceForm::FORM_SRA0,
   DeviceForm::FORM_SRA1,
   DeviceForm::FORM_SRA2,
   DeviceForm::FORM_US_FANFOLD,
   DeviceForm::FORM_FANFOLD_1,                    // @TBD is there a real name for these fanfolds?
   DeviceForm::FORM_FANFOLD_2,
   DeviceForm::FORM_FANFOLD_3,
   DeviceForm::FORM_FANFOLD_4,
   DeviceForm::FORM_FANFOLD_5,
   DeviceForm::FORM_GERMAN_LEGAL_FANFOLD,
   DeviceForm::FORM_EUROPEAN_FANFOLD,
   DeviceForm::FORM_PANORAMIC,
   DeviceForm::FORM_PHOTO_100_150,
   DeviceForm::FORM_PHOTO_200_300,
   DeviceForm::FORM_PHOTO_4_6,
   DeviceForm::FORM_PLOTTER_SIZE_A,
   DeviceForm::FORM_PLOTTER_SIZE_B,
   DeviceForm::FORM_PLOTTER_SIZE_C,
   DeviceForm::FORM_PLOTTER_SIZE_D,
   DeviceForm::FORM_PLOTTER_SIZE_E,
   DeviceForm::FORM_PLOTTER_SIZE_F,
   DeviceForm::FORM_US_A_ARCHITECTURAL,
   DeviceForm::FORM_US_B_ARCHITECTURAL,
   DeviceForm::FORM_US_C_ARCHITECTURAL,
   DeviceForm::FORM_US_D_ARCHITECTURAL,
   DeviceForm::FORM_US_E_ARCHITECTURAL,
   DeviceForm::FORM_SUPER_A3_B,
   DeviceForm::FORM_SUPER_A,
   DeviceForm::FORM_SUPER_B,
   DeviceForm::FORM_ROLL_76_2MM,
   DeviceForm::FORM_ROLL_69_5MM
};

#endif

typedef struct _FormMapping {
   char   *pszUPDFName;
   int     iOmniFormID;
} FORMMAP, *PFORMMAP;

#if defined (USE_TEST_2) || defined (USE_TEST_3)

static FORMMAP vaFormMapping1[] = {
   { "iso_2a0",          DeviceForm::FORM_2A0                },
   { "iso_4a0",          DeviceForm::FORM_4A0                },
   { "iso_a0x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a0",           DeviceForm::FORM_A0                 },
   { "iso_a10",          DeviceForm::FORM_A10                },
   { "iso_a1",           DeviceForm::FORM_A1                 },
   { "iso_a1x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a1x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a2",           DeviceForm::FORM_A2                 },
   { "iso_a2x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a2x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a2x5",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3",           DeviceForm::FORM_A3                 },
   { "iso_a3-extra",     DeviceForm::FORM_A3_EXTRA           },
   { "iso_a3x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x5",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x6",         DeviceForm::FORM_UNLISTED           },
   { "iso_a3x7",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4",           DeviceForm::FORM_A4                 },
   { "iso_a4-extra",     DeviceForm::FORM_A4_EXTRA           },
   { "iso_a4-tab",       DeviceForm::FORM_A4_TAB             },
   { "iso_a4x3",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x4",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x5",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x6",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x7",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x8",         DeviceForm::FORM_UNLISTED           },
   { "iso_a4x9",         DeviceForm::FORM_UNLISTED           },
   { "iso_a5",           DeviceForm::FORM_A5                 },
   { "iso_a5-extra",     DeviceForm::FORM_A5_EXTRA           },
   { "iso_a6",           DeviceForm::FORM_A6                 },
   { "iso_a7",           DeviceForm::FORM_A7                 },
   { "iso_a8",           DeviceForm::FORM_A8                 },
   { "iso_a9",           DeviceForm::FORM_A9                 },
   { "iso_b0",           DeviceForm::FORM_B0                 },
   { "iso_b10",          DeviceForm::FORM_B10                },
   { "iso_b1",           DeviceForm::FORM_B1                 },
   { "iso_b2",           DeviceForm::FORM_B2                 },
   { "iso_b3",           DeviceForm::FORM_B3                 },
   { "iso_b4",           DeviceForm::FORM_B4                 },
   { "iso_b5",           DeviceForm::FORM_B5                 },
   { "iso_b5-extra",     DeviceForm::FORM_B5_EXTRA           },
   { "iso_b6",           DeviceForm::FORM_B6                 },
   { "iso_b6c4",         DeviceForm::FORM_B6_C4              },
   { "iso_b7",           DeviceForm::FORM_B7                 },
   { "iso_b8",           DeviceForm::FORM_B8                 },
   { "iso_b9",           DeviceForm::FORM_B9                 },
   { "iso_c0",           DeviceForm::FORM_C0                 },
   { "iso_c10",          DeviceForm::FORM_C10                },
   { "iso_c1",           DeviceForm::FORM_C1                 },
   { "iso_c2",           DeviceForm::FORM_C2                 },
   { "iso_c3",           DeviceForm::FORM_C3                 },
   { "iso_c4",           DeviceForm::FORM_C4                 },
   { "iso_c5",           DeviceForm::FORM_C5                 },
   { "iso_c6",           DeviceForm::FORM_C6                 },
   { "iso_c6c5",         DeviceForm::FORM_C6_C5              },
   { "iso_c7",           DeviceForm::FORM_C7                 },
   { "iso_c7c6",         DeviceForm::FORM_C7_C6              },
   { "iso_c8",           DeviceForm::FORM_C8                 },
   { "iso_c9",           DeviceForm::FORM_C9                 },
   { "iso_dl",           DeviceForm::FORM_UNLISTED           },
   { "iso_ra0",          DeviceForm::FORM_RA0                },
   { "iso_ra1",          DeviceForm::FORM_RA1                },
   { "iso_ra2",          DeviceForm::FORM_RA2                },
   { "iso_sra0",         DeviceForm::FORM_SRA0               },
   { "iso_sra1",         DeviceForm::FORM_SRA1               },
   { "iso_sra2",         DeviceForm::FORM_SRA2               },
   { "jis_b0",           DeviceForm::FORM_JIS_B0             },
   { "jis_b10",          DeviceForm::FORM_JIS_B10            },
   { "jis_b1",           DeviceForm::FORM_JIS_B1             },
   { "jis_b2",           DeviceForm::FORM_JIS_B2             },
   { "jis_b3",           DeviceForm::FORM_JIS_B3             },
   { "jis_b4",           DeviceForm::FORM_JIS_B4             },
   { "jis_b5",           DeviceForm::FORM_JIS_B5             },
   { "jis_b6",           DeviceForm::FORM_JIS_B6             },
   { "jis_b7",           DeviceForm::FORM_JIS_B7             },
   { "jis_b8",           DeviceForm::FORM_JIS_B8             },
   { "jis_b9",           DeviceForm::FORM_JIS_B9             },
   { "jis_exec",         DeviceForm::FORM_JIS_EXEC           },
   { "jpn_chou2",        DeviceForm::FORM_CHOU2_ENVELOPE     },
   { "jpn_chou3",        DeviceForm::FORM_CHOU3_ENVELOPE     },
   { "jpn_chou4",        DeviceForm::FORM_CHOU4_ENVELOPE     },
   { "jpn_hagaki",       DeviceForm::FORM_HAGAKI_CARD        },
   { "jpn_kahu",         DeviceForm::FORM_KAHU_ENVELOPE      },
   { "jpn_kaku2",        DeviceForm::FORM_KAKU2_ENVELOPE     },
   { "jpn_oufuku",       DeviceForm::FORM_OUFUKU_CARD        },
   { "jpn_you4",         DeviceForm::FORM_YOU4_ENVELOPE      },
   { "na_10x11",         DeviceForm::FORM_10_X_11            },
   { "na_10x13",         DeviceForm::FORM_UNLISTED           },
   { "na_10x14",         DeviceForm::FORM_UNLISTED           },
   { "na_10x15",         DeviceForm::FORM_UNLISTED           },
   { "na_11x12",         DeviceForm::FORM_11_X_12            },
   { "na_11x15",         DeviceForm::FORM_11_X_15            },
   { "na_12x19",         DeviceForm::FORM_12_X_19            },
   { "na_5x7",           DeviceForm::FORM_5_X_7              },
   { "na_6x9",           DeviceForm::FORM_UNLISTED           },
   { "na_7x9",           DeviceForm::FORM_UNLISTED           },
   { "na_9x11",          DeviceForm::FORM_UNLISTED           },
   { "na_a2",            DeviceForm::FORM_A2_ENVELOPE        },
   { "na_arch-a",        DeviceForm::FORM_US_A_ARCHITECTURAL },
   { "na_arch-b",        DeviceForm::FORM_US_B_ARCHITECTURAL },
   { "na_arch-c",        DeviceForm::FORM_US_C_ARCHITECTURAL },
   { "na_arch-d",        DeviceForm::FORM_US_D_ARCHITECTURAL },
   { "na_arch-e",        DeviceForm::FORM_US_E_ARCHITECTURAL },
   { "na_b-plus",        DeviceForm::FORM_B_PLUS             },
   { "na_c5",            DeviceForm::FORM_C5_ENVELOPE        },
   { "na_c",             DeviceForm::FORM_UNLISTED           },
   { "na_d",             DeviceForm::FORM_UNLISTED           },
   { "na_e",             DeviceForm::FORM_UNLISTED           },
   { "na_edp",           DeviceForm::FORM_EDP                },
   { "na_eur-edp",       DeviceForm::FORM_EUROPEAN_EDP       },
   { "na_executive",     DeviceForm::FORM_EXECUTIVE          },
   { "na_f",             DeviceForm::FORM_UNLISTED           },
   { "na_fanfold-eur",   DeviceForm::FORM_EUROPEAN_FANFOLD   },
   { "na_fanfold-us",    DeviceForm::FORM_US_FANFOLD         },
   { "na_foolscap",      DeviceForm::FORM_FOOLSCAP           },
   { "na_govt-legal",    DeviceForm::FORM_GOVERNMENT_LETTER  },
   { "na_govt-letter",   DeviceForm::FORM_GOVERNMENT_LEGAL   },
   { "na_index-3x5",     DeviceForm::FORM_UNLISTED           },
   { "na_index-4x6-ext", DeviceForm::FORM_INDEX_4_X_6_EXT    },
   { "na_index-4x6",     DeviceForm::FORM_UNLISTED           },
   { "na_index-5x8",     DeviceForm::FORM_UNLISTED           },
   { "na_invoice",       DeviceForm::FORM_UNLISTED           },
   { "na_ledger",        DeviceForm::FORM_LEDGER             },
   { "na_legal",         DeviceForm::FORM_LEGAL              },
   { "na_legal-extra",   DeviceForm::FORM_LEGAL_EXTRA        },
   { "na_letter",        DeviceForm::FORM_LETTER             },
   { "na_letter-extra",  DeviceForm::FORM_LETTER_EXTRA       },
   { "na_letter-plus",   DeviceForm::FORM_LETTER_PLUS        },
   { "na_monarch",       DeviceForm::FORM_MONARCH_ENVELOPE   },
   { "na_number-10",     DeviceForm::FORM_NUMBER_10_ENVELOPE },
   { "na_number-11",     DeviceForm::FORM_NUMBER_11_ENVELOPE },
   { "na_number-12",     DeviceForm::FORM_NUMBER_12_ENVELOPE },
   { "na_number-14",     DeviceForm::FORM_NUMBER_14_ENVELOPE },
   { "na_number-9",      DeviceForm::FORM_NUMBER_9_ENVELOPE  },
   { "na_personal",      DeviceForm::FORM_PERSONAL_ENVELOPE  },
   { "na_quarto",        DeviceForm::FORM_QUARTO             },
   { "na_super-a",       DeviceForm::FORM_SUPER_A            },
   { "na_super-b",       DeviceForm::FORM_SUPER_B            },
   { "na_wide-format",   DeviceForm::FORM_WIDE               },
   { "om_dai-pa-kai",    DeviceForm::FORM_DAI_PA_KAI         },
   { "om_folio-sp",      DeviceForm::FORM_FOLIO_SP           },
   { "om_folio",         DeviceForm::FORM_FOLIO              },
   { "om_invite",        DeviceForm::FORM_INVITE_ENVELOPE    },
   { "om_italian",       DeviceForm::FORM_ITALIAN_ENVELOPE   },
   { "om_juuro-ku-kai",  DeviceForm::FORM_JUURO_KU_KAI       },
   { "om_large-photo",   DeviceForm::FORM_UNLISTED           },
   { "om_pa-kai",        DeviceForm::FORM_PA_KAI             },
   { "om_postfix",       DeviceForm::FORM_POSTFIX_ENVELOPE   },
   { "om_small-photo",   DeviceForm::FORM_UNLISTED           },
   { "prc_10",           DeviceForm::FORM_PRC10_ENVELOPE     },
   { "prc_16k",          DeviceForm::FORM_PRC_16K            },
   { "prc_1",            DeviceForm::FORM_PRC1_ENVELOPE      },
   { "prc_2",            DeviceForm::FORM_PRC2_ENVELOPE      },
   { "prc_32k",          DeviceForm::FORM_PRC_32K            },
   { "prc_3",            DeviceForm::FORM_PRC3_ENVELOPE      },
   { "prc_4",            DeviceForm::FORM_PRC4_ENVELOPE      },
   { "prc_5",            DeviceForm::FORM_PRC5_ENVELOPE      },
   { "prc_6",            DeviceForm::FORM_PRC6_ENVELOPE      },
   { "prc_7",            DeviceForm::FORM_PRC7_ENVELOPE      },
   { "prc_8",            DeviceForm::FORM_PRC8_ENVELOPE      },
   { "prc_9",            DeviceForm::FORM_PRC9_ENVELOPE      },
   { "roc_16k",          DeviceForm::FORM_ROC_16K            },
   { "roc_8k",           DeviceForm::FORM_ROC_8K             },
};

#endif

#if defined (USE_TEST_3) || defined (USE_TEST_4)

static FORMMAP vaFormMapping2[] = {
   { "2a0",           DeviceForm::FORM_2A0                },
   { "4a0",           DeviceForm::FORM_4A0                },
   { "a0x3",          DeviceForm::FORM_UNLISTED           },
   { "a0",            DeviceForm::FORM_A0                 },
   { "a10",           DeviceForm::FORM_A10                },
   { "a1",            DeviceForm::FORM_A1                 },
   { "a1x3",          DeviceForm::FORM_UNLISTED           },
   { "a1x4",          DeviceForm::FORM_UNLISTED           },
   { "a2",            DeviceForm::FORM_A2                 },
   { "a2x3",          DeviceForm::FORM_UNLISTED           },
   { "a2x4",          DeviceForm::FORM_UNLISTED           },
   { "a2x5",          DeviceForm::FORM_UNLISTED           },
   { "a3",            DeviceForm::FORM_A3                 },
   { "a3-extra",      DeviceForm::FORM_A3_EXTRA           },
   { "a3x3",          DeviceForm::FORM_UNLISTED           },
   { "a3x4",          DeviceForm::FORM_UNLISTED           },
   { "a3x5",          DeviceForm::FORM_UNLISTED           },
   { "a3x6",          DeviceForm::FORM_UNLISTED           },
   { "a3x7",          DeviceForm::FORM_UNLISTED           },
   { "a4",            DeviceForm::FORM_A4                 },
   { "a4-extra",      DeviceForm::FORM_A4_EXTRA           },
   { "a4-tab",        DeviceForm::FORM_A4_TAB             },
   { "a4x3",          DeviceForm::FORM_UNLISTED           },
   { "a4x4",          DeviceForm::FORM_UNLISTED           },
   { "a4x5",          DeviceForm::FORM_UNLISTED           },
   { "a4x6",          DeviceForm::FORM_UNLISTED           },
   { "a4x7",          DeviceForm::FORM_UNLISTED           },
   { "a4x8",          DeviceForm::FORM_UNLISTED           },
   { "a4x9",          DeviceForm::FORM_UNLISTED           },
   { "a5",            DeviceForm::FORM_A5                 },
   { "a5-extra",      DeviceForm::FORM_A5_EXTRA           },
   { "a6",            DeviceForm::FORM_A6                 },
   { "a7",            DeviceForm::FORM_A7                 },
   { "a8",            DeviceForm::FORM_A8                 },
   { "a9",            DeviceForm::FORM_A9                 },
   { "b0",            DeviceForm::FORM_B0                 },
   { "b10",           DeviceForm::FORM_B10                },
   { "b1",            DeviceForm::FORM_B1                 },
   { "b2",            DeviceForm::FORM_B2                 },
   { "b3",            DeviceForm::FORM_B3                 },
   { "b4",            DeviceForm::FORM_B4                 },
   { "b5",            DeviceForm::FORM_B5                 },
   { "b5-extra",      DeviceForm::FORM_B5_EXTRA           },
   { "b6",            DeviceForm::FORM_B6                 },
   { "b6c4",          DeviceForm::FORM_B6_C4              },
   { "b7",            DeviceForm::FORM_B7                 },
   { "b8",            DeviceForm::FORM_B8                 },
   { "b9",            DeviceForm::FORM_B9                 },
   { "c0",            DeviceForm::FORM_C0                 },
   { "c10",           DeviceForm::FORM_C10                },
   { "c1",            DeviceForm::FORM_C1                 },
   { "c2",            DeviceForm::FORM_C2                 },
   { "c3",            DeviceForm::FORM_C3                 },
   { "c4",            DeviceForm::FORM_C4                 },
   { "c5",            DeviceForm::FORM_C5                 },
   { "c6",            DeviceForm::FORM_C6                 },
   { "c6c5",          DeviceForm::FORM_C6_C5              },
   { "c7",            DeviceForm::FORM_C7                 },
   { "c7c6",          DeviceForm::FORM_C7_C6              },
   { "c8",            DeviceForm::FORM_C8                 },
   { "c9",            DeviceForm::FORM_C9                 },
   { "dl",            DeviceForm::FORM_UNLISTED           },
   { "ra0",           DeviceForm::FORM_RA0                },
   { "ra1",           DeviceForm::FORM_RA1                },
   { "ra2",           DeviceForm::FORM_RA2                },
   { "sra0",          DeviceForm::FORM_SRA0               },
   { "sra1",          DeviceForm::FORM_SRA1               },
   { "sra2",          DeviceForm::FORM_SRA2               },
   { "exec",          DeviceForm::FORM_JIS_EXEC           },
   { "chou2",         DeviceForm::FORM_CHOU2_ENVELOPE     },
   { "chou3",         DeviceForm::FORM_CHOU3_ENVELOPE     },
   { "chou4",         DeviceForm::FORM_CHOU4_ENVELOPE     },
   { "hagaki",        DeviceForm::FORM_HAGAKI_CARD        },
   { "kahu",          DeviceForm::FORM_KAHU_ENVELOPE      },
   { "kaku2",         DeviceForm::FORM_KAKU2_ENVELOPE     },
   { "oufuku",        DeviceForm::FORM_OUFUKU_CARD        },
   { "you4",          DeviceForm::FORM_YOU4_ENVELOPE      },
   { "10x11",         DeviceForm::FORM_10_X_11            },
   { "10x13",         DeviceForm::FORM_UNLISTED           },
   { "10x14",         DeviceForm::FORM_UNLISTED           },
   { "10x15",         DeviceForm::FORM_UNLISTED           },
   { "11x12",         DeviceForm::FORM_11_X_12            },
   { "11x15",         DeviceForm::FORM_11_X_15            },
   { "12x19",         DeviceForm::FORM_12_X_19            },
   { "5x7",           DeviceForm::FORM_5_X_7              },
   { "6x9",           DeviceForm::FORM_UNLISTED           },
   { "7x9",           DeviceForm::FORM_UNLISTED           },
   { "9x11",          DeviceForm::FORM_UNLISTED           },
   { "arch-a",        DeviceForm::FORM_US_A_ARCHITECTURAL },
   { "arch-b",        DeviceForm::FORM_US_B_ARCHITECTURAL },
   { "arch-c",        DeviceForm::FORM_US_C_ARCHITECTURAL },
   { "arch-d",        DeviceForm::FORM_US_D_ARCHITECTURAL },
   { "arch-e",        DeviceForm::FORM_US_E_ARCHITECTURAL },
   { "b-plus",        DeviceForm::FORM_B_PLUS             },
   { "c",             DeviceForm::FORM_UNLISTED           },
   { "d",             DeviceForm::FORM_UNLISTED           },
   { "e",             DeviceForm::FORM_UNLISTED           },
   { "edp",           DeviceForm::FORM_EDP                },
   { "eur-edp",       DeviceForm::FORM_EUROPEAN_EDP       },
   { "executive",     DeviceForm::FORM_EXECUTIVE          },
   { "f",             DeviceForm::FORM_UNLISTED           },
   { "fanfold-eur",   DeviceForm::FORM_EUROPEAN_FANFOLD   },
   { "fanfold-us",    DeviceForm::FORM_US_FANFOLD         },
   { "foolscap",      DeviceForm::FORM_FOOLSCAP           },
   { "govt-legal",    DeviceForm::FORM_GOVERNMENT_LETTER  },
   { "govt-letter",   DeviceForm::FORM_GOVERNMENT_LEGAL   },
   { "index-3x5",     DeviceForm::FORM_UNLISTED           },
   { "index-4x6-ext", DeviceForm::FORM_INDEX_4_X_6_EXT    },
   { "index-4x6",     DeviceForm::FORM_UNLISTED           },
   { "index-5x8",     DeviceForm::FORM_UNLISTED           },
   { "invoice",       DeviceForm::FORM_UNLISTED           },
   { "ledger",        DeviceForm::FORM_LEDGER             },
   { "legal",         DeviceForm::FORM_LEGAL              },
   { "legal-extra",   DeviceForm::FORM_LEGAL_EXTRA        },
   { "letter",        DeviceForm::FORM_LETTER             },
   { "letter-extra",  DeviceForm::FORM_LETTER_EXTRA       },
   { "letter-plus",   DeviceForm::FORM_LETTER_PLUS        },
   { "monarch",       DeviceForm::FORM_MONARCH_ENVELOPE   },
   { "number-10",     DeviceForm::FORM_NUMBER_10_ENVELOPE },
   { "number-11",     DeviceForm::FORM_NUMBER_11_ENVELOPE },
   { "number-12",     DeviceForm::FORM_NUMBER_12_ENVELOPE },
   { "number-14",     DeviceForm::FORM_NUMBER_14_ENVELOPE },
   { "number-9",      DeviceForm::FORM_NUMBER_9_ENVELOPE  },
   { "personal",      DeviceForm::FORM_PERSONAL_ENVELOPE  },
   { "quarto",        DeviceForm::FORM_QUARTO             },
   { "super-a",       DeviceForm::FORM_SUPER_A            },
   { "super-b",       DeviceForm::FORM_SUPER_B            },
   { "wide-format",   DeviceForm::FORM_WIDE               },
   { "dai-pa-kai",    DeviceForm::FORM_DAI_PA_KAI         },
   { "folio-sp",      DeviceForm::FORM_FOLIO_SP           },
   { "folio",         DeviceForm::FORM_FOLIO              },
   { "invite",        DeviceForm::FORM_INVITE_ENVELOPE    },
   { "italian",       DeviceForm::FORM_ITALIAN_ENVELOPE   },
   { "juuro-ku-kai",  DeviceForm::FORM_JUURO_KU_KAI       },
   { "large-photo",   DeviceForm::FORM_UNLISTED           },
   { "pa-kai",        DeviceForm::FORM_PA_KAI             },
   { "postfix",       DeviceForm::FORM_POSTFIX_ENVELOPE   },
   { "small-photo",   DeviceForm::FORM_UNLISTED           },
   { "10",            DeviceForm::FORM_PRC10_ENVELOPE     },
   { "16k",           DeviceForm::FORM_PRC_16K            },
   { "1",             DeviceForm::FORM_PRC1_ENVELOPE      },
   { "2",             DeviceForm::FORM_PRC2_ENVELOPE      },
   { "32k",           DeviceForm::FORM_PRC_32K            },
   { "3",             DeviceForm::FORM_PRC3_ENVELOPE      },
   { "4",             DeviceForm::FORM_PRC4_ENVELOPE      },
   { "5",             DeviceForm::FORM_PRC5_ENVELOPE      },
   { "6",             DeviceForm::FORM_PRC6_ENVELOPE      },
   { "7",             DeviceForm::FORM_PRC7_ENVELOPE      },
   { "8",             DeviceForm::FORM_PRC8_ENVELOPE      },
   { "9",             DeviceForm::FORM_PRC9_ENVELOPE      },
   { "8k",            DeviceForm::FORM_ROC_8K             }
};

#endif

#if defined (USE_TEST_4)

int
comparer (const void* p1, const void* p2)
{
   // Precondition: p1 and p2 are really pointers to integers.
   // Postcondition: The return value is:
   //   (a) negative if *p1 < *p2
   //   (b) zero if *p1 == *p2
   //   (c) positive if *p1 > *p2
   return strcmp (((PFORMMAP)p1)->pszUPDFName, ((PFORMMAP)p2)->pszUPDFName);
}

#endif

#if defined (USE_TEST_6)

static FORMMAP vaResolutionMapping[] = {
   { "60x60",         DeviceResolution::RESOLUTION_60_X_60       },
   { "60x72",         DeviceResolution::RESOLUTION_60_X_72       },
   { "60x180",        DeviceResolution::RESOLUTION_60_X_180      },
   { "60x360",        DeviceResolution::RESOLUTION_60_X_360      },
   { "72x72",         DeviceResolution::RESOLUTION_72_X_72       },
   { "75x75",         DeviceResolution::RESOLUTION_75_X_75       },
   { "80x60",         DeviceResolution::RESOLUTION_80_X_60       },
   { "80x72",         DeviceResolution::RESOLUTION_80_X_72       },
   { "90x60",         DeviceResolution::RESOLUTION_90_X_60       },
   { "90x72",         DeviceResolution::RESOLUTION_90_X_72       },
   { "90x90",         DeviceResolution::RESOLUTION_90_X_90       },
   { "90x180",        DeviceResolution::RESOLUTION_90_X_180      },
   { "90x360",        DeviceResolution::RESOLUTION_90_X_360      },
   { "100x100",       DeviceResolution::RESOLUTION_100_X_100     },
   { "120x60",        DeviceResolution::RESOLUTION_120_X_60      },
   { "120x72",        DeviceResolution::RESOLUTION_120_X_72      },
   { "120x144",       DeviceResolution::RESOLUTION_120_X_144     },
   { "120x180",       DeviceResolution::RESOLUTION_120_X_180     },
   { "120x360",       DeviceResolution::RESOLUTION_120_X_360     },
   { "144x72",        DeviceResolution::RESOLUTION_144_X_72      },
   { "150x150",       DeviceResolution::RESOLUTION_150_X_150     },
   { "180x180",       DeviceResolution::RESOLUTION_180_X_180     },
   { "180x360",       DeviceResolution::RESOLUTION_180_X_360     },
   { "200x200",       DeviceResolution::RESOLUTION_200_X_200     },
   { "240x60",        DeviceResolution::RESOLUTION_240_X_60      },
   { "240x72",        DeviceResolution::RESOLUTION_240_X_72      },
   { "240x144",       DeviceResolution::RESOLUTION_240_X_144     },
   { "240x240",       DeviceResolution::RESOLUTION_240_X_240     },
   { "300x300",       DeviceResolution::RESOLUTION_300_X_300     },
   { "360x180",       DeviceResolution::RESOLUTION_360_X_180     },
   { "360x360",       DeviceResolution::RESOLUTION_360_X_360     },
   { "360x720",       DeviceResolution::RESOLUTION_360_X_720     },
   { "400x400",       DeviceResolution::RESOLUTION_400_X_400     },
   { "600x300",       DeviceResolution::RESOLUTION_600_X_300     },
   { "600x600",       DeviceResolution::RESOLUTION_600_X_600     },
   { "720x360",       DeviceResolution::RESOLUTION_720_X_360     },
   { "720x720",       DeviceResolution::RESOLUTION_720_X_720     },
   { "1200x600",      DeviceResolution::RESOLUTION_1200_X_600    },
   { "1200x1200",     DeviceResolution::RESOLUTION_1200_X_1200   },
   { "1440x720",      DeviceResolution::RESOLUTION_1440_X_720    },
   { "2880x720",      DeviceResolution::RESOLUTION_2880_X_720    },
   { "draft",         DeviceResolution::RESOLUTION_DRAFT         },
   { "fine",          DeviceResolution::RESOLUTION_FINE          },
   { "group-3",       DeviceResolution::RESOLUTION_GROUP_3       },
   { "group-4",       DeviceResolution::RESOLUTION_GROUP_4       },
   { "high",          DeviceResolution::RESOLUTION_HIGH          },
   { "low",           DeviceResolution::RESOLUTION_LOW           },
   { "medium",        DeviceResolution::RESOLUTION_MEDIUM        },
   { "none",          DeviceResolution::RESOLUTION_NONE          },
   { "normal",        DeviceResolution::RESOLUTION_NORMAL        },
   { "photo-quality", DeviceResolution::RESOLUTION_PHOTO_QUALITY },
   { "presentation",  DeviceResolution::RESOLUTION_PRESENTATION  }
};

#endif

typedef struct _OrientationMap {
   int   iOmniId;
   char *pszStandardName;
} ORIENTATIONMAP, *PORIENTATIONMAP;

#if defined (USE_TEST_7)

static ORIENTATIONMAP vaOrientationMapping[] = {
   { DeviceOrientation::ORIENTATION_UNLISTED,          "unlisted"          },
   { DeviceOrientation::ORIENTATION_NONE,              "none"              },
   { DeviceOrientation::ORIENTATION_PORTRAIT,          "portrait"          },
   { DeviceOrientation::ORIENTATION_LANDSCAPE,         "landscape"         },
   { DeviceOrientation::ORIENTATION_REVERSE_PORTRAIT,  "reverse_portrait"  },
   { DeviceOrientation::ORIENTATION_REVERSE_LANDSCAPE, "reverse_landscape" }
};

#endif

int
main (int argc, char *argv[])
{
   Omni::initialize ();

#ifdef USE_TEST_1

   for (register int i = 0; i < (int)dimof (aiOmniIDs); i++)
   {
      cout << DeviceForm::IDToName (aiOmniIDs[i]) << endl;

      if (aiOmniIDs[i] != DeviceForm::nameToID (DeviceForm::IDToName (aiOmniIDs[i])))
      {
         cout << "Error " << DeviceForm::IDToName (aiOmniIDs[i]) << endl;
      }
   }

#endif

#ifdef USE_TEST_2

   for (register int i = 0; i < (int)dimof (aiOmniIDs); i++)
   {
      bool fFound = false;

      for (register int j = 0; j < (int)dimof (vaFormMapping1); j++)
      {
         if (vaFormMapping1[j].iOmniFormID == aiOmniIDs[i])
         {
            cout << "               \"" << vaFormMapping1[j].pszUPDFName << "\"," << endl;
            fFound = true;
         }
      }

      if (!fFound)
      {
         cout << "/* @MISSING */ \"" << DeviceForm::IDToName (aiOmniIDs[i]) << "\"," << endl;
      }
   }

#endif

#ifdef USE_TEST_3

   if (2 == argc)
   {
      int id = DeviceForm::nameToID (argv[1]);

      cout << argv[1]
           << " = ("
           << id
           << ") "
           << DeviceForm::IDToName (id)
           << endl;

      return 0;
   }

   bool fFailed = false;

   for (register int i = 0; i < (int)dimof (vaFormMapping1); i++)
   {
      char *pszName = vaFormMapping1[i].pszUPDFName;
      int   id      = DeviceForm::nameToID (pszName);

      if (id != vaFormMapping1[i].iOmniFormID)
      {
         fFailed = true;

         cout << "Failed1! "
              << pszName
              << " = ("
              << id
              << ") "
              << DeviceForm::IDToName (id)
              << endl;
      }
   }

   if (!fFailed)
   {
      cout << "Success!" << endl;
   }

   fFailed = false;
   for (register int i = 0; i < (int)dimof (vaFormMapping2); i++)
   {
      char *pszName = vaFormMapping2[i].pszUPDFName;
      int   id      = DeviceForm::nameToID (pszName);

      if (id != vaFormMapping2[i].iOmniFormID)
      {
         fFailed = true;

         cout << "Failed2! "
              << pszName
              << " = ("
              << id
              << ") "
              << DeviceForm::IDToName (id)
              << endl;
      }
   }

   if (!fFailed)
   {
      cout << "Success!" << endl;
   }

#endif

#ifdef USE_TEST_4

   qsort (vaFormMapping2, dimof (vaFormMapping2), sizeof (vaFormMapping2[0]), comparer);

   for (register int i = 0; i < (int)dimof (vaFormMapping2); i++)
   {
      cout << vaFormMapping2[i].pszUPDFName << endl;
   }

#endif

#ifdef USE_TEST_5

   if (2 == argc)
   {
      int id = DeviceResolution::nameToID (argv[1]);

      cout << argv[1]
           << " = ("
           << id
           << ") "
           << DeviceResolution::IDToName (id)
           << endl;

      return 0;
   }

#endif

#ifdef USE_TEST_6

   for (register int i = 0; i < (int)dimof (vaResolutionMapping); i++)
   {
      if (vaResolutionMapping[i].iOmniFormID != DeviceResolution::nameToID (vaResolutionMapping[i].pszUPDFName))
      {
         cout << "Error " << vaResolutionMapping[i].pszUPDFName << endl;
      }
   }

#endif

#ifdef USE_TEST_7

   for (register int i = 0; i < (int)dimof (vaOrientationMapping); i++)
   {
      if (vaOrientationMapping[i].iOmniId != DeviceOrientation::nameToID (vaOrientationMapping[i].pszStandardName))
      {
         cout << "Error " << vaOrientationMapping[i].pszStandardName << endl;
      }
   }

#endif

   Omni::terminate ();

   return 0;
}
