/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceTray.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

DeviceTray::
DeviceTray (Device     *pDevice,
            int         id,
            int         iType,
            BinaryData *pbdData)
{
   pDevice_d = pDevice;
   id_d      = id;
   iType_d   = iType;
   pbdData_d = pbdData;
}

DeviceTray::
~DeviceTray ()
{
   delete pbdData_d;

   pDevice_d = 0;
   id_d      = 0;
   iType_d   = 0;
   pbdData_d = 0;
}

char * DeviceTray::
getName (Device *pDevice, int id)
{
   return StringResource::getString (pDevice->getLanguageResource (),
                                     StringResource::STRINGGROUP_TRAYS,
                                     id);
}

char * DeviceTray::
getName ()
{
   return StringResource::getString (pDevice_d->getLanguageResource (),
                                     StringResource::STRINGGROUP_TRAYS,
                                     id_d);
}

bool DeviceTray::
isID (int id)
{
   return id == id_d;
}

int DeviceTray::
getID ()
{
   return id_d;
}

char * DeviceTray::
getDeviceID ()
{
   return 0;
}

int DeviceTray::
getType ()
{
   return iType_d;
}

BinaryData * DeviceTray::
getData ()
{
   return pbdData_d;
}

DeviceTray * DeviceTray::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

#define STRING_OF(x) (apszTrayNames[x - TRAY_UNLISTED])

static char *apszTrayNames[] = {
   "TRAY_UNLISTED",
   "TRAY_NONE",
   "TRAY_AUTO",
   "TRAY_USE_PRINTER_SETTING",
   "TRAY_AUTO_FEEDER",
   "TRAY_AUTOSWITCH",
   "TRAY_TRAY",
   "TRAY_UPPER_TRAY",
   "TRAY_LOWER_TRAY",
   "TRAY_MULTI_TRAY",
   "TRAY_CASSETTE",
   "TRAY_UPPER_CASSETTE",
   "TRAY_LOWER_CASSETTE",
   "TRAY_MULTI_CASSETTE",
   "TRAY_OPTION_CASSETTE",
   "TRAY_MANUAL_FEEDER",
   "TRAY_MANUAL_ENVELOPE",
   "TRAY_PAPER_FEEDER",
   "TRAY_ENVELOPE_FEEDER",
   "TRAY_CSF",
   "TRAY_FRONT_CONTINUOUS",
   "TRAY_REAR_CONTINUOUS",
   "TRAY_SINGLE_SHEET",
   "TRAY_SHEET_FEEDER",
   "TRAY_BIN_1",
   "TRAY_BIN_2",
   "TRAY_PORTABLE_SHEET",
   "TRAY_TRAY_1",
   "TRAY_TRAY_2",
   "TRAY_TRAY_3",
   "TRAY_TRAY_4",
   "TRAY_TRAY_5",
   "TRAY_TRAY_6",
   "TRAY_CASSETTE1",
   "TRAY_CASSETTE2",
   "TRAY_CASSETTE3",
   "TRAY_CASSETTE4",
   "TRAY_CASSETTE5",
   "TRAY_FRONT_TRAY",
   "TRAY_OPTION_MULTI_SF",
   "TRAY_MULTI_FEEDER",
   "TRAY_PAPER_DECK",
   "TRAY_TRACTOR_UNIT",
   "TRAY_ROLL_1",
   "TRAY_ROLL_2",
   "TRAY_ROLL_3",
   "TRAY_ROLL_4",
   "TRAY_ROLL_5",
   "TRAY_AUXILIARY_TRAY",
   "TRAY_LARGE_CAPACITY_TRAY",
   "TRAY_HIGH_CAPACITY_FEEDER",
   "TRAY_ZERO_MARGINS"
};

int DeviceTray::
nameToID (char *pszId)
{
   static short int asiEntries[] = {
      TRAY_AUTO,
      TRAY_AUTOSWITCH,
      TRAY_AUTO_FEEDER,
      TRAY_AUXILIARY_TRAY,
      TRAY_BIN_1,
      TRAY_BIN_2,
      TRAY_CASSETTE,
      TRAY_CASSETTE1,
      TRAY_CASSETTE2,
      TRAY_CASSETTE3,
      TRAY_CASSETTE4,
      TRAY_CASSETTE5,
      TRAY_CSF,
      TRAY_ENVELOPE_FEEDER,
      TRAY_FRONT_CONTINUOUS,
      TRAY_FRONT_TRAY,
      TRAY_HIGH_CAPACITY_FEEDER,
      TRAY_LARGE_CAPACITY_TRAY,
      TRAY_LOWER_CASSETTE,
      TRAY_LOWER_TRAY,
      TRAY_MANUAL_ENVELOPE,
      TRAY_MANUAL_FEEDER,
      TRAY_MULTI_CASSETTE,
      TRAY_MULTI_FEEDER,
      TRAY_MULTI_TRAY,
      TRAY_NONE,
      TRAY_OPTION_CASSETTE,
      TRAY_OPTION_MULTI_SF,
      TRAY_PAPER_DECK,
      TRAY_PAPER_FEEDER,
      TRAY_PORTABLE_SHEET,
      TRAY_REAR_CONTINUOUS,
      TRAY_ROLL_1,
      TRAY_ROLL_2,
      TRAY_ROLL_3,
      TRAY_ROLL_4,
      TRAY_ROLL_5,
      TRAY_SHEET_FEEDER,
      TRAY_SINGLE_SHEET,
      TRAY_TRACTOR_UNIT,
      TRAY_TRAY,
      TRAY_TRAY_1,
      TRAY_TRAY_2,
      TRAY_TRAY_3,
      TRAY_TRAY_4,
      TRAY_TRAY_5,
      TRAY_TRAY_6,
      TRAY_UPPER_CASSETTE,
      TRAY_UPPER_TRAY,
      TRAY_USE_PRINTER_SETTING,
      TRAY_ZERO_MARGINS
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return TRAY_UNLISTED;
}

char * DeviceTray::
IDToName (int id)
{
   id -= TRAY_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszTrayNames)
      )
      return apszTrayNames[id];
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + TRAY_UNLISTED);

      return achUnknown;
   }
}

typedef struct _ReservedMap {
   const char *pszName;
   int         iValue;
} RESERVEDMAP, *PRESERVEDMAP;

static const RESERVEDMAP vaReservedKeywords[] = {
   { "TRAY_TYPE_AUTO",         0 },
   { "TRAY_TYPE_MANUAL",       1 },
   { "TRAY_TYPE_ZERO_MARGINS", 2 }
};

bool DeviceTray::
isReservedKeyword (const char *pszId)
{
   for (int i = 0; i < (int)dimof (vaReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, vaReservedKeywords[i].pszName))
         return true;
   }

   return false;
}

int DeviceTray::
getReservedValue (const char *pszId)
{
   for (int i = 0; i < (int)dimof (vaReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, vaReservedKeywords[i].pszName))
         return vaReservedKeywords[i].iValue;
   }

   return 0;
}

string DeviceTray::
toString (ostringstream& oss)
{
   oss << "{"
       << STRING_OF (id_d)
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceTray& const_self)
{
   DeviceTray&   self = const_cast<DeviceTray&>(const_self);
   ostringstream oss;

   os << self.toString (oss);

   return os;
}
