/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * make DitherLibraryExample.o
 * cc -shared -Wl,-soname,libDitherLibraryExample.so -o libDitherLibraryExample.so DitherLibraryExample.o
 *
 * in /etc/omni
 * should be a line:
 * dither DITHER_EXAMPLE DitherLibraryExample
 * dither DITHER_CATAGORY_EXAMPLE DitherLibraryExample
 */
#include "Device.hpp"

class DitherLibraryExample : public DeviceDither
{
public:
   enum {
      DITHER_UNLISTED = -1,
      DITHER_EXAMPLE
   };
   enum {
      DITHER_CATAGORY_UNLISTED = -1,
      DITHER_CATAGORY_EXAMPLE
   };

                               DitherLibraryExample           (Device      *pDevice,
                                                               int          iDitherType);
   virtual                    ~DitherLibraryExample           ();

   static DeviceDither        *createDitherInstance           (char        *pszDitherType,
                                                               Device      *pDevice,
                                                               char        *pszOptions);

   void                        ditherRGBtoCMYK                (PBITMAPINFO2 pbmi2,
                                                               PBYTE        pbStart);

   bool                        ditherAllPlanesBlank           ();
   bool                        ditherCPlaneBlank              ();
   bool                        ditherMPlaneBlank              ();
   bool                        ditherYPlaneBlank              ();
   bool                        ditherKPlaneBlank              ();
   bool                        ditherLCPlaneBlank             ();
   bool                        ditherLMPlaneBlank             ();

   void                        newFrame                       ();

   BinaryData                 *getCPlane                      ();
   BinaryData                 *getMPlane                      ();
   BinaryData                 *getYPlane                      ();
   BinaryData                 *getKPlane                      ();
   BinaryData                 *getLCPlane                     ();
   BinaryData                 *getLMPlane                     ();

   char                       *getID                          ();

   static bool                 ditherNameValid                (char        *pszId);
   static bool                 ditherCatagoryValid            (char        *pszId);
   static char                *getDitherCatagory              (char        *pszId);
   static char                *getDitherName                  (char        *pszId);

   static Enumeration         *getEnumeration                 ();

private:
   static int                  nameToID                       (char        *pszId);

   int iDitherType_d;
};

#define STRING_OF(x) (pszDitherNames[x - DITHER_EXAMPLE])

static char *pszDitherNames[] = {
   "DITHER_EXAMPLE"
};

int DitherLibraryExample::
nameToID (char *pszId)
{
   static short int asiEntries[] = {
      DITHER_EXAMPLE
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return DITHER_UNLISTED;
}

DeviceDither * DitherLibraryExample::
createDitherInstance (char        *pszDitherType,
                      Device      *pDevice,
                      char        *pszOptions)
{
   DebugOutput::getErrorStream () << "DitherLibraryExample called with " << pszDitherType << ", " << pszOptions << endl;

   if (!ditherNameValid (pszDitherType))
   {
      // Error!
      return 0;
   }

   // Parse the options that were passed into pszOptions

   return new DitherLibraryExample (pDevice, nameToID (pszDitherType));
}

DitherLibraryExample::
DitherLibraryExample (Device *pDevice,
                      int     iDitherType)
   : DeviceDither (pDevice)
{
   iDitherType_d = iDitherType;
}

DitherLibraryExample::
~DitherLibraryExample ()
{
}

char * DitherLibraryExample::
getID ()
{
   return pszDitherNames[iDitherType_d];
}

void DitherLibraryExample::
ditherRGBtoCMYK (PBITMAPINFO2 pbmi2,
                 PBYTE        pbStart)
{
}

bool DitherLibraryExample::
ditherAllPlanesBlank ()
{
   return 0;
}

bool DitherLibraryExample::
ditherCPlaneBlank ()
{
   return 0;
}

bool DitherLibraryExample::
ditherMPlaneBlank ()
{
   return 0;
}

bool DitherLibraryExample::
ditherYPlaneBlank ()
{
   return 0;
}

bool DitherLibraryExample::
ditherKPlaneBlank ()
{
   return 0;
}

bool DitherLibraryExample::
ditherLCPlaneBlank ()
{
   return 0;
}

bool DitherLibraryExample::
ditherLMPlaneBlank ()
{
   return 0;
}

void DitherLibraryExample::
newFrame ()
{
}

BinaryData * DitherLibraryExample::
getCPlane ()
{
   return 0;
}

BinaryData * DitherLibraryExample::
getMPlane ()
{
   return 0;
}

BinaryData * DitherLibraryExample::
getYPlane ()
{
   return 0;
}

BinaryData * DitherLibraryExample::
getKPlane ()
{
   return 0;
}

BinaryData * DitherLibraryExample::
getLCPlane ()
{
   return 0;
}

BinaryData * DitherLibraryExample::
getLMPlane ()
{
   return 0;
}

bool DitherLibraryExample::
ditherNameValid (char *pszId)
{
   static short int asiEntries[] = {
      DITHER_EXAMPLE
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return true;
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return false;
}

char * DitherLibraryExample::
getDitherCatagory (char *pszId)
{
   if (0 == strcmp ("DITHER_EXAMPLE", pszId))
      return "DITHER_CATAGORY_EXAMPLE";

   return 0;
}

char * DitherLibraryExample::
getDitherName (char *pszId)
{
   // @TBD - determine language
   if (0 == strcmp ("DITHER_EXAMPLE", pszId))
      return "An example dithering routine";

   return 0;
}

bool DitherLibraryExample::
ditherCatagoryValid (char *pszId)
{
   if (0 == strcmp ("DITHER_CATAGORY_EXAMPLE", pszId))
      return true;

   return false;
}

class DitherEnumerator : public Enumeration
{
public:
   DitherEnumerator (int cDithers, char *aDithers[])
   {
      iDither_d  = 0;
      cDithers_d = cDithers;
      aDithers_d = aDithers;
   }

   virtual bool hasMoreElements ()
   {
      if (iDither_d < cDithers_d)
         return true;
      else
         return false;
   }

   virtual void *nextElement ()
   {
      if (iDither_d > cDithers_d - 1)
         return 0;

      return aDithers_d[iDither_d++];
   }

private:
   int    iDither_d;
   int    cDithers_d;
   char **aDithers_d;
};

Enumeration * DitherLibraryExample::
getEnumeration ()
{
   return new DitherEnumerator (dimof (pszDitherNames), pszDitherNames);
}

extern "C"
{
DeviceDither        *createDitherInstance           (char        *pszDitherType,
                                                     Device      *pDevice,
                                                     char        *pszOptions);
bool                 ditherNameValid                (char        *pszId);
bool                 ditherCatagoryValid            (char        *pszId);
char                *getDitherCatagory              (char        *pszId);
char                *getDitherName                  (char        *pszId);
Enumeration         *getEnumeration                 ();

DeviceDither *
createDitherInstance (char        *pszDitherType,
                      Device      *pDevice,
                      char        *pszOptions)
{
   return DitherLibraryExample::createDitherInstance (pszDitherType, pDevice, pszOptions);
}

bool
ditherNameValid (char *pszId)
{
   return DitherLibraryExample::ditherNameValid (pszId);
}

bool
ditherCatagoryValid (char *pszId)
{
   return DitherLibraryExample::ditherCatagoryValid (pszId);
}

char *
getDitherCatagory (char *pszId)
{
   return DitherLibraryExample::getDitherCatagory (pszId);
}

char *
getDitherName (char *pszId)
{
   // @TBD - determine language
   return DitherLibraryExample::getDitherName (pszId);
}

Enumeration *
getEnumeration ()
{
   return DitherLibraryExample::getEnumeration ();
}
}
