/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _COM_IBM_LINUX_OMNI_JOB_DriverProperty
#define _COM_IBM_LINUX_OMNI_JOB_DriverProperty

#include <vector>
#include <map>
#include <string>
#include <sstream>

class DriverProperty;

#include "Driver.hpp"

using namespace std;

class DriverProperty
{
public:
                  DriverProperty            (string                propertyName,
                                             string                nameForDisplay,
                                             string                widgetName,
                                             vector<string>        externalValues,
                                             vector<string>        internalValues,
                                             bool                  fEditable = true);
                  DriverProperty            (const DriverProperty& dp);
                  DriverProperty            ();
   virtual       ~DriverProperty            ();

                  DriverProperty operator = (DriverProperty        dp);

   /**************************/
   // @INVESTIGATE COPY CONSTRUCTOR needed for deep copy
   /*************************/

   bool           isEditable                ();

   string         getName                   ();
   string         getPropertyNameForDisplay ();
   string         getWidgetName             ();
   string         getDefaultExternalValue   ();
   string         getDefaultInternalValue   ();

   vector<string> getAvailableValues        ();

   bool           setDefaultValue           (string                defaultValue);

   string         toString                  (ostringstream&        oss);

protected:
   string         propertyName_d;
   string         nameForDisplay_d;
   string         widgetName_d;
   string         externalValue_d;
   bool           fEditable_d;
   vector<string> externalValues_d;
   vector<string> internalValues_d;
};

#endif
