/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <iostream>
#include <memory>
#include <string>

#include "OmniGSInterface.hpp"
#include "Omni.hpp"

using namespace std;

void
printUsage (char *pszExeName)
{
   DebugOutput::getErrorStream () << "Usage: " << pszExeName << " required_parms optional_parms" << endl
      << "   where required_parms are:" << endl
      << "      driver driver_name" << endl
      << "   where optional_parms are:" << endl
      << "      UPDFDeviceName device_name" << endl
      << "      orientation orientation_id" << endl
      << "      form form_id" << endl
      << "      tray tray_id" << endl
      << "      media media_id" << endl
      << "      resolution resolution_id" << endl
      << "      dithering dithering_id" << endl
      << "      printmode printmode_id" << endl
      << "      OtherJobPropertyKey value" << endl;
}

int
main (int argc, char *argv[])
{
   string  stringJobProp    = "";
   char   *pszDriverName    = 0;
   char   *pszJobProperties = 0;
   int     rc               = 0;
   int     rc2;

   Omni::initialize ();

   for (int i = 1; i < argc; i++)
   {
      if (0 == strcmp ("UPDFDeviceName", argv[i]))
      {
         i++;

         if (3 > argc - i)
         {
            cerr << "Error: not enough parameters." << endl;
            cerr << "Error: Must be in form of: UPDFDeviceName begin-quote ... end-quote." << endl;

            rc = 1;
            goto BUGOUT;
         }
         if (0 != strcmp ("begin-quote", argv[i]))
         {
            cerr << "Error: Missing begin-quote keyword." << endl;
            cerr << "Error: Must be in form of: UPDFDeviceName begin-quote ... end-quote." << endl;

            rc = 2;
            goto BUGOUT;
         }

         i++;

         if (stringJobProp.c_str ()[0])
            stringJobProp += " ";
         stringJobProp += "UPDFDeviceName=\"";

         bool fFirst = true;

         while (  1 <= argc - i
               && 0 != strcmp ("end-quote", argv[i])
               )
         {
            if (!fFirst)
            {
               stringJobProp += " ";
            }
            else
            {
               fFirst = false;
            }

            stringJobProp += argv[i];
            i++;
         }

         if (  1 > argc - i
            || 0 != strcmp ("end-quote", argv[i])
            )
         {
            cerr << "Error: Missing end-quote." << endl;
            cerr << "Error: Must be in form of: UPDFDeviceName begin-quote ... end-quote." << endl;

            rc = 3;
            goto BUGOUT;
         }

         stringJobProp += "\"";
      }
      else if (0 == strcmp ("driver", argv[i]))
      {
         pszDriverName = argv[i + 1];

         i++;
      }
      else
      {
         // Assume that it is a Job Property (where there needs to be at least
         // 2 parameters)
         if (2 <= argc - i)
         {
            bool fSpaceFound = false;

            if (strchr (argv[i + 1], ' '))
            {
               fSpaceFound = true;
            }

            if (stringJobProp.c_str ()[0])
               stringJobProp += " ";
            stringJobProp += argv[i];
            stringJobProp += "=";
            if (fSpaceFound)
               stringJobProp += "\"";
            stringJobProp += argv[i + 1];
            if (fSpaceFound)
               stringJobProp += "\"";

            i++;
         }
         else
         {
            printUsage (argv[0]);

            rc = 4;
            goto BUGOUT;
         }
      }
   }

   pszJobProperties = (char *)stringJobProp.c_str ();

   rc2 = GetOmniJobProperties (pszDriverName, &pszJobProperties);

   cout << "rc2 = " << rc2 << ", pszJobProperties = \"" << pszJobProperties << "\"" << endl;

   rc = 0;

BUGOUT:
   // Clean up
   if (pszJobProperties)
   {
      free (pszJobProperties);
   }

   Omni::terminate ();

   return rc;
}
