/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "JobProperties.hpp"

#include <string.h>
#include <stdlib.h>

JobProperties::
JobProperties (char *pszJobProperties)
{
   pszJobProperties_d   = 0;
   pszCurrentProperty_d = 0;
   pszCurrentKey_d      = 0;
   pszCurrentValue_d    = 0;

   if (  pszJobProperties
      && *pszJobProperties
      )
   {
      pszJobProperties_d = (char *)malloc (strlen (pszJobProperties) + 1);

      if (pszJobProperties_d)
      {
         strcpy (pszJobProperties_d, pszJobProperties);
         pszCurrentProperty_d = pszJobProperties_d;
      }
   }

   readNextKey (0);
}

JobProperties::
~JobProperties ()
{
   if (pszJobProperties_d)
   {
      free (pszJobProperties_d);
      pszJobProperties_d = 0;
   }
}

bool JobProperties::
hasKeyNameLeft ()
{
   if (  pszCurrentKey_d
      && pszCurrentValue_d
      )
   {
      return true;
   }
   else
   {
      return false;
   }
}

bool JobProperties::
hasKeyNameLeft (char *pszKey)
{
   if (  pszKey
      && *pszKey
      && pszCurrentKey_d
      && *pszCurrentKey_d
      && 0 != strcmp (pszCurrentKey_d, pszKey)
      )
   {
      getNextKey (pszKey);
   }

   return hasKeyNameLeft ();
}

bool JobProperties::
getNextKey ()
{
   readNextKey (0);

   return hasKeyNameLeft ();
}

bool JobProperties::
getNextKey (char *pszKey)
{
   readNextKey (pszKey);

   return hasKeyNameLeft ();
}

char * JobProperties::
getCurrentKey ()
{
   return pszCurrentKey_d;
}

char * JobProperties::
getCurrentValue ()
{
   return pszCurrentValue_d;
}

void JobProperties::
readNextKey (char *pszKey)
{
   pszCurrentKey_d   = 0;
   pszCurrentValue_d = 0;

   if (  pszCurrentProperty_d
      && *pszCurrentProperty_d
      )
   {
      char *pszSep    = pszCurrentProperty_d;
      char *pszEquals = 0;

      // Skip white space
      while (  ' '  == *pszSep
            || '\t' == *pszSep
            || '\r' == *pszSep
            || '\n' == *pszSep
            )
         *pszSep++ = '\0';

#ifndef RETAIL
//////if (DebugOutput::shouldOutputPrintDevice ()) DebugOutput::getErrorStream () << "PrintDevice::" << __FUNCTION__ << ": found " << pszJobProperties << endl;
#endif

      pszEquals = strpbrk (pszSep, "=");

      if (pszEquals)
      {
         *pszEquals = '\0';
         pszCurrentKey_d   = pszSep;
         pszCurrentValue_d = pszEquals + 1;

         if ('\"' == *pszCurrentValue_d)
         {
            pszCurrentValue_d++;

            pszSep = pszCurrentValue_d;
            while (  *pszSep
                  && '\"' != *pszSep
                  )
            {
               pszSep++;
            }

            if ('\"' == *pszSep)
            {
               *pszSep++ = '\0';
            }
            else
            {
               // Error
               pszSep            = 0;
               pszCurrentKey_d   = 0;
               pszCurrentValue_d = 0;
            }
         }
         else
         {
            pszSep = strpbrk (pszCurrentValue_d, " \n\r\t");

            if (pszSep)
            {
               *pszSep++ = '\0';
            }
         }

#ifndef RETAIL
/////////if (DebugOutput::shouldOutputPrintDevice ())
/////////{
/////////   DebugOutput::getErrorStream () << "PrintDevice::" << __FUNCTION__ << ": LHS = " << pszCurrentKey_d << endl;
/////////   DebugOutput::getErrorStream () << "PrintDevice::" << __FUNCTION__ << ": RHS = " << pszCurrentValue_d << endl;
/////////}
#endif

      }

      if (  pszSep
         && *pszSep
         )
         pszCurrentProperty_d = pszSep;
      else
         pszCurrentProperty_d = 0;
   }
}
