/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2001
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _Okidata_PCL_Instance
#define _Okidata_PCL_Instance

#include "defines.hpp"
#include "Device.hpp"

extern "C" {
   DeviceInstance *createInstance (PrintDevice    *pDevice);
   void            deleteInstance (DeviceInstance *pInstance);
};

class Okidata_PCL_Instance : public DeviceInstance
{
public:
   enum {
      DO1_AUTO_ROTATE              = 0x00000001,
      DO1_SCALABLE_FONT            = 0x00000002,
      DO1_SUPPORT_DUPLEX           = 0x00000004,
      DO1_SUPPORT_RULES            = 0x00000008,
      DO1_SUPPORT_DOT_COORDS       = 0x00000010,
      DO1_SUPPORT_PJL              = 0x00000020,
      DO1_SUPPORT_BACKBIN          = 0x00000040,
      DO1_SUPPORT_JOBOFFSET        = 0x00000080,
      DO1_SUPPORT_PATTERNS         = 0x00000100,
      DO1_SUPPORT_PCL5             = 0x00000200,
      DO1_MIXED_TRAY               = 0x00000400,
      DO1_SUPPORT_COLOR            = 0x00000800,
      DO1_SUPPORT_PS               = 0x00001000,
      DO1_SUPPORT_PAGEPROT_ESC     = 0x00002000,
      DO1_SUPPORT_ES               = 0x00004000,
      DO1_SUPPORT_HPGL2            = 0x00008000,
      DO1_SUPPORT_HPGL2_MC         = 0x00010000,
      DO1_PAGEPROT_AUTO            = 0x00020000,
      DO1_SUPPORT_COMPRESS5        = 0x00040000,
      DO1_SUPPORT_COLORSMART       = 0x00080000,
      DO1_SUPPORT_HPGL_DEFAULT     = 0x00100000,
      DO1_SUPPORT_OUTPUTOPTION     = 0x00200000,
      DO1_PRINTER_DISPLAY          = 0x00400000,
      DO1_SUPPORT_COLOR_RET        = 0x00800000,
      DO1_SUPPORT_HGRAYSCALE       = 0x01000000,
      DO1_SUPPORT_WATERMARK        = 0x02000000,
      DO1_SUPPORT_HPGL2MACRO       = 0x04000000,
      DO1_SUPPORT_PAN_SUPER        = 0x08000000,
      DO1_SUPPORT_CONFIG           = 0x10000000,
      DO1_SUPPORT_HPGL2LOW         = 0x20000000,
      DO1_SUPPORT_BIDI             = 0x40000000,
      DO1_SUPPORT_STAPLER          = 0x80000000
   };
   enum {
      DO2_SUPPORT_PCLXL            = 0x00000001,
      DO2_SUPPORT_AUTOSELECT       = 0x00000002,
      DO2_SUPPORT_PCLXL_DEFAULT    = 0x00000004,
      DO2_SUPPORT_PCLXL_1200       = 0x00000008,
      DO2_SUPPORT_LARGECLRFMT      = 0x00000010,
      DO2_SUPPORT_2STAPLES         = 0x00000020,
      DO2_SUPPORT_PCLXL_600        = 0x00000040,
      DO2_SUPPORT_NEWCLR           = 0x00000080,
      DO2_SUPPORT_PJLOUTPUTBIN     = 0x00000100,
      DO2_SUPPORT_COLLATOR         = 0x00000200,
      DO2_SUPPORT_MET              = 0x00000400,
      DO2_SUPPORT_MOPIER           = 0x00000800,
      DO2_SUPPORT_MOPIER_DEFAULT   = 0x00001000,
      DO2_SUPPORT_EUROCHAR         = 0x00002000,
      DO2_SUPPORT_JOB_RETENTION    = 0x00004000,
      DO2_SUPPORT_PCLXL_1200R      = 0x00008000,
      DO2_SUPPORT_FAX              = 0x00010000,
      DO2_SUPPORT_SORTER           = 0x00020000,
      DO2_SUPPORT_COLOR_OEM        = 0x00040000,
      DO2_SUPPORT_COLLATE          = 0x00080000,
      DO2_SUPPORT_QUIETRESET       = 0x00100000,
      DO2_SUPPORT_EDGEPRINTING     = 0x00200000,
      DO2_SUPPORT_OPTIMIZEPQ       = 0x00400000,
      DO2_SUPPORT_COLORSMART2      = 0x00800000,
      DO2_SUPPORT_MANUALDUPLEX     = 0x01000000,
      DO2_SUPPORT_NOFNTMGR         = 0x02000000,
      DO2_SUPPORT_PRINTERSELECTDEF = 0x04000000,
      DO2_SUPPORT_PCL6_REV20       = 0x08000000,
      DO2_SUPPORT_PCL6_INT         = 0x10000000
   };

                     Okidata_PCL_Instance  (PrintDevice                    *pDevice);
   virtual          ~Okidata_PCL_Instance  ();

   void              initializeInstance        ();

   Enumeration      *listDeviceJobPropertyKeys ();
   Enumeration      *listKeyValues             (char                           *pszKey);
   bool              handleDeviceJobProperty   (char                           *pszKey,
                                                char                           *pszValue);
   string           *getJobPropertyType        (char                           *pszKey);
   string           *getJobProperty            (char                           *pszKey);

   string           *translateKeyValue         (char                           *pszKey,
                                                char                           *pszValue);

   virtual bool      beginJob                  ();
   virtual bool      newFrame                  ();
   virtual bool      newFrame                  (bool                            fJobPropertiesChanged);
   virtual bool      endJob                    ();
   virtual bool      abortJob                  ();

   virtual string    toString                  (ostringstream&                  oss);
   friend ostream&   operator<<                (ostream&                        os,
                                                const Okidata_PCL_Instance& device);

   void              setupPrinter              ();

   POINTL            ptlPrintHead_d;
   int               iUOM_d;
   int               iXScalingFactor_d;
   int               iYScalingFactor_d;
   int               iYUOMScalingFactor_d;
   int               iVerticalOffset_d;

private:
   bool              fHaveInitialized_d;
   bool              fHaveSetupPrinter_d;
   int               iHardwareScaling_d;
};

#endif
