/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Omni.hpp"

void
printUsage (int argc, char *argv[])
{
   cerr << "Error: Usage: "
        << argv[0]
        << " [-v] libDeviceName.so [VERSION]"
        << endl;
   cerr << "       Command line is ";
   for (int i = 0; i < argc; i++)
   {
      cerr << "\""
           << argv[i]
           << "\"";
   }
   cerr << endl;
}

/* Usage:
**
**   OmniDeviceValid [-v] libDeviceName.so [VERSION]
*/
int
main (int argc, char *argv[])
{
   GModule *hDevice        = 0;
   int      rc             = 1;
   bool     fVerbose       = false;
   char    *pszLibraryName = 0;
   char    *pszVersion     = 0;
   int      i              = 1;

   if (2 > argc)
   {
      // No parameters
      printUsage (argc, argv);

      return rc;
   }

   if (  0 == strcasecmp (argv[i], "-v")
      || 0 == strcasecmp (argv[i], "-V")
      )
   {
      fVerbose = true;
      i++;
   }

   if (1 > argc - i)
   {
      printUsage (argc, argv);

      return rc;
   }

   Omni::initialize ();

   pszLibraryName = argv[i++];

   hDevice = g_module_open (pszLibraryName, (GModuleFlags)0);

   if (hDevice)
   {
      rc = 0;
   }
   else
   {
      if (fVerbose)
         cerr << "Error: " << g_module_error () << endl;
   }

   if (  1 <= argc - i
      && hDevice
      )
   {
      pszVersion = argv[i++];

      if (Omni::libraryValid (hDevice, pszLibraryName, pszVersion, fVerbose))
      {
         rc = 0;
      }
      else
      {
         rc = 1;
      }
   }

   if (hDevice)
   {
      g_module_close (hDevice);
   }

   Omni::terminate ();

   return rc;
}
