/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "StringResourceEn.hpp"
#include "DeviceOrientation.hpp"
#include "DeviceForm.hpp"
#include "DeviceMedia.hpp"
#include "DevicePrintMode.hpp"
#include "DeviceResolution.hpp"
#include "DeviceTray.hpp"

StringResourceEn::
StringResourceEn (StringResource *pFallback)
{
   pFallback_d = pFallback;
}

char * StringResourceEn::
getStringV (int iGroup, int iID)
{
   char **apszStrings = 0;
   int    iNumStrings = 0;
   int    iOffset     = 0;

   switch (iGroup)
   {
   case STRINGGROUP_ORIENTATIONS:
   {
      static char *apszOrientationNames[] = {
      /*                                   Translation start: */
      /*  ORIENTATION_UNLISTED          */ "Unlisted",
      /*  ORIENTATION_NONE              */ "None",
      /*  ORIENTATION_PORTRAIT          */ "Portrait",
      /*  ORIENTATION_LANDSCAPE         */ "Landscape",
      /*  ORIENTATION_REVERSE_PORTRAIT  */ "Reverse Portrait",
      /*  ORIENTATION_REVERSE_LANDSCAPE */ "Reverse Landscape"
      /*                                   Translation end! */
      };

      apszStrings = apszOrientationNames;
      iNumStrings = dimof (apszOrientationNames);
      iOffset     = DeviceOrientation::ORIENTATION_UNLISTED;
      break;
   }

   case STRINGGROUP_FORMS:
   {
      static char *apszFormNames[] = {
      /*                               Translation start: */
      /* FORM_UNLISTED              */ "Unlisted",
      /* FORM_NONE                  */ "None",
      /* FORM_USER_DEFINED          */ "User defined",
      /* FORM_TEST                  */ "Test",
      /* FORM_TEST2                 */ "Test2",
      /* FORM_LEDGER                */ "Ledger",
      /* FORM_LEGAL                 */ "Legal",
      /* FORM_LEGAL_EXTRA           */ "Legal extra",
      /* FORM_LETTER                */ "Letter",
      /* FORM_LETTER_PLUS           */ "Letter plus",
      /* FORM_LETTER_EXTRA          */ "Letter extra",
      /* FORM_HALF_LETTER           */ "Half Letter",
      /* FORM_EXECUTIVE             */ "Executive",
      /* FORM_STATEMENT             */ "Statement",
      /* FORM_TABLOID               */ "Tabloid",
      /* FORM_UNIVERSAL             */ "Universal",
      /* FORM_FOOLSCAP              */ "Foolscap",
      /* FORM_QUARTO                */ "Quarto",
      /* FORM_FOLIO                 */ "Folio",
      /* FORM_FOLIO_SP              */ "Folio sp",
      /* FORM_ROC_8K                */ "Roc 8K",
      /* FORM_ROC_16K               */ "Roc 16K",
      /* FORM_GOVERNMENT_LETTER     */ "Government Letter",
      /* FORM_GOVERNMENT_LEGAL      */ "Government Legal",
      /* FORM_EDP                   */ "EDP",
      /* FORM_EUROPEAN_EDP          */ "European EDP",
      /* FORM_PRC_16K               */ "PRC 16K",
      /* FORM_PRC_32K               */ "PRC 32K",
      /* FORM_DAI_PA_KAI            */ "Dai Pa Kai",
      /* FORM_JURRO_KU_KAI          */ "Jurro Ku Kai",
      /* FORM_PA_KAI                */ "Pa Kai",
      /* FORM_WIDE                  */ "Wide",
      /* FORM_LETTER_WIDE           */ "Letter Wide",
      /* FORM_FOOLSCAP_WIDE         */ "Foolscap Wide",
      /* FORM_A3_WIDE               */ "A3 Wide",
      /* FORM_A4_WIDE               */ "A4 Wide",
      /* FORM_WIDE_FORMAT           */ "Wide Format",
      /* FORM_10_X_11               */ "10 x 11",
      /* FORM_11_X_12               */ "11 x 12",
      /* FORM_11_X_15               */ "11 x 15",
      /* FORM_12_X_19               */ "12 x 19",
      /* FORM_15_X_11               */ "15 x 11",
      /* FORM_5_X_7                 */ "5 x 7",
      /* FORM_3_X_5_CARD            */ "3 x 5 Card",
      /* FORM_4_X_6_CARD            */ "4 x 6 Card",
      /* FORM_5_X_8_CARD            */ "5 x 8 Card",
      /* FORM_8_X_10_CARD           */ "8 x 10 Card",
      /* FORM_A6_CARD               */ "A6 Card",
      /* FORM_CARD_148              */ "Card 148",
      /* FORM_HAGAKI_CARD           */ "Hagaki Card",
      /* FORM_OUFUKU_CARD           */ "Oufuku Card",
      /* FORM_POSTCARD              */ "Postcard",
      /* FORM_INDEX_4_X_6_EXT       */ "Index 4 x 6 Ext",
      /* FORM_A2_ENVELOPE           */ "A2 Envelope",
      /* FORM_C5_ENVELOPE           */ "C5 Envelope",
      /* FORM_C6_ENVELOPE           */ "C6 Envelope",
      /* FORM_C7_ENVELOPE           */ "C7 Envelope",
      /* FORM_C9_ENVELOPE           */ "C9 Envelope",
      /* FORM_C10_ENVELOPE          */ "C10 Envelope",
      /* FORM_D5_ENVELOPE           */ "D5 Envelope",
      /* FORM_DL_ENVELOPE           */ "Dl Envelope",
      /* FORM_ENVELOPE_132_X_220    */ "Envelope 132 x 220",
      /* FORM_ENVELOPE_6_X_9        */ "Envelope 6 x 9",
      /* FORM_ENVELOPE_7_X_9        */ "Envelope 7 x 9",
      /* FORM_ENVELOPE_6_1_2        */ "Envelope 6 1/2",
      /* FORM_ENVELOPE_9_X_11       */ "Envelope 9 x 11",
      /* FORM_ENVELOPE_10_X_13      */ "Envelope 10 x 13",
      /* FORM_ENVELOPE_10_X_14      */ "Envelope 10 x 14",
      /* FORM_ENVELOPE_10_X_15      */ "Envelope 10 x 15",
      /* FORM_MONARCH_ENVELOPE      */ "Monarch Envelope",
      /* FORM_PERSONAL_ENVELOPE     */ "Personal Envelope",
      /* FORM_NUMBER_9_ENVELOPE     */ "Number 9 Envelope",
      /* FORM_NUMBER_10_ENVELOPE    */ "Number 10 Envelope",
      /* FORM_NUMBER_11_ENVELOPE    */ "Number 11 Envelope",
      /* FORM_NUMBER_12_ENVELOPE    */ "Number 12 Envelope",
      /* FORM_NUMBER_14_ENVELOPE    */ "Number 14 Envelope",
      /* FORM_ITALIAN_ENVELOPE      */ "Italian Envelope",
      /* FORM_POSTFIX_ENVELOPE      */ "Postfix Envelope",
      /* FORM_INVITE_ENVELOPE       */ "Invite Envelope",
      /* FORM_CHOU2_ENVELOPE        */ "Chou2 Envelope",
      /* FORM_CHOU3_ENVELOPE        */ "Chou3 Envelope",
      /* FORM_CHOU4_ENVELOPE        */ "Chou4 Envelope",
      /* FORM_KAHU_ENVELOPE         */ "Kahu Envelope",
      /* FORM_KAKU2_ENVELOPE        */ "Kaku2 Envelope",
      /* FORM_YOU4_ENVELOPE         */ "You4 Envelope",
      /* FORM_PRC1_ENVELOPE         */ "PRC1 Envelope",
      /* FORM_PRC2_ENVELOPE         */ "PRC2 Envelope",
      /* FORM_PRC3_ENVELOPE         */ "PRC3 Envelope",
      /* FORM_PRC4_ENVELOPE         */ "PRC4 Envelope",
      /* FORM_PRC5_ENVELOPE         */ "PRC5 Envelope",
      /* FORM_PRC6_ENVELOPE         */ "PRC6 Envelope",
      /* FORM_PRC7_ENVELOPE         */ "PRC7 Envelope",
      /* FORM_PRC8_ENVELOPE         */ "PRC8 Envelope",
      /* FORM_PRC9_ENVELOPE         */ "PRC9 Envelope",
      /* FORM_PRC10_ENVELOPE        */ "PRC10 Envelope",
      /* FORM_DISK_LABELS           */ "Disk Labels",
      /* FORM_EURO_LABELS           */ "Euro Labels",
      /* FORM_SHIPPING_LABELS       */ "Shipping Labels",
      /* FORM_STANDARD_LABELS_CLEAR */ "Standard Labels Clear",
      /* FORM_STANDARD_LABELS_WHITE */ "Standard Labels White",
      /* FORM_2A0                   */ "2A0",
      /* FORM_4A0                   */ "4A0",
      /* FORM_A0                    */ "A0",
      /* FORM_A1                    */ "A1",
      /* FORM_A2                    */ "A2",
      /* FORM_A3                    */ "A3",
      /* FORM_A3_EXTRA              */ "A3 extra",
      /* FORM_A4                    */ "A4",
      /* FORM_A4_EXTRA              */ "A4 extra",
      /* FORM_A4_TAB                */ "A4 tab",
      /* FORM_A5                    */ "A5",
      /* FORM_A5_EXTRA              */ "A5 extra",
      /* FORM_A6                    */ "A6",
      /* FORM_A7                    */ "A7",
      /* FORM_A8                    */ "A8",
      /* FORM_A9                    */ "A9",
      /* FORM_A10                   */ "A10",
      /* FORM_B_PLUS                */ "B plus",
      /* FORM_B0                    */ "B0",
      /* FORM_B1                    */ "B1",
      /* FORM_B2                    */ "B2",
      /* FORM_B3                    */ "B3",
      /* FORM_B4                    */ "B4 Envelope",
      /* FORM_B5                    */ "B5 Envelope",
      /* FORM_B5_EXTRA              */ "B5 extra",
      /* FORM_B6                    */ "B6 Envelope",
      /* FORM_B6_C4                 */ "B6/C4 Envelope",
      /* FORM_B7                    */ "B7",
      /* FORM_B8                    */ "B8",
      /* FORM_B9                    */ "B9",
      /* FORM_B10                   */ "B10",
      /* FORM_JIS_B0                */ "JIS B0",
      /* FORM_JIS_B1                */ "JIS B1",
      /* FORM_JIS_B2                */ "JIS B2",
      /* FORM_JIS_B3                */ "JIS B3",
      /* FORM_JIS_B4                */ "JIS B4",
      /* FORM_JIS_B5                */ "JIS B5",
      /* FORM_JIS_B6                */ "JIS B6",
      /* FORM_JIS_B7                */ "JIS B7",
      /* FORM_JIS_B8                */ "JIS B8",
      /* FORM_JIS_B9                */ "JIS B9",
      /* FORM_JIS_B10               */ "JIS B10",
      /* FORM_JIS_EXEC              */ "JIS Exec",
      /* FORM_C0                    */ "C0",
      /* FORM_C1                    */ "C1",
      /* FORM_C2                    */ "C2",
      /* FORM_C3                    */ "C3",
      /* FORM_C4                    */ "C4",
      /* FORM_C5                    */ "C5",
      /* FORM_C6                    */ "C6",
      /* FORM_C6_C5                 */ "C6/C5",
      /* FORM_C7                    */ "C7",
      /* FORM_C7_C6                 */ "C7/C6",
      /* FORM_C8                    */ "C8",
      /* FORM_C9                    */ "C9",
      /* FORM_C10                   */ "C10",
      /* FORM_E1                    */ "E1",
      /* FORM_RA0                   */ "RA0",
      /* FORM_RA1                   */ "RA1",
      /* FORM_RA2                   */ "RA2",
      /* FORM_SRA0                  */ "SRA0",
      /* FORM_SRA1                  */ "SRA1",
      /* FORM_SRA2                  */ "SRA2",
      /* FORM_US_FANFOLD            */ "Fanfold",
      /* FORM_FANFOLD_1             */ "Fanfold #1",
      /* FORM_FANFOLD_2             */ "Fanfold #2",
      /* FORM_FANFOLD_3             */ "Fanfold #3",
      /* FORM_FANFOLD_4             */ "Fanfold #4",
      /* FORM_FANFOLD_5             */ "Fanfold #5",
      /* FORM_GERMAN_LEGAL_FANFOLD  */ "German Legal Fanfold",
      /* FORM_EUROPEAN_FANFOLD      */ "European Fanfold",
      /* FORM_PANORAMIC             */ "Panoramic",
      /* FORM_PHOTO_100_150         */ "Photo 100x150",
      /* FORM_PHOTO_200_300         */ "Photo 200x300",
      /* FORM_PHOTO_4_6             */ "Photo 4x6",
      /* FORM_PLOTTER_SIZE_A        */ "Plotter A paper",
      /* FORM_PLOTTER_SIZE_B        */ "Plotter B paper",
      /* FORM_PLOTTER_SIZE_C        */ "Plotter C paper",
      /* FORM_PLOTTER_SIZE_D        */ "Plotter D paper",
      /* FORM_PLOTTER_SIZE_E        */ "Plotter E paper",
      /* FORM_PLOTTER_SIZE_F        */ "Plotter F paper",
      /* FORM_US_A_ARCHITECTURAL    */ "US A Architectural",
      /* FORM_US_B_ARCHITECTURAL    */ "US B Architectural",
      /* FORM_US_C_ARCHITECTURAL    */ "US C Architectural",
      /* FORM_US_D_ARCHITECTURAL    */ "US D Architectural",
      /* FORM_US_E_ARCHITECTURAL    */ "US E Architectural",
      /* FORM_SUPER_A3_B            */ "Super A3-B",
      /* FORM_SUPER_A               */ "Super A",
      /* FORM_SUPER_B               */ "Super B",
      /* FORM_8_5_X_12_4            */ "8.5 x 12.4",
      /* FORM_8_X_10_5              */ "8 x 10.5",
      /* FORM_8_25_X_13             */ "8.25 x 13",
      /* FORM_170_X_210             */ "170 x 210 mm",
      /* FORM_182_X_210             */ "182 x 210 mm",
      /* FORM_ROLL_76_2MM           */ "76.2 mm Roll",
      /* FORM_ROLL_69_5MM           */ "69.5 mm Roll"
      /*                               Translation end! */
      };

      apszStrings = apszFormNames;
      iNumStrings = dimof (apszFormNames);
      iOffset     = DeviceForm::FORM_UNLISTED;
      break;
   }

   case STRINGGROUP_TRAYS:
   {
      static char *apszTrayNames[] = {
         /*                           Translation start: */
      /* TRAY_UNLISTED             */ "Unlisted",
      /* TRAY_NONE                 */ "None",
      /* TRAY_AUTO                 */ "Auto",
      /* TRAY_USE_PRINTER_SETTING  */ "Use printer setting",
      /* TRAY_AUTO_FEEDER          */ "Auto Feeder",
      /* TRAY_AUTOSWITCH           */ "Auto-switch",
      /* TRAY_TRAY                 */ "Tray",
      /* TRAY_UPPER_TRAY           */ "Upper Tray",
      /* TRAY_LOWER_TRAY           */ "Lower Tray",
      /* TRAY_MULTI_TRAY           */ "Multi Tray",
      /* TRAY_CASSETTE             */ "Cassette",
      /* TRAY_UPPER_CASSETTE       */ "Upper Cassette",
      /* TRAY_LOWER_CASSETTE       */ "Lower Cassette",
      /* TRAY_MULTI_CASSETTE       */ "Multi Cassette",
      /* TRAY_OPTION_CASSETTE      */ "Option Cassette",
      /* TRAY_MANUAL_FEED          */ "Manual Feed",
      /* TRAY_MANUAL_ENVELOPE      */ "Manual Envelope",
      /* TRAY_PAPER_FEED           */ "Paper Feed",
      /* TRAY_ENVELOPE_FEED        */ "Env Feed",
      /* TRAY_CSF                  */ "Continuous Sheet Feeder",
      /* TRAY_FRONT_CONTINUOUS     */ "Front Continuous",
      /* TRAY_REAR_CONTINUOUS      */ "Rear Continuous",
      /* TRAY_SINGLE_SHEET         */ "Single Sheet",
      /* TRAY_SHEET_FEEDER         */ "Sheet Feeder",
      /* TRAY_BIN_1                */ "Bin 1",
      /* TRAY_BIN_2                */ "Bin 2",
      /* TRAY_PORTABLE_SHEET       */ "Portable Sheet",
      /* TRAY_TRAY_1               */ "Tray 1",
      /* TRAY_TRAY_2               */ "Tray 2",
      /* TRAY_TRAY_3               */ "Tray 3",
      /* TRAY_TRAY_4               */ "Tray 4",
      /* TRAY_TRAY_5               */ "Tray 5",
      /* TRAY_TRAY_6               */ "Tray 6",
      /* TRAY_CASSTTE1             */ "Casstte 1",
      /* TRAY_CASSTTE2             */ "Casstte 2",
      /* TRAY_CASSTTE3             */ "Casstte 3",
      /* TRAY_CASSTTE4             */ "Casstte 4",
      /* TRAY_CASSTTE5             */ "Casstte 5",
      /* TRAY_FRONT_TRAY           */ "Front tray",
      /* TRAY_OPTION_MULTI_SF      */ "Tray option multi-sf",
      /* TRAY_MULTI_FEEDER         */ "Tray multi-feeder",
      /* TRAY_PAPER_DECK           */ "Paper deck",
      /* TRAY_TRACTOR_UNIT         */ "Tractor unit",
      /* TRAY_ROLL_1               */ "Roll 1",
      /* TRAY_ROLL_2               */ "Roll 2",
      /* TRAY_ROLL_3               */ "Roll 3",
      /* TRAY_ROLL_4               */ "Roll 4",
      /* TRAY_ROLL_5               */ "Roll 5",
      /* TRAY_AUXILIARY_TRAY       */ "Auxiliary tray",
      /* TRAY_LARGE_CAPACITY_TRAY  */ "Large capacity tray",
      /* TRAY_HIGH_CAPACITY_FEEDER */ "High capacity feeder",
      /* TRAY_ZERO_MARGINS         */ "Tray with zero margins"
      /*                              Translation end! */
      };

      apszStrings = apszTrayNames;
      iNumStrings = dimof (apszTrayNames);
      iOffset     = DeviceTray::TRAY_UNLISTED;
      break;
   }

   case STRINGGROUP_MEDIAS:
   {
      static char *apszMediaNames[] = {
      /*                                        Translation start: */
      /* MEDIA_UNLISTED                      */ "Unlisted",
      /* MEDIA_NONE                          */ "None",
      /* MEDIA_AUTO                          */ "Auto",
      /* MEDIA_USE_PRINTER_SETTING           */ "Use printer setting",
      /* MEDIA_PLAIN                         */ "Plain",
      /* MEDIA_TRANSPARENCY                  */ "Transparency",
      /* MEDIA_GLOSSY                        */ "Glossy",
      /* MEDIA_SPECIAL                       */ "Special",
      /* MEDIA_COATED                        */ "Coated",
      /* MEDIA_BACKPRINT                     */ "Backprint",
      /* MEDIA_CLOTH                         */ "Cloth",
      /* MEDIA_THICK                         */ "Thick",
      /* MEDIA_HIGH_GLOSS_FILM               */ "High Gloss Film",
      /* MEDIA_HIGH_RESOLUTION               */ "High Resolution",
      /* MEDIA_SPECIAL_360                   */ "Special 360",
      /* MEDIA_SPECIAL_720                   */ "Special 720",
      /* MEDIA_PLAIN_ENHANCED                */ "Plain Enhanced",
      /* MEDIA_IRON_ON                       */ "Iron-on",
      /* MEDIA_LABECA                        */ "Labeca",
      /* MEDIA_THERMAL                       */ "Thermal",
      /* MEDIA_CD_MASTER                     */ "CD-master",
      /* MEDIA_CARDBOARD                     */ "Cardboard",
      /* MEDIA_POSTCARD                      */ "Postcard",
      /* MEDIA_PHOTOGRAPHIC_PAPER            */ "Photographic Paper",
      /* MEDIA_PHOTOGRAPHIC_LABEL            */ "Photographic Label",
      /* MEDIA_HP_PREMIUM_PAPER              */ "HP Premium Paper",
      /* MEDIA_HP_PHOTOGRAPHIC_PAPER         */ "HP Photographic Paper",
      /* MEDIA_ENVELOPE                      */ "Envelope",
      /* MEDIA_PREPRINTED                    */ "Preprinted",
      /* MEDIA_LETTERHEAD                    */ "Letter Head",
      /* MEDIA_PREPUNCHED                    */ "Prepunched",
      /* MEDIA_LABELS                        */ "Labels",
      /* MEDIA_BOND                          */ "Bond",
      /* MEDIA_RECYCLED                      */ "Recycled",
      /* MEDIA_COLOR                         */ "Color",
      /* MEDIA_CARDSTOCK                     */ "Cardstock",
      /* MEDIA_ROUGH                         */ "Rough",
      /* MEDIA_ARCHIVAL_MATTE_PAPER          */ "Archival Matte Paper",
      /* MEDIA_BRIGHT_WHITE_INK_JET_PAPER    */ "Bright White Ink Jet Paper",
      /* MEDIA_COLORLIFE_PHOTO_PAPER         */ "ColorLife Photo Paper",
      /* MEDIA_DOUBLE_SIDED_MATTE_PAPER      */ "Double-Sided Matte Paper",
      /* MEDIA_HEAVYWEIGH_MATTE_PAPER        */ "Heavyweight Matte Paper",
      /* MEDIA_PREMIUM_SEMIGLOSS_PHOTO_PAPER */ "Premium Semigloss Photo Paper",
      /* MEDIA_PHOTOGRAPHIC_INK_JET_PAPER    */ "Photo Quality Ink Jet Paper",
      /* MEDIA_CUSTOM_1                      */ "Custom 1",
      /* MEDIA_CUSTOM_2                      */ "Custom 2",
      /* MEDIA_CUSTOM_3                      */ "Custom 3",
      /* MEDIA_CUSTOM_4                      */ "Custom 4",
      /* MEDIA_CUSTOM_5                      */ "Custom 5",
      /* MEDIA_SPECIAL_BLUE                  */ "Blue",
      /* MEDIA_SPECIAL_GREEN                 */ "Green",
      /* MEDIA_SPECIAL_GREY                  */ "Grey",
      /* MEDIA_SPECIAL_IVORY                 */ "Ivory",
      /* MEDIA_SPECIAL_LETTERHEAD            */ "Letterhead",
      /* MEDIA_SPECIAL_ORANGE                */ "Orange",
      /* MEDIA_SPECIAL_PINK                  */ "Pink",
      /* MEDIA_SPECIAL_PURPLE                */ "Purple",
      /* MEDIA_SPECIAL_RED                   */ "Red",
      /* MEDIA_SPECIAL_YELLOW                */ "Yellow",
      /* MEDIA_SPECIAL_USER_COLOR            */ "User Color",
      /* MEDIA_TABSTOCK                      */ "Tabstock",
      /* MEDIA_TABSTOCK_2                    */ "2-Tab",
      /* MEDIA_TABSTOCK_3                    */ "3-Tab",
      /* MEDIA_TABSTOCK_4                    */ "4-Tab",
      /* MEDIA_TABSTOCK_5                    */ "5-Tab",
      /* MEDIA_TABSTOCK_6                    */ "6-Tab",
      /* MEDIA_TABSTOCK_7                    */ "7-Tab",
      /* MEDIA_TABSTOCK_8                    */ "8-Tab",
      /* MEDIA_TABSTOCK_9                    */ "9-Tab",
      /* MEDIA_THICK_1                       */ "Thick 1",
      /* MEDIA_THICK_2                       */ "Thick 2",
      /* MEDIA_THICK_3                       */ "Thick 3",
      /* MEDIA_THICK_BLUE                    */ "Thick Blue",
      /* MEDIA_THICK_GREEN                   */ "Thick Green",
      /* MEDIA_THICK_GREY                    */ "Thick Grey",
      /* MEDIA_THICK_IVORY                   */ "Thick Ivory",
      /* MEDIA_THICK_LETTERHEAD              */ "Thick Letterhead",
      /* MEDIA_THICK_ORANGE                  */ "Thick Orange",
      /* MEDIA_THICK_PINK                    */ "Thick Pink",
      /* MEDIA_THICK_PURPLE                  */ "Thick Purple",
      /* MEDIA_THICK_RED                     */ "Thick Red",
      /* MEDIA_THICK_YELLOW                  */ "Thick Yellow",
      /* MEDIA_THICK_USER_COLOR              */ "Thick User Color",
      /* MEDIA_TRANSLUCENT                   */ "Translucent",
      /* MEDIA_OTHER                         */ "Other"
      /*                                        Translation end! */
      };

      apszStrings = apszMediaNames;
      iNumStrings = dimof (apszMediaNames);
      iOffset     = DeviceMedia::MEDIA_UNLISTED;
      break;
   }

   case STRINGGROUP_PRINTMODES:
   {
      static char *apszPrintModeNames[] = {
      /*                              Translation start: */
      /* PRINT_MODE_UNLISTED       */ "Unlisted",
      /* PRINT_MODE_NONE           */ "None",
      /* PRINT_MODE_1_ANY          */ "Monochrome",
      /* PRINT_MODE_8_K            */ "Monochrome",
      /* PRINT_MODE_8_CMY          */ "8-bit 3 Color with composite Black",
      /* PRINT_MODE_8_CMYK         */ "8-bit 4 Color with separate Black",
      /* PRINT_MODE_8_CcMmYK       */ "8-bit 6 Color",
      /* PRINT_MODE_8_CcMmYyK      */ "8-bit 7 Color",
      /* PRINT_MODE_8_RGB          */ "8-bit RGB Color",
      /* PRINT_MODE_24_K           */ "Monochrome",
      /* PRINT_MODE_24_CMY         */ "24-bit Color 3 Color with composite Black",
      /* PRINT_MODE_24_CMYK        */ "24-bit Color 4 Color with separate Black",
      /* PRINT_MODE_24_CcMmYK      */ "24-bit 6 Color",
      /* PRINT_MODE_24_CcMmYyK     */ "24-bit 7 Color",
      /* PRINT_MODE_24_RGB         */ "24-bit RGB Color"
      /*                              Translation end! */
      };

      apszStrings = apszPrintModeNames;
      iNumStrings = dimof (apszPrintModeNames);
      iOffset     = DevicePrintMode::PRINT_MODE_UNLISTED;
      break;
   }

   case STRINGGROUP_RESOLUTIONS:
   {
      static char *apszResolutionNames[] = {
      /*                             Translation start: */
      /* RESOLUTION_UNLISTED      */ "Unlisted",
      /* RESOLUTION_NONE          */ "None",
      /* RESOLUTION_60_X_60       */ "60x60",
      /* RESOLUTION_60_X_72       */ "60x72",
      /* RESOLUTION_60_X_180      */ "60x180",
      /* RESOLUTION_60_X_360      */ "60x360",
      /* RESOLUTION_72_X_72       */ "72x72",
      /* RESOLUTION_75_X_75       */ "75x75",
      /* RESOLUTION_80_X_60       */ "80x60",
      /* RESOLUTION_80_X_72       */ "80x72",
      /* RESOLUTION_90_X_180      */ "90x180",
      /* RESOLUTION_90_X_360      */ "90x360",
      /* RESOLUTION_90_X_60       */ "90x60",
      /* RESOLUTION_90_X_72       */ "90x72",
      /* RESOLUTION_90_X_90       */ "90x90",
      /* RESOLUTION_100_X_100     */ "100x100",
      /* RESOLUTION_120_X_60      */ "120x60",
      /* RESOLUTION_120_X_72      */ "120x72",
      /* RESOLUTION_120_X_120     */ "120x120",
      /* RESOLUTION_120_X_144     */ "120x144",
      /* RESOLUTION_120_X_180     */ "120x180",
      /* RESOLUTION_120_X_360     */ "120x360",
      /* RESOLUTION_144_X_72      */ "144x72",
      /* RESOLUTION_150_X_150     */ "150x150",
      /* RESOLUTION_180_X_180     */ "180x180",
      /* RESOLUTION_180_X_360     */ "180x360",
      /* RESOLUTION_200_X_200     */ "200x200",
      /* RESOLUTION_240_X_60      */ "240x60",
      /* RESOLUTION_240_X_72      */ "240x72",
      /* RESOLUTION_240_X_144     */ "240x144",
      /* RESOLUTION_240_X_240     */ "240x240",
      /* RESOLUTION_300_X_300     */ "300x300",
      /* RESOLUTION_360_X_180     */ "360x180",
      /* RESOLUTION_360_X_360     */ "360x360",
      /* RESOLUTION_360_X_720     */ "360x720",
      /* RESOLUTION_400_X_400     */ "400x400",
      /* RESOLUTION_600_X_300     */ "600x300",
      /* RESOLUTION_600_X_600     */ "600x600",
      /* RESOLUTION_720_X_360     */ "720x360",
      /* RESOLUTION_720_X_720     */ "720x720",
      /* RESOLUTION_1200_X_600    */ "1200x600",
      /* RESOLUTION_1200_X_1200   */ "1200x1200",
      /* RESOLUTION_1440_X_720    */ "1440x720",
      /* RESOLUTION_2880_X_720    */ "2880x720",
      /* RESOLUTION_DRAFT         */ "Draft",
      /* RESOLUTION_FINE          */ "Fine",
      /* RESOLUTION_GROUP_3       */ "Group 3",
      /* RESOLUTION_GROUP_4       */ "Group 4",
      /* RESOLUTION_HIGH          */ "High",
      /* RESOLUTION_LOW           */ "Low",
      /* RESOLUTION_MEDIUM        */ "Medium",
      /* RESOLUTION_NORMAL        */ "Normal",
      /* RESOLUTION_PHOTO_QUALITY */ "Photo Quality",
      /* RESOLUTION_PRESENTATION  */ "Presentation"
      /*                             Translation end! */
      };

      apszStrings = apszResolutionNames;
      iNumStrings = dimof (apszResolutionNames);
      iOffset     = DeviceResolution::RESOLUTION_UNLISTED;
      break;
   }

   case STRINGGROUP_DEVICE_COMMON:
   {
      static char *apszDeviceCommonNames[] = {
      /*                                   Translation start: */
      /* DEVICE_COMMON_UNKNOWN          */ "Unknown",
      /* DEVICE_COMMON_UNLISTED         */ "Unlisted",
      /* DEVICE_COMMON_ORIENTATION      */ "Orientation",
      /* DEVICE_COMMON_DITHER           */ "Dither",
      /* DEVICE_COMMON_FORM             */ "Form",
      /* DEVICE_COMMON_TRAY             */ "Tray",
      /* DEVICE_COMMON_MEDIA            */ "Media",
      /* DEVICE_COMMON_RESOLUTION       */ "Resolution",
      /* DEVICE_COMMON_PRINTMODE        */ "Print Mode",
      /* DEVICE_COMMON_BIDIRECTIONAL    */ "Bi-Directional",
      /* DEVICE_COMMON_TRUE             */ "True",
      /* DEVICE_COMMON_FALSE            */ "False",
      /* DEVICE_COMMON_JOURNAL          */ "Journal",
      /* DEVICE_COMMON_CUTMODE          */ "Cut Mode",
      /* DEVICE_COMMON_NONE             */ "None",
      /* DEVICE_COMMON_FULL             */ "Full",
      /* DEVICE_COMMON_PARTIAL          */ "Partial",
      /* DEVICE_COMMON_HARDWARE_SCALING */ "Hardware Scaling",
      /* DEVICE_COMMON_QUALITY          */ "Quality",
      /* DEVICE_COMMON_STANDARD         */ "Standard",
      /* DEVICE_COMMON_SUPER_FINE       */ "Super Fine",
      /* DEVICE_COMMON_PHOTO            */ "Photo",
      /* DEVICE_COMMON_SUPER_PHOTO      */ "Super Photo",
      /* DEVICE_COMMON_HIGH_SPEED       */ "High Speed",
      /* DEVICE_COMMON_ON               */ "On",
      /* DEVICE_COMMON_OFF              */ "Off",
      /* DEVICE_COMMON_COLOR_MODE       */ "Color Mode",
      /* DEVICE_COMMON_AUTO             */ "Automatic",
      /* DEVICE_COMMON_GRAPH            */ "Graph",
      /* DEVICE_COMMON_MICRO_WEAVE      */ "Micro Weave",
      /* DEVICE_COMMON_RET              */ "RET (Resolution Enhancement Technology)",
      /* DEVICE_COMMON_ECONO_MODE       */ "Economy Mode",
      /* DEVICE_COMMON_PAGE_PROTECT     */ "Page Protect",
      /* DEVICE_COMMON_JAM_RECOVERY     */ "Jam Recovery",
      /* DEVICE_COMMON_STAPLE           */ "Staple",
      /* DEVICE_COMMON_OFFSET           */ "Offset"
      /*                                   Translation end! */
      };

      apszStrings = apszDeviceCommonNames;
      iNumStrings = dimof (apszDeviceCommonNames);
      iOffset     = 0;
      break;
   }

   case STRINGGROUP_COLLATIONS:
   {
      static char *apszCollationNames[] = {
      /*                                   Translation start: */
      /* COLLATION_UNLISTED             */ "Unlisted",
      /* COLLATION_UNCOLLATED           */ "Uncollated",
      /* COLLATION_TRAY_COLLATION       */ "Tray Collation"
      /*                                   Translation end! */
      };

      apszStrings = apszCollationNames;
      iNumStrings = dimof (apszCollationNames);
      iOffset     = 0;
      break;
   }

   case STRINGGROUP_DUPLEXES:
   {
      static char *apszDuplexNames[] = {
      /*                                   Translation start: */
      /* DUPLEX_UNLISTED                */ "Unlisted",
      /* DUPLEX_SIMPLEX                 */ "Simplex",
      /* DUPLEX_LONG_EDGE               */ "Long Edge"
      /* DUPLEX_SHORT_EDGE              */ "Short Edge"
      /*                                   Translation end! */
      };

      apszStrings = apszDuplexNames;
      iNumStrings = dimof (apszDuplexNames);
      iOffset     = 0;
      break;
   }

   case STRINGGROUP_BOOKLETS:
   {
      static char *apszBookletNames[] = {
      /*                                   Translation start: */
      /* BOOKLET_UNLISTED               */ "Unlisted",
      /* BOOKLET_UNSORTED               */ "Unsorted",
      /* BOOKLET_SORTED                 */ "Sorted"
      /*                                   Translation end! */
      };

      apszStrings = apszBookletNames;
      iNumStrings = dimof (apszBookletNames);
      iOffset     = 0;
      break;
   }

   case STRINGGROUP_NUPS:
   {
      static char *apszNUpNames[] = {
      /*                                   Translation start: */
      /* NUP_UNLISTED                   */ "Unlisted",
      /* NUP_1                          */ "1 Up",
      /* NUP_2_V                        */ "2 Up with 1 vertical column",
      /* NUP_2_H                        */ "2 Up with 2 vertical columns",
      /* NUP_4                          */ "4 Up",
      /* NUP_6_V                        */ "6 Up with 2 vertical column",
      /* NUP_6_H                        */ "6 Up with 3 vertical columns",
      /* NUP_9                          */ "9 Up",
      /* NUP_16                         */ "16 Up",
      /*                                   Translation end! */
      };

      apszStrings = apszNUpNames;
      iNumStrings = dimof (apszNUpNames);
      iOffset     = 0;
      break;
   }

   default:
      return 0;
   }

   iID -= iOffset;

   if (  apszStrings
      && iID < iNumStrings
      )
      return apszStrings[iID];
   else
      return 0;
}

char * StringResourceEn::
getStringV (int iGroup, char *pszID)
{
   switch (iGroup)
   {
   case STRINGGROUP_DITHERS:
   {
      typedef struct _StringMap {
         char *pszFrom;
         char *pszTo;
      } STRINGMAP, *PSTRINGMAP;
      static STRINGMAP aDitherNames[] = {
      /* Translation ignore!             Translation start: */
         { "DITHER_CMYK_DIFFUSION",      "Cyan Magenta Yellow Black Diffusion" },
         { "DITHER_DITHER_4x4",          "4x4 Grid"                            },
         { "DITHER_DITHER_8x8",          "8x8 Grid"                            },
         { "DITHER_ESTUCKI_DIFFUSION",   "Enhanced Stucki"                     },
         { "DITHER_FAST_DIFFUSION",      "Fast Diffusion"                      },
         { "DITHER_HSV_BIDIFFUSION",     "Hue Saturaion Value Bidiffusion"     },
         { "DITHER_HSV_DIFFUSION",       "Hue Saturaion Value Diffusion"       },
         { "DITHER_JANIS_STUCKI",        "Janis Stucki"                        },
         { "DITHER_LEVEL",               "Level"                               },
         { "DITHER_MAGIC_SQUARES",       "Magic Square"                        },
         { "DITHER_ORDERED_SQUARES",     "Ordered Square"                      },
         { "DITHER_SMOOTH_DIFFUSION",    "Smooth Diffusion"                    },
         { "DITHER_SNAP",                "Snap"                                },
         { "DITHER_STEINBERG_DIFFUSION", "Steinberg Diffusion"                 },
         { "DITHER_STUCKI_BIDIFFUSION",  "Stucki Bidiffusion"                  },
         { "DITHER_STUCKI_DIFFUSION",    "Stucki Diffusion"                    },
         { "DITHER_UNLISTED",            "Unlisted"                            },
         { "DITHER_VOID_CLUSTER",        "Void Cluster"                        }
      /* Translation ignore!             Translation end! */
      };

      int iLow  = 0;
      int iMid  = (int)dimof (aDitherNames) / 2;
      int iHigh = (int)dimof (aDitherNames) - 1;
      int iResult;

      while (iLow <= iHigh)
      {
         iResult = strcmp (pszID, aDitherNames[iMid].pszFrom);

         if (0 == iResult)
         {
            return aDitherNames[iMid].pszTo;
         }
         else if (0 > iResult)
         {
            // str1 < str2
            iHigh = iMid - 1;
            iMid  = iLow + (iHigh - iLow) / 2;
         }
         else // (0 < iResult)
         {
            // str1 > str2
            iLow  = iMid + 1;
            iMid  = iLow + (iHigh - iLow) / 2;
         }
      }

      return 0;
   }

   default:
      return 0;
   }
}
