/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <UPDFDeviceCopies.hpp>

UPDFDeviceCopies::
UPDFDeviceCopies (Device      *pDevice,
                  int          id,
                  BinaryData  *data,
                  xmlNodePtr   node,
                  int          iMinimum,
                  int          iMaximum,
                  bool         fDeviceFeature)
   : DeviceCopies (pDevice,
                   id,
                   data,
                   iMinimum,
                   iMaximum,
                   fDeviceFeature)
{
   node_d = node;
}

UPDFDeviceCopies::
~UPDFDeviceCopies ()
{
}

DeviceCopies * UPDFDeviceCopies::
createV (int id)
{
   return UPDFDeviceCopies::create (pDevice_d, id);
}

DeviceCopies * UPDFDeviceCopies::
create (Device *pDevice, char *pszId)
{
   if (  !pszId
      || !*pszId
      )
   {
      return 0;
   }

   int id = 0;

   sscanf (pszId, "%d", &id);

   return create (pDevice, id);
}

DeviceCopies * UPDFDeviceCopies::
create (Device *pDevice, int id)
{
   UPDFDevice *pUPDFDevice = UPDFDevice::isAUPDFDevice (pDevice);
   xmlNodePtr  nodeCopies  = 0;

   nodeCopies = findCopies (pUPDFDevice);

   if (!nodeCopies)
   {
      return 0;
   }

   char         *pszMaximum = (char *)xmlGetProp (nodeCopies, (xmlChar *)"MaximumValue");
   int           iMaximum   = -1;
   DeviceCopies *pRet       = 0;

   if (pszMaximum)
   {
      sscanf (pszMaximum, "%d", &iMaximum);
   }

   if (id <= iMaximum)
   {
      BinaryData *data           = 0;    // @TBD
      bool        fDeviceFeature = true; // @TBD

      pRet = new UPDFDeviceCopies (pDevice,
                                   id,
                                   data,
                                   nodeCopies,
                                   1,
                                   iMaximum,
                                   fDeviceFeature);
   }

   if (pszMaximum)
   {
      free (pszMaximum);
   }

   return pRet;
}

char * UPDFDeviceCopies::
getDeviceID ()
{
   return 0;
}

bool UPDFDeviceCopies::
isCopiesSupported (int id)
{
   return false;
}

xmlNodePtr UPDFDeviceCopies::
findCopies (UPDFDevice *pUPDFDevice)
{
   xmlNodePtr nodeCopies = 0;

   if (!pUPDFDevice)
   {
      return nodeCopies;
   }

   if (  ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "Device_Features",    UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "PrintCapabilities",  UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "Features",           UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "PrintMediaHandling", UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "MediaCopy",          UPDFDeviceCopies)) != 0)
      )
      return nodeCopies;

   return nodeCopies;
}

string UPDFDeviceCopies::
toString (ostringstream& oss)
{
   ostringstream oss2;

   oss << "{"
       << DeviceCopies::toString (oss2)
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const UPDFDeviceCopies& const_self)
{
   UPDFDeviceCopies& self = const_cast<UPDFDeviceCopies&>(const_self);
   ostringstream  oss;

   os << self.toString (oss);

   return os;
}
