/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "IBM_Pages_Instance.hpp"
#include "defines.hpp"

#include <iostream>
#include <sstream>

const static bool fTestNoCompression = true;

DeviceInstance *
createInstance (PrintDevice *pDevice)
{
   return new IBM_Pages_Instance (pDevice);
}

void
deleteInstance (DeviceInstance *pInstance)
{
   delete pInstance;
}


IBM_Pages_Instance::
IBM_Pages_Instance (PrintDevice *pDevice)
   : DeviceInstance (pDevice)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::IBM_Pages_Instance ()" << endl;
#endif

   fHaveInitialized_d  = false;
   fHaveSetupPrinter_d = false;
   ptlPrintHead_d.x    = 0;
   ptlPrintHead_d.y    = 0;
}

IBM_Pages_Instance::
~IBM_Pages_Instance ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::~IBM_Pages_Instance ()" << endl;
#endif
}

void IBM_Pages_Instance::
initializeInstance ()
{
   if (fHaveInitialized_d)
      return;

   fHaveInitialized_d = true;

   fRET_d         = TSTATE_NONE;
   fEconoMode_d   = TSTATE_NONE;
   fPageProtect_d = TSTATE_NONE;
   fJamRecovery_d = TSTATE_NONE;
   fStaple_d      = TSTATE_NONE;
   fOffset_d      = TSTATE_NONE;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::initializeInstance ()" << endl;
#endif
}

static char *apszDeviceJobPropertyKeys[] = {
   "RET",
   "EconoMode",
   "PageProtect",
   "JamRecovery",
   "Staple",
   "Offset"
};
#define JOBPROP_RET              apszDeviceJobPropertyKeys[0]
#define JOBPROP_ECONOMODE        apszDeviceJobPropertyKeys[1]
#define JOBPROP_PAGEPROTECT      apszDeviceJobPropertyKeys[2]
#define JOBPROP_JAMRECOVERY      apszDeviceJobPropertyKeys[3]
#define JOBPROP_STAPLE           apszDeviceJobPropertyKeys[4]
#define JOBPROP_OFFSET           apszDeviceJobPropertyKeys[5]

static char *apszDeviceJobPropertyValues[] = {
   "on",
   "off",
   "none"
};
#define JOBPROP_ON      apszDeviceJobPropertyValues[0]
#define JOBPROP_OFF     apszDeviceJobPropertyValues[1]
#define JOBPROP_NONE    apszDeviceJobPropertyValues[2]

class DeviceJobPropertyKeyEnumerator : public Enumeration
{
public:
   DeviceJobPropertyKeyEnumerator (char **apszKeys, int iDimOfKeys)
   {
      iWhichKey_d = iDimOfKeys;
      apszKeys_d  = apszKeys;
   }

   virtual bool hasMoreElements ()
   {
      return iWhichKey_d > 0;
   }

   virtual void * nextElement ()
   {
      if (0 < iWhichKey_d)
      {
         iWhichKey_d--;
         return apszKeys_d[iWhichKey_d];
      }

      return 0;
   }

private:
   int    iWhichKey_d;
   char **apszKeys_d;
};

Enumeration * IBM_Pages_Instance::
listDeviceJobPropertyKeys ()
{
   return new DeviceJobPropertyKeyEnumerator (apszDeviceJobPropertyKeys, dimof (apszDeviceJobPropertyKeys));
}

bool IBM_Pages_Instance::
handleDeviceJobProperty (char *pszKey,
                         char *pszValue)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::handleDeviceJobProperty ("
                          << pszKey << ", "
                          << pszValue << ")"
                          << endl;
#endif

   if (0 == strcasecmp (pszKey, JOBPROP_RET))
   {
      if (0 == strcmp (pszValue, JOBPROP_ON))
      {
         fRET_d = TSTATE_ON;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_OFF))
      {
         fRET_d = TSTATE_OFF;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_NONE))
      {
         fRET_d = TSTATE_NONE;
         return true;
      }
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_ECONOMODE))
   {
      if (0 == strcmp (pszValue, JOBPROP_ON))
      {
         fEconoMode_d = TSTATE_ON;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_OFF))
      {
         fEconoMode_d = TSTATE_OFF;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_NONE))
      {
         fEconoMode_d = TSTATE_NONE;
         return true;
      }
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_PAGEPROTECT))
   {
      if (0 == strcmp (pszValue, JOBPROP_ON))
      {
         fPageProtect_d = TSTATE_ON;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_OFF))
      {
         fPageProtect_d = TSTATE_OFF;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_NONE))
      {
         fPageProtect_d = TSTATE_NONE;
         return true;
      }
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_JAMRECOVERY))
   {
      if (0 == strcmp (pszValue, JOBPROP_ON))
      {
         fJamRecovery_d = TSTATE_ON;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_OFF))
      {
         fJamRecovery_d = TSTATE_OFF;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_NONE))
      {
         fJamRecovery_d = TSTATE_NONE;
         return true;
      }
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_STAPLE))
   {
      if (0 == strcmp (pszValue, JOBPROP_ON))
      {
         fStaple_d = TSTATE_ON;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_OFF))
      {
         fStaple_d = TSTATE_OFF;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_NONE))
      {
         fStaple_d = TSTATE_NONE;
         return true;
      }
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_OFFSET))
   {
      if (0 == strcmp (pszValue, JOBPROP_ON))
      {
         fOffset_d = TSTATE_ON;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_OFF))
      {
         fOffset_d = TSTATE_OFF;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_NONE))
      {
         fOffset_d = TSTATE_NONE;
         return true;
      }
   }

   return false;
}

string * IBM_Pages_Instance::
getJobPropertyType (char *pszKey)
{
   TRISTATE state = TSTATE_NONE;

   if (0 == strcasecmp (pszKey, JOBPROP_RET))
   {
      state = fRET_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_ECONOMODE))
   {
      state = fEconoMode_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_PAGEPROTECT))
   {
      state = fPageProtect_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_JAMRECOVERY))
   {
      state = fJamRecovery_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_STAPLE))
   {
      state = fStaple_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_OFFSET))
   {
      state = fOffset_d;
   }
   else
   {
      return 0;
   }

   switch (fOffset_d)
   {
   case TSTATE_NONE: return new string ("string none");
   case TSTATE_ON:   return new string ("string on");
   case TSTATE_OFF:  return new string ("string off");
   default:          return 0;
   }

   return 0;
}

string * IBM_Pages_Instance::
getJobProperty (char *pszKey)
{
   TRISTATE state = TSTATE_NONE;

   if (0 == strcasecmp (pszKey, JOBPROP_RET))
   {
      state = fRET_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_ECONOMODE))
   {
      state = fEconoMode_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_PAGEPROTECT))
   {
      state = fPageProtect_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_JAMRECOVERY))
   {
      state = fJamRecovery_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_STAPLE))
   {
      state = fStaple_d;
   }
   else if (strcasecmp (pszKey, JOBPROP_OFFSET))
   {
      state = fOffset_d;
   }
   else
   {
      return 0;
   }

   switch (fOffset_d)
   {
   case TSTATE_NONE: return new string (JOBPROP_NONE);
   case TSTATE_ON:   return new string (JOBPROP_ON);
   case TSTATE_OFF:  return new string (JOBPROP_OFF);
   default:          return 0;
   }

   return 0;
}

string * IBM_Pages_Instance::
translateKeyValue (char *pszKey,
                   char *pszValue)
{
   string *pRet        = 0;
   char   *pszRetKey   = 0;
   char   *pszRetValue = 0;
   int     iKeyEnum    = StringResource::DEVICE_COMMON_UNKNOWN;
   int     iValueEnum  = StringResource::DEVICE_COMMON_UNKNOWN;

   if (0 == strcasecmp (pszKey, JOBPROP_RET))
   {
      iKeyEnum = StringResource::DEVICE_COMMON_RET;
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_ECONOMODE))
   {
      iKeyEnum = StringResource::DEVICE_COMMON_ECONO_MODE;
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_PAGEPROTECT))
   {
      iKeyEnum = StringResource::DEVICE_COMMON_PAGE_PROTECT;
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_JAMRECOVERY))
   {
      iKeyEnum = StringResource::DEVICE_COMMON_JAM_RECOVERY;
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_STAPLE))
   {
      iKeyEnum = StringResource::DEVICE_COMMON_STAPLE;
   }
   else if (0 == strcasecmp (pszKey, JOBPROP_OFFSET))
   {
      iKeyEnum = StringResource::DEVICE_COMMON_OFFSET;
   }

   if (pszValue)
   {
      if (0 == strcasecmp (pszValue, JOBPROP_ON))
      {
         iValueEnum = StringResource::DEVICE_COMMON_ON;
      }
      else if (0 == strcasecmp (pszValue, JOBPROP_ON))
      {
         iValueEnum = StringResource::DEVICE_COMMON_OFF;
      }
      else if (0 == strcasecmp (pszValue, JOBPROP_NONE))
      {
         iValueEnum = StringResource::DEVICE_COMMON_NONE;
      }
   }

   if (StringResource::DEVICE_COMMON_UNKNOWN != iKeyEnum)
   {
      pszRetKey = StringResource::getString (pDevice_d->getLanguageResource (),
                                             StringResource::STRINGGROUP_DEVICE_COMMON,
                                             iKeyEnum);
   }
   if (StringResource::DEVICE_COMMON_UNKNOWN != iValueEnum)
   {
      pszRetValue = StringResource::getString (pDevice_d->getLanguageResource (),
                                               StringResource::STRINGGROUP_DEVICE_COMMON,
                                               iValueEnum);
   }

   if (pszRetKey)
   {
      pRet = new string (pszRetKey);
   }
   if (  pszRetValue
      && pRet
      )
   {
      *pRet += "=";
      *pRet += pszRetValue;
   }

   return pRet;
}

void IBM_Pages_Instance::
setupPrinter ()
{
   if (fHaveSetupPrinter_d)
      return;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::setupPrinter ()" << endl;
#endif

   fHaveSetupPrinter_d = true;

///DeviceResolution *pRes      = getCurrentResolution ();
///DeviceForm       *pDF       = getCurrentForm ();
///HardCopyCap      *pHCC      = pDF->getHardCopyCap ();
///DeviceCommand    *pCommands = getCommands ();
///BinaryData       *pCmd      = 0;
}

bool IBM_Pages_Instance::
beginJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::beginJob ()" << endl;
#endif

   if (hasDeviceOption (SUPPORTS_PJL))
   {
      pCmd = pCommands->getCommandData ("cmdPJLSignature");
      if (pCmd)
      {
#ifndef RETAIL
         if (DebugOutput::shouldOutputInstance ()) cerr << "cmdPJLSignature = " << *pCmd << endl;
#endif

         pDevice_d->sendBinaryDataToDevice (pCmd);
      }

      pCmd = pCommands->getCommandData ("cmdBeginJob");
      if (pCmd)
      {
#ifndef RETAIL
         if (DebugOutput::shouldOutputInstance ()) cerr << "cmdBeginJob = " << *pCmd << endl;
#endif

         pDevice_d->sendBinaryDataToDevice (pCmd);
      }

      // @TBD Job prop stuff

      pCmd = pCommands->getCommandData ("cmdEnterLanguage");
      if (pCmd)
      {
#ifndef RETAIL
         if (DebugOutput::shouldOutputInstance ()) cerr << "cmdEnterLanguage = " << *pCmd << endl;
#endif

         pDevice_d->sendBinaryDataToDevice (pCmd);
      }

      // @TBD Job prop stuff
   }
   else if (hasDeviceOption (SUPPORTS_EMULATIONMODE))
   {
      pCmd = pCommands->getCommandData ("cmdPagesMode");
      if (pCmd)
      {
#ifndef RETAIL
         if (DebugOutput::shouldOutputInstance ()) cerr << "cmdPagesMode = " << *pCmd << endl;
#endif

         pDevice_d->sendBinaryDataToDevice (pCmd);
      }
   }

   pCmd = pCommands->getCommandData ("cmdInit");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdInit = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   // @TBD hardware copies

   DeviceOrientation *pDO = getCurrentOrientation ();

   if (pDO->isID (DeviceOrientation::ORIENTATION_PORTRAIT))
   {
      pCmd = pCommands->getCommandData ("cmdPortrait");
      if (pCmd)
      {
#ifndef RETAIL
         if (DebugOutput::shouldOutputInstance ()) cerr << "cmdPortrait = " << *pCmd << endl;
#endif

         pDevice_d->sendBinaryDataToDevice (pCmd);
      }
   }
   else if (pDO->isID (DeviceOrientation::ORIENTATION_LANDSCAPE))
   {
      pCmd = pCommands->getCommandData ("cmdLandscape");
      if (pCmd)
      {
#ifndef RETAIL
         if (DebugOutput::shouldOutputInstance ()) cerr << "cmdLandscape = " << *pCmd << endl;
#endif

         pDevice_d->sendBinaryDataToDevice (pCmd);
      }
   }

   return true;
}

bool IBM_Pages_Instance::
beginJob (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::beginJob (with props)" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The job properties are as follows:" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The orientation is = " << getCurrentOrientation() << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "iDitherID = " << getCurrentDitherID () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The form is = " << getCurrentForm () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The tray is = " << getCurrentTray () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The media is = " << getCurrentMedia () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The resolution is = " << getCurrentResolution () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The print mode is = " << getCurrentPrintMode () << endl;
#endif

   // @TBD - reinitialize with new job properties

   // Call common code
   return beginJob ();
}

bool IBM_Pages_Instance::
newFrame ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::newFrame ()" << endl;
#endif

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdPageEject = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   ditherNewFrame ();

   ptlPrintHead_d.x = 0;
   ptlPrintHead_d.y = 0;

   return true;
}

bool IBM_Pages_Instance::
newFrame (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::newFrame (with props)" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The job properties are as follows:" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The orientation is = " << getCurrentOrientation() << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "iDitherID = " << getCurrentDitherID () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The form is = " << getCurrentForm () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The tray is = " << getCurrentTray () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The media is = " << getCurrentMedia () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The resolution is = " << getCurrentResolution () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The print mode is = " << getCurrentPrintMode () << endl;
#endif

   // @TBD - reinitialize with new job properties

   // Call common code
   return newFrame ();
}

bool IBM_Pages_Instance::
endJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::endJob ()" << endl;
#endif

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdPageEject = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdTerm");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdTerm = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool IBM_Pages_Instance::
abortJob ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "IBM_Pages_Instance::endJob ()" << endl;
#endif

   // @TBD
   return false;
}

string IBM_Pages_Instance::
toString (ostringstream& oss)
{
   oss << "{ "
       << DeviceInstance::toString (oss) // Add parent's output as well
       << " }";

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const IBM_Pages_Instance& const_self)
{
   IBM_Pages_Instance& self = const_cast<IBM_Pages_Instance&>(const_self);
   ostringstream       oss;

   os << self.toString (oss);

   return os;
}
