/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _IBM_Pages_Instance
#define _IBM_Pages_Instance

#include "defines.hpp"
#include "Device.hpp"

extern "C" {
   DeviceInstance *createInstance (PrintDevice    *pDevice);
   void            deleteInstance (DeviceInstance *pInstance);
};

class IBM_Pages_Instance : public DeviceInstance
{
public:
   enum {
      SUPPORTS_OTHER_SETTINGS = 0x00000001,
      SUPPORTS_PJL            = 0x00000002,
      SUPPORTS_EMULATIONMODE  = 0x00000004
   };

   typedef enum {
      TSTATE_NONE,
      TSTATE_ON,
      TSTATE_OFF
   } TRISTATE;

                      IBM_Pages_Instance        (PrintDevice              *pDevice);
   virtual           ~IBM_Pages_Instance        ();

   void               initializeInstance        ();

   Enumeration       *listDeviceJobPropertyKeys ();
   bool               handleDeviceJobProperty   (char                     *pszKey,
                                                 char                     *pszValue);
   string            *getJobPropertyType        (char                     *pszKey);
   string            *getJobProperty            (char                     *pszKey);
   string            *translateKeyValue         (char                     *pszKey,
                                                 char                     *pszValue);

   virtual bool       beginJob                  ();
   virtual bool       beginJob                  (bool                      fJobPropertiesChanged);
   virtual bool       newFrame                  ();
   virtual bool       newFrame                  (bool                      fJobPropertiesChanged);
   virtual bool       endJob                    ();
   virtual bool       abortJob                  ();

   virtual string     toString                  (ostringstream&            oss);
   friend ostream&    operator<<                (ostream&                  os,
                                                 const IBM_Pages_Instance& device);

   void               setupPrinter              ();

   POINTL             ptlPrintHead_d;

private:
   bool               fHaveInitialized_d;
   bool               fHaveSetupPrinter_d;

   TRISTATE           fRET_d;
   TRISTATE           fEconoMode_d;
   TRISTATE           fPageProtect_d;
   TRISTATE           fJamRecovery_d;
   TRISTATE           fStaple_d;
   TRISTATE           fOffset_d;
};

#endif
