/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <fstream>
#include <string>
#include <sys/stat.h>

#include <DebugOutput.hpp>

#include <libxml/parser.h>
#include <libxml/tree.h>

#include "XMLDevice.hpp"

void
copyFile (string& stringFromFileName,
          string& stringToFileName)
{
   const char *pszFromFileName = stringFromFileName.c_str ();
   const char *pszToFileName   = stringToFileName.c_str ();
   struct stat statFile;

   if (-1 == stat (pszToFileName, &statFile))
   {
      ifstream ifFileFrom (pszFromFileName, ios::in  | ios::binary);
      ofstream ofFileTo   (pszToFileName,   ios::out | ios::binary);

      // copy the file in one line o' code!
      ofFileTo << ifFileFrom.rdbuf ();

      // @TBD - need better error detection
      if (-1 == stat (pszToFileName, &statFile))
      {
         cerr << "Error: cp "
              << "\""
              << stringFromFileName
              << "\""
              << " "
              << "\""
              << stringToFileName
              << "\""
              << endl;
      }
      else
      {
         cout << "cp "
              << "\""
              << stringFromFileName
              << "\""
              << " "
              << "\""
              << stringToFileName
              << "\""
              << endl;
      }
   }
}

int
main (int argc, char *argv[])
{
   char    achLine[512]; // @TBD
   char   *pszTargetRoot    = "/opt/Omni/data/";
   string  stringTargetRoot;

   if (1 == argc)
   {
      cout << "Usage: " << argv[0] << "[-r root] directory [directory]*" << endl;
   }

   for (int i = 1; i < argc; i++)
   {
      if ('-' == argv[i][0])
      {
         switch (argv[i][1])
         {
         case 'r':
         case 'R':
            stringTargetRoot = argv[++i];

            if (  stringTargetRoot.length () > 0
               && stringTargetRoot[stringTargetRoot.length ()-1] != '/'
               )
               stringTargetRoot = stringTargetRoot + "/";

            pszTargetRoot = (char *)stringTargetRoot.c_str ();
            break;
         }
         continue;
      }

      string stringFileNameDeviceList ("../");

      stringFileNameDeviceList += argv[i];
      stringFileNameDeviceList += "/Device List";

      ifstream ifIn (stringFileNameDeviceList.c_str ());

      while (0 < ifIn.getline (achLine, sizeof (achLine)))
      {
         string      stringFileNameMasterXML;
         string      stringToFileNameXML;
         string      stringFromFileNameXML;
         xmlDocPtr   docDevice               = 0;
         xmlNodePtr  nodeRoot                = 0;
         xmlNodePtr  nodeElm                 = 0;
         bool        fSuccess                = false;

         stringFileNameMasterXML  = "../";
         stringFileNameMasterXML += argv[i];
         stringFileNameMasterXML += "/";
         stringFileNameMasterXML += achLine;
         stringFileNameMasterXML += ".xml";

         docDevice = xmlParseFile (stringFileNameMasterXML.c_str ());

         if (!docDevice)
            goto done;

         nodeRoot = xmlDocGetRootElement (docDevice);

         if (!nodeRoot)
            goto done;

         nodeElm = firstNode (nodeRoot);

         if (!nodeElm)
            goto done;

         if (0 != strcmp ((char *)nodeElm->name, "Device"))
            goto done;

         if (nodeElm)
            nodeElm = firstNode (nodeElm->children);

         while (nodeElm)
         {
            if (  0 == strcmp ((char *)nodeElm->name, "Has")
               || 0 == strcmp ((char *)nodeElm->name, "Uses")
               )
            {
               char *pszTargetXMLFile = 0;

               pszTargetXMLFile = (char *)xmlNodeListGetString (docDevice,
                                                                nodeElm->xmlChildrenNode,
                                                                1);

               if (pszTargetXMLFile)
               {
                  xmlDocPtr docFile = 0;

                  stringFromFileNameXML  = "../";
                  stringFromFileNameXML += argv[i];
                  stringFromFileNameXML += "/";
                  stringFromFileNameXML += pszTargetXMLFile;

                  docFile = xmlParseFile (stringFromFileNameXML.c_str ());

                  if (docFile)
                  {
                     stringToFileNameXML  = pszTargetRoot;
                     stringToFileNameXML += pszTargetXMLFile;

                     copyFile (stringFromFileNameXML, stringToFileNameXML);

                     xmlFreeDoc (docFile);
                  }

                  free (pszTargetXMLFile);
               }
            }

            nodeElm = nextNode (nodeElm);
         }

         fSuccess = true;

done:
         if (fSuccess)
         {
            stringToFileNameXML  = pszTargetRoot;
            stringToFileNameXML += achLine;
            stringToFileNameXML += ".xml";

            copyFile (stringFileNameMasterXML, stringToFileNameXML);
         }
         else
         {
            cout << "Error: " << stringFileNameMasterXML << endl;
         }

         if (docDevice)
         {
            xmlFreeDoc (docDevice);
         }
      }
   }

   return 0;
}
